@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can addresstribute it and/or modify
@@ -25,9 +25,10 @@
 #include "ntop_includes.h"
 
 class AddressResolution {
-  AddressTree localNetworks;
+  AddressList localNetworks;
+  int num_resolvers;
   u_int32_t num_resolved_addresses, num_resolved_fails;
-  pthread_t resolveThreadLoop;
+  pthread_t *resolveThreadLoop;
   Mutex m;
 
  public:
@@ -40,9 +41,10 @@ class AddressResolution {
   inline u_int8_t getNumLocalNetworks()       { return localNetworks.getNumAddresses();    };
   inline char *get_local_network(u_int8_t id) { return localNetworks.getAddressString(id); };
   bool setLocalNetworks(char *rule);
-  int16_t findAddress(int family, void *addr); /* if(rc > 0) networdId else notfound */
-  void setLocalNetwork(char *net){localNetworks.addAddress(net);};
+  int16_t findAddress(int family, void *addr, u_int8_t *network_mask_bits = NULL);
+  void setLocalNetwork(char *net)             { localNetworks.addAddresses(net);           };
   void getLocalNetworks(lua_State* vm);
+  inline void dump()                          { localNetworks.dump(); }
 };
 
 #endif /* _ADDRESS_RESOLUTION_H_ */
