@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can addresstribute it and/or modify
@@ -24,29 +24,37 @@
 
 #include "ntop_includes.h"
 
+typedef struct {
+  u_int8_t mac[6];
+  int16_t value;
+  UT_hash_handle hh; /* makes this structure hashable */
+} MacKey_t;
+
 class AddressTree {
-  u_int8_t numAddresses;
-  char *addressString[CONST_MAX_NUM_NETWORKS];
-  patricia_tree_t *ptree;
+ private:
+  u_int16_t numAddresses;
+  patricia_tree_t *ptree_v4, *ptree_v6;
+  MacKey_t *macs;
+  
+  patricia_tree_t* getPatricia(char* what);
   
  public:
   AddressTree();
+  AddressTree(const AddressTree &at);
   ~AddressTree();
 
-  u_int8_t getNumAddresses();
-  /*
-   Returns the id of the network added. A negative number is returned on error.
-   */
-  int16_t addAddress(char *_net);
-  bool addAddresses(char *net);
-  bool removeAddress(char *net);
-  int16_t findAddress(int family, void *addr); /* if(rc > 0) networdId else notfound */
+  void init();
+  void cleanup();
+
+  inline u_int16_t getNumAddresses() { return(numAddresses); }
+
+  inline patricia_tree_t* getTree(bool isV4) { return(isV4 ? ptree_v4 : ptree_v6); }
+  bool addAddress(char *_net, const int16_t user_data = -1);
+  bool addAddresses(char *net, const int16_t user_data = -1);
   void getAddresses(lua_State* vm);
-  inline char *getAddressString(u_int8_t id) { return((id < numAddresses) ? addressString[id] : NULL); };
+  int16_t findAddress(int family, void *addr, u_int8_t *network_mask_bits = NULL);
+  int16_t findMac(u_int8_t addr[]);
+  void dump();
 };
 
-extern patricia_node_t* ptree_add_rule(patricia_tree_t *ptree, char *line);
-extern patricia_node_t* ptree_match(patricia_tree_t *tree, int family, void *addr, int bits);
-extern void free_ptree_data(void *data);
-
 #endif /* _ADDRESS_TREE_H_ */
