@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,14 +34,16 @@
 
 class AlertCounter {
  private:
+  bool thresholdTrepassed;
   u_int32_t max_num_hits_sec; /**< Threshold above which we trigger an alert. */
   u_int32_t num_hits_since_first_alert; /**< Number of hits since the first one that contributed to generate an alert. */
-  u_int8_t over_threshold_duration_sec; /**< Consecutive duration of threshold trepassing before triggering an alert. */
+  u_int8_t over_threshold_duration_sec; /**< Consecutive duration of threshold trespassing before triggering an alert. */
   time_t time_last_hit; /**< Time of last hit received. */ 
   time_t time_last_alert_reported; /**< Time of last alert issued. */ 
-  time_t last_trepassed_threshold; /**< Time of last event that trepassed the threshold. */
-  u_int32_t num_trepassed_threshold; /**< Number of consecutives threshold trepassing. */
-  u_int32_t num_hits_rcvd_last_second; /**< Number of hits reported in the last second. */ 
+  time_t last_trespassed_threshold; /**< Time of last event that trespassed the threshold. */
+  u_int32_t num_trespassed_threshold; /**< Number of consecutives threshold trespassing. */
+  u_int32_t num_hits_rcvd_last_second; /**< Number of hits reported in the last second. */
+  u_int32_t last_trespassed_hits; /**< Number of hits during last threshold trespassing */
   
   void init();
 
@@ -50,8 +52,11 @@ class AlertCounter {
 	       u_int8_t _over_threshold_duration_sec);
   
   bool incHits(time_t when);
-  inline u_int32_t getCurrentHits() { return(num_hits_since_first_alert); };
+  inline u_int32_t getCurrentHits()          { return(num_hits_since_first_alert);  };
   inline u_int8_t getOverThresholdDuration() { return(over_threshold_duration_sec); };
+  inline bool isAboveThreshold(time_t when)  { return(thresholdTrepassed && (time_last_hit >= (when-1)) ); };
+  void resetThresholds(u_int32_t _max_num_hits_sec, u_int8_t _over_threshold_duration_sec);
+  void lua(lua_State* vm, const char *table_key);
 };
 
 #endif /* _ALERT_COUNTER_H_ */
