@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,8 @@
 
 #include "ntop_includes.h"
 
+#ifndef HAVE_NEDGE
+
 class Lua;
 
 typedef struct {
@@ -34,27 +36,34 @@ typedef struct {
 class CollectorInterface : public ParserInterface {
  private:
   void *context;
-  u_int32_t num_drops;
+  struct {
+    u_int32_t num_flows, num_events, num_counters, zmq_msg_drops, last_zmq_msg_id;
+  } recvStats;
+  bool is_collector;
   u_int8_t num_subscribers;
-  zmq_subscriber subscriber[CONST_MAX_NUM_ZMQ_SUBSCRIBERS];
+  zmq_subscriber subscriber[MAX_ZMQ_SUBSCRIBERS];
 
  public:
   CollectorInterface(const char *_endpoint);
   ~CollectorInterface();
 
   inline const char* get_type()         { return(CONST_INTERFACE_TYPE_ZMQ);      };
+  inline InterfaceType getIfType()      { return(interface_type_ZMQ); }
   inline bool is_ndpi_enabled()         { return(false);      };
   inline char* getEndpoint(u_int8_t id) { return((id < num_subscribers) ?
 						 subscriber[id].endpoint : (char*)""); };
-  inline void incrDrops(u_int32_t num)  { num_drops += num;   };
-  inline u_int getNumDroppedPackets()   { return(num_drops);  };
-  inline bool is_packet_interface()     { return(false);      };
+  inline bool isPacketInterface()       { return(false);      };
   void collect_flows();
 
+  virtual void purgeIdle(time_t when);
+
   void startPacketPolling();
   void shutdown();
   bool set_packet_filter(char *filter);
+  virtual void lua(lua_State* vm);
 };
 
+#endif /* HAVE_NEDGE */
+
 #endif /* _COLLECTOR_INTERFACE_H_ */
 
