@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2016 - ntop.org
+ * (C) 2016-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,8 +21,8 @@
 
 #if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
 
-#ifndef _DIVERT_INTEFACE_H_
-#define _DIVERT_INTEFACE_H_
+#ifndef _DIVERT_INTERFACE_H_
+#define _DIVERT_INTERFACE_H_
 
 #include "ntop_includes.h"
 
@@ -35,11 +35,12 @@ class DivertInterface : public NetworkInterface {
   ~DivertInterface();
 
   inline const char* get_type()                 { return(CONST_INTERFACE_TYPE_DIVERT); };
+  inline InterfaceType getIfType()              { return(interface_type_DIVERT);       };
   inline int get_fd()                           { return(sock);                        };
   void startPacketPolling();
 };
 
-#endif /* _DIVERT_INTEFACE_H_ */
+#endif /* _DIVERT_INTERFACE_H_ */
 
 #endif /* defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__) */
 
