@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,19 +24,26 @@
 
 #include "ntop_includes.h"
 
+#ifndef HAVE_NEDGE
+
 class DummyInterface : public ParserInterface {
+ private:
+  inline u_int32_t getNumDroppedPackets()   { return(0); };
+
  public:
   DummyInterface();
 
-  inline const char* get_type()         { return(CONST_INTERFACE_TYPE_DUMMY);      };
+  const char* get_type()                { return(CONST_INTERFACE_TYPE_DUMMY);      };
+  inline InterfaceType getIfType()      { return(interface_type_DUMMY);            };
   inline bool is_ndpi_enabled()         { return(false);  };
-  inline u_int getNumDroppedPackets()   { return(0);      };
-  inline bool is_packet_interface()     { return(false);  };
+  inline bool isPacketInterface()       { return(false);  };
 
   void forgeFlow(u_int iteration);
   void startPacketPolling();
   void shutdown();
 };
 
+#endif
+
 #endif /* _DUMMY_INTERFACE_H_ */
 
