@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -47,54 +47,84 @@ typedef enum {
   flow_state_fin,
 } FlowState;
 
+typedef enum {
+  SSL_STAGE_UNKNOWN = 0,
+  SSL_STAGE_HELLO,
+  SSL_STAGE_CCS
+} FlowSSLStage;
+
+typedef enum {
+  SSL_ENCRYPTION_PLAIN = 0x0,
+  SSL_ENCRYPTION_SERVER = 0x1,
+  SSL_ENCRYPTION_CLIENT = 0x2,
+  SSL_ENCRYPTION_BOTH = 0x3,
+} FlowSSLEncryptionStatus;
+
 class Flow : public GenericHashEntry {
  private:
   Host *cli_host, *srv_host;
   u_int16_t cli_port, srv_port;
   u_int16_t vlanId;
   FlowState state;
+  u_int32_t vrfId;
   u_int8_t protocol, src2dst_tcp_flags, dst2src_tcp_flags;
   struct ndpi_flow_struct *ndpiFlow;
   bool detection_completed, protocol_processed, blacklist_alarm_emitted,
     cli2srv_direction, twh_over, dissect_next_http_packet, passVerdict,
-    check_tor, l7_protocol_guessed,
-    good_low_flow_detected;
+    check_tor, l7_protocol_guessed, flow_alerted, flow_dropped_counts_increased,
+    good_low_flow_detected, good_ssl_hs,
+    quota_exceeded, cli_quota_app_proto, cli_quota_is_category, srv_quota_app_proto, srv_quota_is_category;
   u_int16_t diff_num_http_requests;
 #ifdef NTOPNG_PRO
+  bool ingress2egress_direction;
+  u_int8_t routing_table_id;
+#ifndef HAVE_NEDGE
   FlowProfile *trafficProfile;
+#endif
   CounterTrend throughputTrend, goodputTrend, thptRatioTrend;
 #endif
   ndpi_protocol ndpiDetectedProtocol;
   void *cli_id, *srv_id;
-  char *json_info, *host_server_name, *ndpi_proto_name, *bt_hash;
-  bool dump_flow_traffic, badFlow;
+  char *json_info, *host_server_name, *bt_hash;
+  bool dump_flow_traffic;
 
   union {
     struct {
       char *last_url, *last_method;
+      char *last_content_type;
       u_int16_t last_return_code;
     } http;
-    
+
     struct {
       char *last_query;
+      bool invalid_query;
     } dns;
-    
+
     struct {
-      char *certificate;
-      bool hs_started, hs_over, firstdata_seen;
+      char *client_signature, *server_signature;
+    } ssh;
+
+    struct {
+      char *certificate, *server_certificate;
+      FlowSSLStage cli_stage, srv_stage;
+      u_int8_t hs_packets;
+      bool is_data;
+      /* firstdata refers to the time where encryption is active on both ends */
+      bool firstdata_seen;
       struct timeval clienthello_time, hs_end_time, lastdata_time;
       float hs_delta_time, delta_firstData, deltaTime_data;
     } ssl;
 
     struct {
       u_int8_t icmp_type, icmp_code;
+      u_int16_t icmp_echo_id;
     } icmp;
   } protos;
 
   struct {
-    struct site_categories category;
-    bool categorized_requested;
-  } categorization;
+    u_int32_t device_ip;
+    u_int16_t in_index, out_index;
+  } flow_device;
 
   /* Process Information */
   ProcessInfo *client_proc, *server_proc;
@@ -130,6 +160,17 @@ class Flow : public GenericHashEntry {
     u_int32_t last_dump;
   } last_db_dump;
 
+#ifdef NTOPNG_PRO
+  struct {
+    struct {
+      TrafficShaper *ingress, *egress;
+    } cli2srv;
+
+    struct {
+      TrafficShaper *ingress, *egress;
+    } srv2cli;
+  } flowShaperIds;
+#endif
   struct timeval last_update_time;
 
   float bytes_thpt, goodput_bytes_thpt, top_bytes_thpt, top_goodput_bytes_thpt, top_pkts_thpt;
@@ -150,52 +191,66 @@ class Flow : public GenericHashEntry {
   void processLua(lua_State* vm, ProcessInfo *proc, bool client);
   void processJson(bool is_src, json_object *my_object, ProcessInfo *proc);
   void checkBlacklistedFlow();
-  void allocFlowMemory();
+  void allocDPIMemory();
   bool checkTor(char *hostname);
-  void checkFlowCategory();
   void setBittorrentHash(char *hash);
   bool isLowGoodput();
   void updatePacketStats(InterarrivalStats *stats, const struct timeval *when);
   void dumpPacketStats(lua_State* vm, bool cli2srv_direction);
   inline u_int32_t getCurrentInterArrivalTime(time_t now, bool cli2srv_direction) {
-    return(1000 /* msec */ 
+    return(1000 /* msec */
 	   * (now - (cli2srv_direction ? cli2srvStats.pktTime.lastTime.tv_sec : srv2cliStats.pktTime.lastTime.tv_sec)));
   }
-  void dissectSSL(u_int8_t *payload, u_int16_t payload_len, const struct bpf_timeval *when);
-  FlowStatus getFlowStatus();
   char* printTCPflags(u_int8_t flags, char *buf, u_int buf_len);
-
-  inline bool isProtoSSL(u_int16_t p ) { return((ndpi_get_lower_proto(ndpiDetectedProtocol) == p) ? true : false); }
-  inline bool isSSL()                  { return(isProtoSSL(NDPI_PROTOCOL_SSL));  }
-  inline bool isDNS()                  { return(isProtoSSL(NDPI_PROTOCOL_DNS));  }
-  inline bool isHTTP()                 { return(isProtoSSL(NDPI_PROTOCOL_HTTP)); }
-  inline bool isICMP()                 { return(isProtoSSL(NDPI_PROTOCOL_IP_ICMP) || isProtoSSL(NDPI_PROTOCOL_IP_ICMPV6)); }
+  inline bool isProto(u_int16_t p ) { return((ndpi_get_lower_proto(ndpiDetectedProtocol) == p) ? true : false); }
+#ifdef NTOPNG_PRO
+  bool updateDirectionShapers(bool src2dst_direction, TrafficShaper **ingress_shaper, TrafficShaper **egress_shaper);
+#endif
+  void dumpFlowAlert();
 
  public:
   Flow(NetworkInterface *_iface,
        u_int16_t _vlanId, u_int8_t _protocol,
-       u_int8_t cli_mac[6], IpAddress *_cli_ip, u_int16_t _cli_port,
-       u_int8_t srv_mac[6], IpAddress *_srv_ip, u_int16_t _srv_port,
+       Mac *_cli_mac, IpAddress *_cli_ip, u_int16_t _cli_port,
+       Mac *_srv_mac, IpAddress *_srv_ip, u_int16_t _srv_port,
        time_t _first_seen, time_t _last_seen);
   ~Flow();
 
+  FlowStatus getFlowStatus();
   struct site_categories* getFlowCategory(bool force_categorization);
-  void categorizeFlow();
-  void deleteFlowMemory();
-  char* serialize(bool partial_dump = false, bool es_json = false);
-  json_object* flow2json(bool partial_dump);
+  void freeDPIMemory();
+  bool isTiny();
+  inline bool isSSL()                  { return(isProto(NDPI_PROTOCOL_SSL));  }
+  inline bool isSSH()                  { return(isProto(NDPI_PROTOCOL_SSH));  }
+  inline bool isDNS()                  { return(isProto(NDPI_PROTOCOL_DNS));  }
+  inline bool isDHCP()                 { return(isProto(NDPI_PROTOCOL_DHCP)); }
+  inline bool isHTTP()                 { return(isProto(NDPI_PROTOCOL_HTTP)); }
+  inline bool isICMP()                 { return(isProto(NDPI_PROTOCOL_IP_ICMP) || isProto(NDPI_PROTOCOL_IP_ICMPV6)); }
+  inline bool isMaskedFlow() {
+    return(!get_cli_host() || Utils::maskHost(get_cli_host()->isLocalHost())
+	   || !get_srv_host() || Utils::maskHost(get_srv_host()->isLocalHost()));
+  };
+  char* serialize(bool es_json = false);
+  json_object* flow2json();
   json_object* flow2es(json_object *flow_object);
-  inline u_int8_t getTcpFlags() { return(src2dst_tcp_flags | dst2src_tcp_flags);  };
-  bool isPassVerdict();
-  void setDropVerdict()         { passVerdict = false; };
+  inline u_int8_t getTcpFlags()        { return(src2dst_tcp_flags | dst2src_tcp_flags);  };
+  inline u_int8_t getTcpFlagsCli2Srv() { return(src2dst_tcp_flags);                      };
+  inline u_int8_t getTcpFlagsSrv2Cli() { return(dst2src_tcp_flags);                      };
+#ifdef NTOPNG_PRO
+  bool checkPassVerdict(const struct tm *now);
+  inline bool isPassVerdict()            { return passVerdict;  };
+#endif
+  inline void setDropVerdict()           { passVerdict = false; };
+  void incFlowDroppedCounters();
+
   u_int32_t getPid(bool client);
   u_int32_t getFatherPid(bool client);
   char* get_username(bool client);
   char* get_proc_name(bool client);
   u_int32_t getNextTcpSeq(u_int8_t tcpFlags, u_int32_t tcpSeqNum, u_int32_t payloadLen) ;
-  void makeVerdict(bool reset);
   double toMs(const struct timeval *t);
   void timeval_diff(struct timeval *begin, const struct timeval *end, struct timeval *result, u_short divide_by_two);
+  char* getFlowInfo();
   inline char* getFlowServerInfo() {
     return (isSSL() && protos.ssl.certificate) ? protos.ssl.certificate : host_server_name;
   }
@@ -204,6 +259,8 @@ class Flow : public GenericHashEntry {
   inline void  setServerName(char *v)  { if(host_server_name) free(host_server_name);  host_server_name = strdup(v); }
   void updateTcpFlags(const struct bpf_timeval *when,
 		      u_int8_t flags, bool src2dst_direction);
+  void incTcpBadStats(bool src2dst_direction,
+          u_int32_t ooo_pkts, u_int32_t retr_pkts, u_int32_t lost_pkts);
 
   void updateTcpSeqNum(const struct bpf_timeval *when,
 		       u_int32_t seq_num, u_int32_t ack_seq_num,
@@ -212,12 +269,12 @@ class Flow : public GenericHashEntry {
 
   void updateSeqNum(time_t when, u_int32_t sN, u_int32_t aN);
   void processDetectedProtocol();
-  void setDetectedProtocol(ndpi_protocol proto_id);
+  void setDetectedProtocol(ndpi_protocol proto_id, bool forceDetection);
   void setJSONInfo(const char *json);
   bool isFlowPeer(char *numIP, u_int16_t vlanId);
   void incStats(bool cli2srv_direction, u_int pkt_len,
-		u_int payload_len, const struct bpf_timeval *when);
-  void updateActivities();
+		u_int8_t *payload, u_int payload_len, u_int8_t l4_proto,
+		const struct timeval *when);
   void addFlowStats(bool cli2srv_direction, u_int in_pkts, u_int in_bytes, u_int in_goodput_bytes,
 		    u_int out_pkts, u_int out_bytes, u_int out_goodput_bytes, time_t last_seen);
   inline bool isDetectionCompleted()              { return(detection_completed);             };
@@ -226,6 +283,8 @@ class Flow : public GenericHashEntry {
   inline void* get_srv_id()                       { return(srv_id);                          };
   inline u_int32_t get_cli_ipv4()                 { return(cli_host->get_ip()->get_ipv4());  };
   inline u_int32_t get_srv_ipv4()                 { return(srv_host->get_ip()->get_ipv4());  };
+  inline struct ndpi_in6_addr* get_cli_ipv6()     { return(cli_host->get_ip()->get_ipv6());  };
+  inline struct ndpi_in6_addr* get_srv_ipv6()     { return(srv_host->get_ip()->get_ipv6());  };
   inline u_int16_t get_cli_port()                 { return(ntohs(cli_port));                 };
   inline u_int16_t get_srv_port()                 { return(ntohs(srv_port));                 };
   inline u_int16_t get_vlan_id()                  { return(vlanId);                          };
@@ -235,9 +294,13 @@ class Flow : public GenericHashEntry {
   inline u_int64_t get_bytes_srv2cli()            { return(srv2cli_bytes);                   };
   inline u_int64_t get_goodput_bytes()            { return(cli2srv_goodput_bytes+srv2cli_goodput_bytes);     };
   inline u_int64_t get_packets()                  { return(cli2srv_packets+srv2cli_packets); };
+  inline u_int64_t get_packets_cli2srv()          { return(cli2srv_packets);                 };
+  inline u_int64_t get_packets_srv2cli()          { return(srv2cli_packets);                 };
   inline u_int64_t get_partial_bytes()            { return(get_bytes() - (last_db_dump.cli2srv_bytes+last_db_dump.srv2cli_bytes));       };
   inline u_int64_t get_partial_bytes_cli2srv()    { return(cli2srv_bytes - last_db_dump.cli2srv_bytes);       };
   inline u_int64_t get_partial_bytes_srv2cli()    { return(srv2cli_bytes - last_db_dump.srv2cli_bytes);       };
+  inline u_int64_t get_partial_packets_cli2srv()  { return(cli2srv_packets - last_db_dump.cli2srv_packets);   };
+  inline u_int64_t get_partial_packets_srv2cli()  { return(srv2cli_packets - last_db_dump.srv2cli_packets);   };
   inline u_int64_t get_partial_goodput_bytes()    { return(get_goodput_bytes() - (last_db_dump.cli2srv_goodput_bytes+last_db_dump.srv2cli_goodput_bytes));       };
   inline u_int64_t get_partial_packets()          { return(get_packets() - (last_db_dump.cli2srv_packets+last_db_dump.srv2cli_packets)); };
   inline float get_bytes_thpt()                   { return(bytes_thpt);                      };
@@ -248,14 +311,19 @@ class Flow : public GenericHashEntry {
   inline u_int32_t get_duration()                 { return((u_int32_t)(get_last_seen()-get_first_seen())); };
   inline char* get_protocol_name()                { return(Utils::l4proto2name(protocol));   };
   inline ndpi_protocol get_detected_protocol()    { return(ndpiDetectedProtocol);          };
+  void fixAggregatedFlowFields();
+  inline ndpi_protocol_category_t get_detected_protocol_category() { return ndpi_get_proto_category(iface->get_ndpi_struct(), ndpiDetectedProtocol); };
   inline Host* get_cli_host()                     { return(cli_host);                        };
   inline Host* get_srv_host()                     { return(srv_host);                        };
   inline char* get_json_info()			  { return(json_info);                       };
-  inline ndpi_protocol_breed_t get_protocol_breed() { return(ndpi_get_proto_breed(iface->get_ndpi_struct(), ndpiDetectedProtocol.protocol)); };
+  inline ndpi_protocol_breed_t get_protocol_breed() { return(ndpi_get_proto_breed(iface->get_ndpi_struct(), ndpiDetectedProtocol.app_protocol)); };
   inline char* get_protocol_breed_name()            { return(ndpi_get_proto_breed_name(iface->get_ndpi_struct(),
 										       ndpi_get_proto_breed(iface->get_ndpi_struct(),
-													    ndpiDetectedProtocol.protocol))); };
-  char* get_detected_protocol_name();
+													    ndpiDetectedProtocol.app_protocol))); };
+  char* get_detected_protocol_name(char *buf, u_int buf_len) {
+    return(ndpi_protocol2name(iface->get_ndpi_struct(), ndpiDetectedProtocol, buf, buf_len));
+  }
+
   u_int32_t get_packetsLost();
   u_int32_t get_packetsRetr();
   u_int32_t get_packetsOOO();
@@ -267,43 +335,71 @@ class Flow : public GenericHashEntry {
   u_int64_t get_current_packets_cli2srv();
   u_int64_t get_current_packets_srv2cli();
   void handle_process(ProcessInfo *pinfo, bool client_process);
-  bool idle();
-  int compare(Flow *fb);
+  inline bool idle() { return(false); } /* Idle flows are checked in Flow::update_hosts_stats */
+  bool isReadyToPurge();
   inline bool is_l7_protocol_guessed() { return(l7_protocol_guessed); };
   char* print(char *buf, u_int buf_len);
   void update_hosts_stats(struct timeval *tv);
-  void print_peers(lua_State* vm, patricia_tree_t * ptree, bool verbose);
+#ifdef NTOPNG_PRO
+  void update_pools_stats(const struct timeval *tv,
+			  u_int64_t diff_sent_packets, u_int64_t diff_sent_bytes,
+			  u_int64_t diff_rcvd_packets, u_int64_t diff_rcvd_bytes);
+#endif
   u_int32_t key();
-  void lua(lua_State* vm, patricia_tree_t * ptree, bool detailed_dump, bool asListElement);
+  static u_int32_t key(Host *cli, u_int16_t cli_port,
+		       Host *srv, u_int16_t srv_port,
+		       u_int16_t vlan_id,
+		       u_int16_t protocol);
+  void lua(lua_State* vm, AddressTree * ptree, DetailsLevel details_level, bool asListElement);
   bool equal(IpAddress *_cli_ip, IpAddress *_srv_ip,
 	     u_int16_t _cli_port, u_int16_t _srv_port,
 	     u_int16_t _vlanId, u_int8_t _protocol,
 	     bool *src2srv_direction);
   void sumStats(nDPIStats *stats);
   void guessProtocol();
-  bool dumpFlow(bool partial_dump);
+  bool dumpFlow(bool idle_flow);
   bool dumpFlowTraffic(void);
-  bool match(patricia_tree_t *ptree);
+  bool match(AddressTree *ptree);
   inline Host* get_real_client() { return(cli2srv_direction ? cli_host : srv_host); }
   inline Host* get_real_server() { return(cli2srv_direction ? srv_host : cli_host); }
-  inline bool isBadFlow()        { return(badFlow); }
   inline bool isSuspiciousFlowThpt();
+  void dissectSSL(u_int8_t *payload, u_int16_t payload_len, const struct bpf_timeval *when, bool cli2srv);
   void dissectHTTP(bool src2dst_direction, char *payload, u_int16_t payload_len);
+  void dissectSSDP(bool src2dst_direction, char *payload, u_int16_t payload_len);
+  void dissectMDNS(u_int8_t *payload, u_int16_t payload_len);
   void dissectBittorrent(char *payload, u_int16_t payload_len);
   void updateInterfaceLocalStats(bool src2dst_direction, u_int num_pkts, u_int pkt_len);
-
-  inline void  setICMP(u_int8_t icmp_type, u_int8_t icmp_code) { if(isICMP()) { protos.icmp.icmp_type = icmp_type, protos.icmp.icmp_code = icmp_code; } }
+  inline void setICMP(bool src2dst_direction, u_int8_t icmp_type, u_int8_t icmp_code, u_int8_t *icmpdata) {
+    if(isICMP()) {
+      protos.icmp.icmp_type = icmp_type, protos.icmp.icmp_code = icmp_code;
+      if(get_cli_host()) get_cli_host()->incICMP(icmp_type, icmp_code, src2dst_direction ? true : false, get_srv_host());
+      if(get_srv_host()) get_srv_host()->incICMP(icmp_type, icmp_code, src2dst_direction ? false : true, get_cli_host());
+      protos.icmp.icmp_echo_id = ntohs(*((u_int16_t*)&icmpdata[4]));
+    }
+  }
+  inline void getICMP(u_int8_t *_icmp_type, u_int8_t *_icmp_code, u_int16_t *_icmp_echo_id) {
+    *_icmp_type = protos.icmp.icmp_type, *_icmp_code = protos.icmp.icmp_code, *_icmp_echo_id = protos.icmp.icmp_echo_id;
+  }
   inline char* getDNSQuery()        { return(isDNS() ? protos.dns.last_query : (char*)"");  }
   inline void  setDNSQuery(char *v) { if(isDNS()) { if(protos.dns.last_query) free(protos.dns.last_query);  protos.dns.last_query = strdup(v); } }
   inline char* getHTTPURL()         { return(isHTTP() ? protos.http.last_url : (char*)"");   }
   inline void  setHTTPURL(char *v)  { if(isHTTP()) { if(protos.http.last_url) free(protos.http.last_url);  protos.http.last_url = strdup(v); } }
+  inline char* getHTTPContentType() { return(isHTTP() ? protos.http.last_content_type : (char*)"");   }
   inline char* getSSLCertificate()  { return(isSSL() ? protos.ssl.certificate : (char*)""); }
-  void setDumpFlowTraffic(bool what)  { dump_flow_traffic = what; }
-  bool getDumpFlowTraffic(void)       { return dump_flow_traffic; }
-  void getFlowShapers(bool src2dst_direction, int *a_shaper_id, int *b_shaper_id, u_int16_t *ndpiProtocol);
+  bool isSSLProto();
+  inline bool isSSLData()              { return(isSSLProto() && good_ssl_hs && protos.ssl.is_data);  }
+  inline bool isSSLHandshake()         { return(isSSLProto() && good_ssl_hs && !protos.ssl.is_data); }
+  inline bool hasSSLHandshakeEnded()   { return(getSSLEncryptionStatus() == SSL_ENCRYPTION_BOTH);    }
+  FlowSSLEncryptionStatus getSSLEncryptionStatus();
+  void setDumpFlowTraffic(bool what)   { dump_flow_traffic = what; }
+  bool getDumpFlowTraffic(void)        { return dump_flow_traffic; }
 #ifdef NTOPNG_PRO
-  inline void updateProfile()   { trafficProfile = iface->getFlowProfile(this); }
+#ifndef HAVE_NEDGE
+  inline void updateProfile()     { trafficProfile = iface->getFlowProfile(this); }
   inline char* get_profile_name() { return(trafficProfile ? trafficProfile->getName() : (char*)"");}
+#endif
+  void updateFlowShapers();
+  void recheckQuota(const struct tm *now);
 #endif
   inline float getFlowRTT() { return(rttSec); }
   /* http://bradhedlund.com/2008/12/19/how-to-calculate-tcp-throughput-for-long-distance-links/ */
@@ -320,7 +416,42 @@ class Flow : public GenericHashEntry {
   inline u_int32_t getSrv2CliMaxInterArrivalTime()  { return(srv2cliStats.pktTime.max_ms); }
   inline u_int32_t getSrv2CliAvgInterArrivalTime()  { return((srv2cli_packets < 2) ? 0 : srv2cliStats.pktTime.total_delta_ms / (srv2cli_packets-1)); }
   bool isIdleFlow();
-  inline FlowState getFlowState()                   { return(state); }
+  inline FlowState getFlowState()                   { return(state);                          }
+  inline bool      isEstablished()                  { return state == flow_state_established; }
+  inline bool      isFlowAlerted()                  { return(flow_alerted);                   }
+  inline void      setFlowAlerted()                 { flow_alerted = true;                    }
+  inline void      setVRFid(u_int32_t v)            { vrfId = v;                              }
+
+  inline bool      setFlowDevice(u_int32_t device_ip, u_int16_t inidx, u_int16_t outidx) {
+    if((flow_device.device_ip > 0 && flow_device.device_ip != device_ip)
+       || (flow_device.in_index > 0 && flow_device.in_index != inidx)
+       || (flow_device.out_index > 0 && flow_device.out_index != outidx))
+      return false;
+    if(device_ip) flow_device.device_ip = device_ip;
+    if(inidx)     flow_device.in_index = inidx;
+    if(outidx)    flow_device.out_index = outidx;
+    return true;
+  }
+  inline u_int32_t getFlowDeviceIp()       { return flow_device.device_ip; };
+  inline u_int16_t getFlowDeviceInIndex()  { return flow_device.in_index;  };
+  inline u_int16_t getFlowDeviceOutIndex() { return flow_device.out_index; };
+  void setPacketsBytes(u_int32_t s2d_pkts, u_int32_t d2s_pkts, u_int32_t s2d_bytes, u_int32_t d2s_bytes);  
+
+#ifdef NTOPNG_PRO
+  void getFlowShapers(bool src2dst_direction, TrafficShaper **shaper_ingress, TrafficShaper **shaper_egress) {
+    if(src2dst_direction) {
+      *shaper_ingress = flowShaperIds.cli2srv.ingress,
+	*shaper_egress = flowShaperIds.cli2srv.egress;
+    } else {
+      *shaper_ingress = flowShaperIds.srv2cli.ingress,
+	*shaper_egress = flowShaperIds.srv2cli.egress;
+    }
+  }
+
+  inline u_int8_t getFlowRoutingTableId() { return(routing_table_id); }
+  inline void setIngress2EgressDirection(bool _ingress2egress) { ingress2egress_direction = _ingress2egress; }
+  inline bool isIngress2EgressDirection() { return(ingress2egress_direction); }
+#endif
 };
 
 #endif /* _FLOW_H_ */
