@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -45,7 +45,8 @@ class GenericHash {
   Mutex **locks, purgeLock;
   NetworkInterface *iface; /**< Pointer of network interface for this generic hash.*/
   u_int last_purged_hash; /**< Index of last purged hash.*/
-
+  u_int purge_step;
+  
  public:
   /**
    * @brief A Constructor
@@ -93,10 +94,13 @@ class GenericHash {
    * @brief generic walker for the hash.
    * @details This method uses the walker function to compare each elements of the hash with the user data.
    *
+   * @param begin_slot begin hash slot. Use 0 to walk all slots
+   * @param walk_all true = walk all hash, false, walk only one (non NULL) slot
    * @param walker A pointer to the comparison function.
    * @param user_data Value to be compared with the values of hash.
    */
-  void walk(bool (*walker)(GenericHashEntry *h, void *user_data), void *user_data);
+  bool walk(u_int32_t *begin_slot, bool walk_all,
+	    bool (*walker)(GenericHashEntry *h, void *user_data, bool *entryMatched), void *user_data);
 
   /**
    * @brief Purge idle hash entries.
@@ -134,8 +138,8 @@ class GenericHash {
   inline bool hasEmptyRoom() { return((current_size < max_hash_size) ? true : false); };
   inline u_int32_t getCurrentSize() { return current_size;}
 
-  inline void disablePurge() { purgeLock.lock(__FILE__, __LINE__);   }
-  inline void enablePurge()  { purgeLock.unlock(__FILE__, __LINE__); }
+  inline void disablePurge() { /* purgeLock.lock(__FILE__, __LINE__);   */ }
+  inline void enablePurge()  { /* purgeLock.unlock(__FILE__, __LINE__); */ }
 };
 
 #endif /* _GENERIC_HASH_H_ */
