@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,21 +29,21 @@ extern bool enable_users_login;
 
 class HTTPserver {
  private:
-  u_int16_t port;
   char *docs_dir, *scripts_dir;
   struct mg_context *httpd_v4;
   bool ssl_enabled;
-
+  u_int16_t http_splash_port;
+  
  public:
-  HTTPserver(u_int16_t _port, const char *_docs_dir, const char *_scripts_dir);
+  HTTPserver(const char *_docs_dir, const char *_scripts_dir);
   ~HTTPserver();
 
   bool valid_user_pwd(char *user, char *pass);
-
-  inline u_int16_t get_port()        { return(port);        };
-  inline char*     get_docs_dir()    { return(docs_dir);    };
-  inline char*     get_scripts_dir() { return(scripts_dir); };
-  inline bool      is_ssl_enabled()  { return(ssl_enabled); };
+  
+  inline char*     get_docs_dir()    { return(docs_dir);         };
+  inline char*     get_scripts_dir() { return(scripts_dir);      };
+  inline bool      is_ssl_enabled()  { return(ssl_enabled);      };
+  inline u_int16_t getSplashPort()   { return(http_splash_port); };
 };
 
 extern int send_error(struct mg_connection *conn, int status, const char *reason, const char *fmt, ...);
