@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,7 +32,7 @@ class HostHash : public GenericHash {
  public:
   HostHash(NetworkInterface *iface, u_int _num_hashes, u_int _max_hash_size);
 
-  Host* get(u_int16_t vlanId, const u_int8_t mac[6]);
+  /* Search for an host by IP and VLAN */
   Host* get(u_int16_t vlanId, IpAddress *key);
 
   void incNumHTTPEntries();  
