@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 
 #include "ntop_includes.h"
 
-#define MUTEX_DEBUG 1
+/* #define MUTEX_DEBUG 1 */
 
 /* ******************************* */
 
@@ -44,6 +44,9 @@ class Mutex {
   void lock(const char *filename, const int line);
   void unlock(const char *filename, const int line);  
   inline bool is_locked() { return(locked); };
+
+  /* NOTE: this must be called while locked */
+  inline int cond_wait(pthread_cond_t *condvar) { return pthread_cond_wait(condvar, &the_mutex); };
 };
 
 
