@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,7 +28,6 @@
  * Main ntopng group.
  */
 
-
 class NtopPro;
 
 /** @class Ntop
@@ -39,39 +38,50 @@ class NtopPro;
  */
 class Ntop {
  private:
-  patricia_tree_t *local_interface_addresses;
-  char working_dir[MAX_PATH]; /**< Array of working directory.*/
-  char install_dir[MAX_PATH]; /**< Array of install directory.*/
-  char startup_dir[MAX_PATH]; /**< Array of startup directory.*/
-  char *custom_ndpi_protos; /**< Pointer of a custom protocol for nDPI.*/
-  NetworkInterface *iface[MAX_NUM_DEFINED_INTERFACES];/**< Array of network interfaces.*/
-  NetworkInterfaceView *ifaceViews[MAX_NUM_DEFINED_INTERFACES];/**< Array of network interface views.*/
-  u_int8_t num_defined_interfaces; /**< Number of defined interfaces.*/
-  u_int8_t num_defined_interface_views; /**< Number of defined interface views.*/
-  HTTPserver *httpd; /**< Pointer of httpd server.*/
-  NtopGlobals *globals; /**< Pointer of Ntop globals info and variables.*/
+  AddressTree local_interface_addresses;
+  char epoch_buf[11];
+  char working_dir[MAX_PATH]; /**< Array of working directory. */
+  char install_dir[MAX_PATH]; /**< Array of install directory. */
+  char startup_dir[MAX_PATH]; /**< Array of startup directory. */
+  char *custom_ndpi_protos; /**< Pointer of a custom protocol for nDPI. */
+  NetworkInterface **iface; /**< Array of network interfaces. */
+  u_int8_t num_defined_interfaces; /**< Number of defined interfaces. */
+  HTTPserver *httpd; /**< Pointer of httpd server. */
+  NtopGlobals *globals; /**< Pointer of Ntop globals info and variables. */
   u_int num_cpus; /**< Number of physical CPU cores. */
-  Redis *redis; /**< Pointer of Radius server.*/
-  ElasticSearch *elastic_search; /**< Pointer of Elastic Search.*/
-  PeriodicActivities *pa; /**< Instance of periodical activities.*/
+  Redis *redis; /**< Pointer to the Redis server. */
+#ifndef HAVE_NEDGE
+  ElasticSearch *elastic_search; /**< Pointer of Elastic Search. */
+  Logstash *logstash; /**< Pointer of Logstash. */
+  ExportInterface *export_interface;
+#endif
+  PeriodicActivities *pa; /**< Instance of periodical activities. */
   AddressResolution *address;
   Prefs *prefs;
-  RuntimePrefs *runtimeprefs;
   Geolocation *geo;
+  MacManufacturers *mac_manufacturers;
+  void *trackers_automa;
   HTTPBL *httpbl;
-  Flashstart *flashstart;
-  ExportInterface *export_interface;
   long time_offset;
   time_t start_time; /**< Time when start() was called */
   int udp_socket;
   NtopPro *pro;
+  DeviceProtocolBitmask deviceProtocolPresets[device_max_type];
+  
 #ifdef NTOPNG_PRO
+#ifndef WIN32
   NagiosManager *nagios_manager;
+#endif
   FlowChecker *flow_checker;
 #endif
+#ifdef HAVE_NDB
+  Nseries *nseries[NUM_NSERIES];
+#endif
+  AddressTree *hostBlacklist, *hostBlacklistShadow;
 
   void loadLocalInterfaceAddress();
-
+  void initAllowedProtocolPresets();
+  
  public:
   /**
    * @brief A Constructor
@@ -95,6 +105,14 @@ class Ntop {
    */
   void registerPrefs(Prefs *_prefs, bool quick_registration);
 
+  /**
+   * @brief Register an ntopng log file
+   * @details Log file is used under windows and in daemon mode
+   *
+   * @param logFile A valid path to a log file
+   */
+  inline void registerLogFile(const char* logFile)    { getTrace()->set_log_file(logFile);       };
+  inline void rotateLogs(bool force_rotation = false) { getTrace()->rotate_logs(force_rotation); };
 #ifdef NTOPNG_PRO
   void registerNagios(void);
   inline FlowChecker *getFlowChecker() { return(flow_checker); };
@@ -142,6 +160,20 @@ class Ntop {
    * @param dir Path to database home directory.
    */
   void loadGeolocation(char *dir);
+  /**
+   * @brief Load the @ref MacManufacturers module.
+   * @details Initialize the variable @ref dir with the input directory.
+   *
+   * @param dir Path to database home directory.
+   */
+  void loadMacManufacturers(char *dir);
+
+  inline void getMacManufacturer(const char *mac, lua_State *vm) {
+    u_int8_t mac_bytes[6];
+    Utils::parseMac(mac_bytes, mac);
+    mac_manufacturers->getMacManufacturer(mac_bytes, vm);
+  }
+
   /**
    * @brief Set the local networks.
    * @details Set the local networks to @ref AddressResolution instance.
@@ -156,9 +188,10 @@ class Ntop {
    * @param family Internetwork: UDP, TCP, etc.
    * @param addr Internet Address.
    * @param network_id It returns the networkId to which the host belongs to
+   * @param network_mask_bits It returns the number of bits of the network mask
    * @return True if the address is in the local networks, false otherwise.
    */
-  bool isLocalAddress(int family, void *addr, int16_t *network_id);
+  bool isLocalAddress(int family, void *addr, int16_t *network_id, u_int8_t *network_mask_bits = NULL);
 
   /**
    * @brief Start ntopng packet processing.
@@ -180,7 +213,13 @@ class Ntop {
    *
    * @return Current geolocation instance.
    */
-  inline Geolocation* getGeolocation()               { return(geo);                        };
+  inline Geolocation* getGeolocation()               { return(geo);                };
+  /**
+   * @brief Get the mac manufacturers instance.
+   *
+   * @return Current mac manufacturers instance.
+   */
+  inline MacManufacturers* getMacManufacturers()     { return(mac_manufacturers); };
   /**
    * @brief Get the ifName.
    * @details Find the ifName by id parameter.
@@ -188,8 +227,8 @@ class Ntop {
    * @param id Index of ifName.
    * @return ....
    */
-  inline char* get_if_name(int id)              { return(prefs->get_if_name(id));     };
-  inline char* get_if_descr(int id)             { return(prefs->get_if_descr(id));    };
+  inline char* get_if_name(int id)                   { return(prefs->get_if_name(id));     };
+  inline char* get_if_descr(int id)                  { return(prefs->get_if_descr(id));    };
   inline char* get_data_dir()                        { return(prefs->get_data_dir());      };
   inline char* get_callbacks_dir()                   { return(prefs->get_callbacks_dir()); };
   /**
@@ -206,9 +245,6 @@ class Ntop {
    */
   inline HTTPBL* get_httpbl()                        { return(httpbl);             };
 
-  inline Flashstart* get_flashstart()                { return(flashstart);         };
-  inline void set_flashstart(Flashstart *f)          { flashstart = f;             };
-
   /**
    * @brief Register the network interface.
    * @details Check for duplicated interface and add the network interface in to @ref iface.
@@ -217,22 +253,13 @@ class Ntop {
    */
   void registerInterface(NetworkInterface *i);
 
-  /**
-   * @brief Register a network interface view.
-   * @details Check for duplicated interface views and add the network interface view in to @ref iface.
-   *
-   * @param v Network interface view.
-   */
-  void registerInterfaceView(NetworkInterfaceView *v);
-
   /**
    * @brief Get the number of defined network interfaces.
    *
    * @return Number of defined network interfaces.
    */
   inline u_int8_t get_num_interfaces()               { return(num_defined_interfaces); }
-  inline u_int8_t get_num_interface_views()               { return(num_defined_interface_views); }
-  int getInterfaceViewIdByName(char *name);
+
   /**
    * @brief Get the i-th network interface.
    * @details Retrieves the pointer the network interface
@@ -243,7 +270,7 @@ class Ntop {
    * @return The network interface instance if exists, NULL otherwise.
    */
   inline NetworkInterface* getInterfaceAtId(lua_State *vm, int i) const {
-    if(i<num_defined_interfaces && iface[i]) {
+    if((i < num_defined_interfaces) && iface[i]) {
       return isInterfaceAllowed(vm, iface[i]->get_name()) ? iface[i] : NULL;
     }
     return NULL;
@@ -260,33 +287,7 @@ class Ntop {
   inline NetworkInterface* getInterfaceAtId(int i) const {
     return getInterfaceAtId(NULL, i);
   }
-  /**
-   * @brief Get the i-th network interface.
-   * @details Retrieves the pointer the network interface view
-   *  identified by id i and enforces constraints on
-   *  user allowed interfaces.
-   *
-   * @param i The i-th network interface.
-   * @return The network interface instance if exists, NULL otherwise.
-   */
-  inline NetworkInterfaceView* getInterfaceViewAtId(lua_State *vm, int i) const {
-    if(i<num_defined_interface_views && ifaceViews[i]) {
-      return isInterfaceAllowed(vm, ifaceViews[i]->get_name()) ? ifaceViews[i] : NULL;
-    }
-    return NULL;
-  }
-  /**
-   * @brief Get the i-th network interface.
-   * @details Retrieves the pointer the network interface view
-   *  identified by id i WITHOUT ENFORCING constraints on
-   *  user allowed interfaces.
-   *
-   * @param i The i-th network interface.
-   * @return The network interface instance if exists, NULL otherwise.
-   */
-  inline NetworkInterfaceView* getInterfaceViewAtId(int i) const {
-    return getInterfaceViewAtId(NULL, i);
-  }
+
   /**
    * @brief Get the Id of network interface.
    * @details This method accepts both interface names or Ids.
@@ -295,6 +296,15 @@ class Ntop {
    * @return The network interface Id if exists, -1 otherwise.
    */
   int getInterfaceIdByName(char *name);
+
+  /**
+   * @brief Get the network interface with the specified Id
+   *
+   * @param if_id Id of network interface.
+   * @return Pointer to the network interface, NULL otherwise.
+   */
+  NetworkInterface* getInterfaceById(int if_id);
+
   /**
    * @brief Register the HTTP server.
    *
@@ -317,7 +327,7 @@ class Ntop {
    * @param name Names or Id of network interface.
    * @return The network interface instance if exists, NULL otherwise.
    */
-  NetworkInterface* getNetworkInterface(lua_State* vm, const char *name);
+  NetworkInterface* getNetworkInterface(lua_State *vm, const char *name);
   /**
    * @brief Get the network interface identified by name or Id.
    * @details This method accepts both interface names or Ids.
@@ -328,27 +338,21 @@ class Ntop {
    * @return The network interface instance if exists, NULL otherwise.
    */
   inline NetworkInterface* getNetworkInterface(const char *name) {
-    return getNetworkInterface(NULL, name);
+    return getNetworkInterface(NULL /* don't enforce the check on the allowed interface */,
+			       name);
+  };
+  inline NetworkInterface* getNetworkInterface(lua_State *vm, int ifid) {
+    char ifname[MAX_INTERFACE_NAME_LEN];
+    snprintf(ifname, sizeof(ifname), "%d", ifid);
+    return getNetworkInterface(vm /* enforce the check on the allowed interface */,
+			       ifname);
   };
   inline NetworkInterface* getNetworkInterface(int ifid) {
     char ifname[MAX_INTERFACE_NAME_LEN];
     snprintf(ifname, sizeof(ifname), "%d", ifid);
-    return getNetworkInterface(NULL, ifname);
+    return getNetworkInterface(NULL /* don't enforce the check on the allowed interface */,
+			       ifname);
   };
-  NetworkInterfaceView* getNetworkInterfaceView(lua_State* vm, const char *name);
-  inline NetworkInterfaceView* getNetworkInterfaceView(const char *name) {
-    return getNetworkInterfaceView(NULL, name);
-  };
-  inline NetworkInterfaceView* getNetworkInterfaceView(lua_State *vm, int ifid) {
-    char ifname[MAX_INTERFACE_NAME_LEN];
-    snprintf(ifname, sizeof(ifname), "%d", ifid);
-    return getNetworkInterfaceView(vm, ifname);
-  };
-  inline NetworkInterfaceView* getNetworkInterfaceView(int ifid) {
-    return getNetworkInterfaceView(NULL, ifid);
-  };
-
-  void sanitizeInterfaceView(NetworkInterfaceView *view);
 
   /**
    * @brief Get the current HTTPserver instance.
@@ -373,41 +377,64 @@ class Ntop {
   inline NtopGlobals*      getGlobals()              { return(globals); };
   inline Trace*            getTrace()                { return(globals->getTrace()); };
   inline Redis*            getRedis()                { return(redis);               };
+#ifndef HAVE_NEDGE
   inline ElasticSearch*    getElasticSearch()        { return(elastic_search);      };
-  inline Prefs*            getPrefs()                { return(prefs);               };
-  inline RuntimePrefs*     getRuntimePrefs()         { return(runtimeprefs);        };
+  inline Logstash*         getLogstash()             { return(logstash);            };
   inline ExportInterface*  get_export_interface()    { return(export_interface);    };
-
-#ifdef NTOPNG_PRO
-  inline NagiosManager*    getNagios()               { return(nagios_manager);      };
 #endif
 
-  void getUsers(lua_State* vm); 
-  void getUserGroup(lua_State* vm);  
+  inline Prefs*            getPrefs()                { return(prefs);               };
+  
+#ifdef NTOPNG_PRO
+#ifdef WIN32
+  char* getIfName(int if_id, char *name, u_int name_len);
+#else
+  inline NagiosManager*    getNagios()               { return(nagios_manager);      };
+#endif
+#endif
+
+  void getUsers(lua_State* vm);
+  void getUserGroup(lua_State* vm);
   void getAllowedNetworks(lua_State* vm);
   bool getInterfaceAllowed(lua_State* vm, char *ifname)      const;
   bool isInterfaceAllowed(lua_State* vm, const char *ifname) const;
   bool isInterfaceAllowed(lua_State* vm, int ifid)           const;
   bool checkUserPassword(const char *user, const char *password);
   bool resetUserPassword(char *username, char *old_password, char *new_password);
+  bool mustChangePassword(const char *user);
   bool changeUserRole(char *username, char *user_role) const;
   bool changeAllowedNets(char *username, char *allowed_nets)     const;
   bool changeAllowedIfname(char *username, char *allowed_ifname) const;
+  bool changeUserHostPool(const char * const username, const char * const host_pool_id) const;
+  bool changeUserLanguage(const char * const username, const char * const language) const;
   bool addUser(char *username, char *full_name, char *password, char *host_role,
-	       char *allowed_networks, char *allowed_ifname);
+	       char *allowed_networks, char *allowed_ifname, char *host_pool_id,
+	       char *language);
+  bool addUserLifetime(const char * const username, u_int32_t lifetime_secs); /* Captive portal users may expire */
+  bool clearUserLifetime(const char * const username);
+  bool isCaptivePortalUser(const char * const username);
   bool deleteUser(char *username);
+  bool getUserHostPool(char *username, u_int16_t *host_pool_id);
+  bool getUserAllowedIfname(char *username, char *buf, size_t buflen);
+  bool hasUserLimitedLifetime(const char * const username, int32_t *lifetime_secs);
   void setWorkingDir(char *dir);
   void fixPath(char *str, bool replaceDots = true);
   void removeTrailingSlash(char *str);
   void daemonize();
   void shutdown();
   void runHousekeepingTasks();
-  bool isLocalInterfaceAddress(int family, void *addr);
-  inline char* getLocalNetworkName(int16_t local_network_id) { return(address->get_local_network((u_int8_t)local_network_id)); };
+  bool isLocalInterfaceAddress(int family, void *addr)       { return(local_interface_addresses.findAddress(family, addr) == -1 ? false : true);    };
+  inline char* getLocalNetworkName(int16_t local_network_id) {
+    return(address->get_local_network((u_int8_t)local_network_id));
+  };
+  IpAddress* getLocalNetworkIp(int16_t local_network_id);
   void createExportInterface();
-  void initRedis();
+  void initNetworkInterfaces();
   void initElasticSearch();
+  void initLogstash(); 
 
+  inline u_int32_t getStarttime()        { return((u_int32_t)start_time); }
+  inline char*     getStarttimeString()  { return(epoch_buf);             }
   inline u_int32_t getUptime()          { return((u_int32_t)((start_time > 0) ? (time(NULL)-start_time) : 0)); }
   inline int getUdpSock()               { return(udp_socket); }
 
@@ -418,8 +445,38 @@ class Ntop {
 
   inline void getLocalNetworks(lua_State* vm) { address->getLocalNetworks(vm);          };
   inline u_int8_t getNumLocalNetworks()       { return(address->getNumLocalNetworks()); };
+  void reloadInterfacesLuaInterpreter();
+  void loadTrackers();
+  bool isATrackerHost(char *host);
+  void allocHostBlacklist();
+  void swapHostBlacklist();
+  void addToHostBlacklist(char *net);
+  bool isBlacklistedIP(IpAddress *ip);
+  bool isExistingInterface(char *name);
+  inline NetworkInterface* getFirstInterface() { return(iface[0]);         }
+  inline NetworkInterface* getInterface(int i) { return(((i < num_defined_interfaces) && iface[i]) ? iface[i] : NULL); }
+#ifdef NTOPNG_PRO
+  bool addIPToLRUMatches(u_int32_t client_ip, u_int16_t user_pool_id,
+			 char *label, int32_t lifetime_secs, char *ifname);
+#ifdef HAVE_NDB
+  inline void tsSet(u_int8_t series_id, u_int32_t tsSec, bool isCounter,
+		    u_int8_t ifaceId, u_int16_t step, const char *label,
+		    const char *key, const char *metric, u_int64_t sent, u_int64_t rcvd) {
+    nseries[series_id]->set(tsSec, isCounter, ifaceId, step, label ? label : "",
+			    key ? key : "", metric ? metric : "", sent, rcvd);
+  }
+  
+  inline void tsFlush(u_int8_t series_id) { nseries[series_id]->flush(); }
+#endif /* HAVE_NDB */
+#endif /* NTOPNG_PRO */
+  
+  DeviceProtocolBitmask* getDeviceAllowedProtocols(DeviceType t) { return(&deviceProtocolPresets[t]); }
 };
 
 extern Ntop *ntop;
 
+#ifdef NTOPNG_PRO
+#include "ntoppro_defines.h"
+#endif
+
 #endif /* _NTOP_CLASS_H_ */
