@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,13 +30,18 @@ class PF_RINGInterface : public NetworkInterface {
  private:
   pfring *pfring_handle;
 
+  pfring_stat last_pfring_stat;
+  u_int32_t getNumDroppedPackets();
+
  public:
   PF_RINGInterface(const char *name);
   ~PF_RINGInterface();
 
-  inline const char* get_type()      { return(CONST_INTERFACE_TYPE_PF_RING);     };
-  inline pfring* get_pfring_handle() { return(pfring_handle); };
-  u_int getNumDroppedPackets();
+  inline bool areTrafficDirectionsSupported()  { return(true); };
+  inline bool isDiscoverableInterface()        { return(true);                         };
+  inline InterfaceType getIfType()             { return(interface_type_PF_RING);       };
+  inline const char* get_type()                { return(CONST_INTERFACE_TYPE_PF_RING); };
+  inline pfring* get_pfring_handle()           { return(pfring_handle);                };
   void startPacketPolling();
   void shutdown();
   bool set_packet_filter(char *filter);
