@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,15 +30,26 @@ class PacketStats {
     upTo512, upTo1024, upTo1518,
     upTo2500, upTo6500, upTo9000,
     above9000;
+  u_int64_t syn, synack, finack, rst;
 
  public:
   PacketStats();
-  
-  void incStats(u_int pkt_len);  
+
+  void incFlagStats(u_int8_t flags);
+  void incStats(u_int pkt_len);
   char* serialize();
   void deserialize(json_object *o);
   json_object* getJSONObject();
   void lua(lua_State* vm, const char *label);
+  inline void sum(PacketStats *s) {
+    s->upTo64 += upTo64, s->upTo128 += upTo128,
+      s->upTo128 += upTo128, s->upTo256 += upTo256,
+      s->upTo512 += upTo512, s->upTo1024 += upTo1024,
+      s->upTo1518 += upTo1518, s->upTo2500 += upTo2500,
+      s->upTo6500 += upTo6500, s->upTo9000 += upTo9000,
+      s->above9000 += above9000,
+      s->syn += syn, s->synack += synack, s->finack += finack, s->rst += rst;
+  };
 };
 
 #endif /* _PACKETS_STATS_H_ */
