@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,16 +34,25 @@ class ParserInterface : public NetworkInterface {
  private:
   struct FlowFieldMap *map;
   bool once;
+  u_int64_t zmq_initial_bytes, zmq_initial_pkts,
+    zmq_remote_initial_exported_flows;
+  ZMQ_RemoteStats *zmq_remote_stats, *zmq_remote_stats_shadow;
 
   int getKeyId(char *sym);
   void addMapping(const char *sym, int num);
-
+  void parseSingleFlow(json_object *o, u_int8_t source_id, NetworkInterface *iface);
+    
  public:
-  ParserInterface(const char *endpoint);
+  ParserInterface(const char *endpoint, const char *custom_interface_type = NULL);
   ~ParserInterface();
 
   u_int8_t parseFlow(char *payload, int payload_size, u_int8_t source_id, void *data);
   u_int8_t parseEvent(char *payload, int payload_size, u_int8_t source_id, void *data);
+  u_int8_t parseCounter(char *payload, int payload_size, u_int8_t source_id, void *data);
+
+  virtual void setRemoteStats(ZMQ_RemoteStats *zrs);
+  u_int32_t getNumDroppedPackets() { return zmq_remote_stats ? zmq_remote_stats->sflow_pkt_sample_drops : 0; };
+  virtual void lua(lua_State* vm);
 };
 
 #endif /* _PARSER_INTERFACE_H_ */
