@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,17 +30,21 @@ class PcapInterface : public NetworkInterface {
   bool read_pkts_from_pcap_dump;
   FILE *pcap_list;
 
+  pcap_stat last_pcap_stat;
+  u_int32_t getNumDroppedPackets();
+
  public:
   PcapInterface(const char *name);
   ~PcapInterface();
 
+  inline bool isDiscoverableInterface(){ return(true); };
+  inline InterfaceType getIfType() { return(read_pkts_from_pcap_dump ? interface_type_PCAP_DUMP : interface_type_PCAP); }
   inline const char* get_type()    { return(read_pkts_from_pcap_dump ? CONST_INTERFACE_TYPE_PCAP_DUMP : CONST_INTERFACE_TYPE_PCAP); };
   inline pcap_t* get_pcap_handle() { return(pcap_handle);   };
   inline void set_pcap_handle(pcap_t *p) { pcap_handle = p; };
   inline FILE*   get_pcap_list()   { return(pcap_list);     };
   void startPacketPolling();
   void shutdown();
-  u_int getNumDroppedPackets();
   bool set_packet_filter(char *filter);
   inline bool read_from_pcap_dump() { return(read_pkts_from_pcap_dump); };
 };
