@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,28 +24,16 @@
 
 #include "ntop_includes.h"
 
-#define SECOND_SCRIPT_PATH   "second.lua"
-#define MINUTE_SCRIPT_PATH   "minute.lua"
-#define HOURLY_SCRIPT_PATH   "hourly.lua"
-#define DAILY_SCRIPT_PATH    "daily.lua"
-
 class PeriodicActivities {
  private:
-  pthread_t secondLoop, minuteLoop, hourLoop, dayLoop;
-
-  u_int32_t roundTime(u_int32_t now, u_int32_t rounder);
-  void runScript(char *path, u_int32_t when);
+  ThreadedActivity *activities[CONST_MAX_NUM_THREADED_ACTIVITIES];
+  u_int16_t num_activities;
 
  public:
   PeriodicActivities();
   ~PeriodicActivities();
 
-  void loop();
   void startPeriodicActivitiesLoop();
-  void secondActivitiesLoop();
-  void minuteActivitiesLoop();
-  void hourActivitiesLoop();
-  void dayActivitiesLoop();
 };
 
 #endif /* _PERIODIC_ACTIVITIES_H_ */
