@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,29 +31,38 @@ class Redis {
   redisContext *redis;
   Mutex *l;
   char *redis_host;
+  char *redis_password;
+  u_int32_t num_requests, num_reconnections;
   u_int16_t redis_port;
   u_int8_t redis_db_id;
   pthread_t esThreadLoop;
+  pthread_t lsThreadLoop;
   bool operational;
-
-  void setDefaults();
+  StringCache_t *stringCache;
+  u_int numCached;
+  
   void reconnectRedis();
-  int msg_push(const char *cmd, const char *queue_name, char *msg, u_int queue_trim_size);
+  int msg_push(const char *cmd, const char *queue_name, char *msg, u_int queue_trim_size, bool trace_errors = true);
   int oneOperator(const char *operation, char *key);
   int twoOperators(const char *operation, char *op1, char *op2);
   int pushHost(const char* ns_cache, const char* ns_list, char *hostname,
-	       bool dont_check_for_existance, bool localHost);
+	       bool dont_check_for_existence, bool localHost);
   int popHost(const char* ns_list, char *hostname, u_int hostname_len);
-
+  void addToCache(char *key, char *value, u_int expire_secs);
+  bool isCacheable(char *key);
+  bool expireCache(char *key, u_int expire_sec);
+	  
  public:
-  Redis(char *redis_host = (char*)"127.0.0.1", u_int16_t redis_port = 6379, u_int8_t _redis_db_id = 0);
+  Redis(const char *redis_host = (char*)"127.0.0.1",
+	const char *redis_password = NULL,
+	u_int16_t redis_port = 6379, u_int8_t _redis_db_id = 0);
   ~Redis();
 
   char* getVersion(char *str, u_int str_len);
-
+  void setDefaults();
   inline bool isOperational() { return(operational); };
   int expire(char *key, u_int expire_sec);
-  int get(char *key, char *rsp, u_int rsp_len);
+  int get(char *key, char *rsp, u_int rsp_len, bool cache_it = false);
   int hashGet(char *key, char *member, char *rsp, u_int rsp_len);
   int hashDel(char *key, char *field);
   int hashSet(char *key, char *field, char *value);
@@ -62,14 +71,15 @@ class Redis {
   char* popSet(char *pop_name, char *rsp, u_int rsp_len);
   int keys(const char *pattern, char ***keys_p);
   int hashKeys(const char *pattern, char ***keys_p);
+  int hashGetAll(const char *key, char ***keys_p, char ***values_p);
   int del(char *key);
   int zIncr(char *key, char *member);
   int zTrim(char *key, u_int trim_len);
   int zRevRange(const char *pattern, char ***keys_p);
-  int pushHostToResolve(char *hostname, bool dont_check_for_existance, bool localHost);
+  int pushHostToResolve(char *hostname, bool dont_check_for_existence, bool localHost);
   int popHostToResolve(char *hostname, u_int hostname_len);
 
-  int pushHostToTrafficFiltering(char *hostname, bool dont_check_for_existance, bool localHost);
+  int pushHostToTrafficFiltering(char *hostname, bool dont_check_for_existence, bool localHost);
   int popHostToTrafficFiltering(char *hostname, u_int hostname_len);
 
   char* getTrafficFilteringCategory(char *numeric_ip, char *buf, u_int buf_len, bool query_httpbl_if_unknown);
@@ -85,67 +95,35 @@ class Redis {
 
   int addHostToDBDump(NetworkInterface *iface, IpAddress *ip, char *name);
 
+  int sadd(const char *set_name, char *item);
+  int srem(const char *set_name, char *item);
   int smembers(lua_State* vm, char *setName);
+  int smembers(const char *set_name, char ***members);
 
   void setHostId(NetworkInterface *iface, char *daybuf, char *host_name, u_int32_t id);
   u_int32_t host_to_id(NetworkInterface *iface, char *daybuf, char *host_name, bool *new_key);
   int id_to_host(char *daybuf, char *host_idx, char *buf, u_int buf_len);
-  int lpush(const char *queue_name, char *msg, u_int queue_trim_size);
+  int lpush(const char *queue_name, char *msg, u_int queue_trim_size, bool trace_errors = true);
   int rpush(const char *queue_name, char *msg, u_int queue_trim_size);
   int lindex(const char *queue_name, int idx, char *buf, u_int buf_len);
   u_int llen(const char *queue_name);
+  int lset(const char *queue_name, u_int32_t idx, const char *value);
+  int lrem(const char *queue_name, const char *value);
+  int lrange(const char *list_name, char ***elements, int start_offset, int end_offset);
   int lpop(const char *queue_name, char *buf, u_int buf_len);
   int lpop(const char *queue_name, char ***elements, u_int num_elements);
-
+  int flushDb();
+  void flushCache();
   /**
    * @brief Increment a redis key and return its new value
    *
    * @param key The key whose value will be incremented.
    */
   u_int32_t incrKey(char *key);
-
-  /**
-   * @brief Queue an alert in redis
-   *
-   * @param level The alert level
-   * @param s     The alert status (alert on/off)
-   * @param t     The alert type
-   * @param msg   The alert message
-   */
-  void queueAlert(AlertLevel level, AlertStatus s, AlertType t, char *msg);
-
-  /**
-   * @brief Returns the number of queued alerts in redis generated by ntopng
-   *
-   */
-  inline u_int getNumQueuedAlerts() { return(llen((char*)CONST_ALERT_MSG_QUEUE)); };
-
-  /**
-   * @brief Returns up to the specified number of alerts, and removes them from redis. The first parameter must be long enough to hold the returned results
-   * @param allowed_hosts The list of hosts allowed to be returned by this function
-   * @param alerts The returned alerts
-   * @param start_idx The initial queue index from which extract messages. Zero (0) is the first (i.e. most recent) queue element.
-   * @param num The maximum number of alerts to return.
-   * @return The number of elements read.
-   *
-   */
-  u_int getQueuedAlerts(patricia_tree_t *allowed_hosts, char **alerts, u_int start_idx, u_int num);
-
-  /**
-   * @brief Delete the alert identified by the specified index.
-   * @param idx The queued alert index to delete. Zero (0) is the first (i.e. most recent) queue element.
-   * @return The number of elements read.
-   *
-   */
-  void deleteQueuedAlert(u_int32_t idx);
-
-  /**
-   * @brief Flush all queued alerts
-   *
-   */
-  inline void flushAllQueuedAlerts() { delKey((char*)CONST_ALERT_MSG_QUEUE);       };
-  int delKey(char *key)              { return(oneOperator("DEL", key));            };
   int rename(char *oldk, char *newk) { return(twoOperators("RENAME", oldk, newk)); };
+  void lua(lua_State *vm);
+  char* dump(char *key);
+  int restore(char *key, char *buf);
 };
 
 #endif /* _REDIS_H_ */
