@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -40,6 +40,10 @@ class TcpFlowStats {
   void deserialize(json_object *o);
   json_object* getJSONObject();
   void lua(lua_State* vm, const char *label);
+  inline void sum(TcpFlowStats *s) {
+    s->numSynFlows += numSynFlows, s->numEstablishedFlows += numEstablishedFlows,
+      s->numResetFlows += numResetFlows, s->numFinFlows += numFinFlows;
+  };
 };
 
 #endif /* _TCP_FLOW_STATS_H_ */
