@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,33 +29,49 @@
 #define TRACE_LEVEL_NORMAL    2
 #define TRACE_LEVEL_INFO      3
 #define TRACE_LEVEL_DEBUG     6
+#define TRACE_LEVEL_TRACE     9
 
 #define TRACE_ERROR     TRACE_LEVEL_ERROR, __FILE__, __LINE__
 #define TRACE_WARNING   TRACE_LEVEL_WARNING, __FILE__, __LINE__
 #define TRACE_NORMAL    TRACE_LEVEL_NORMAL, __FILE__, __LINE__
 #define TRACE_INFO      TRACE_LEVEL_INFO, __FILE__, __LINE__
 #define TRACE_DEBUG     TRACE_LEVEL_DEBUG, __FILE__, __LINE__
+#define TRACE_TRACE     TRACE_LEVEL_TRACE, __FILE__, __LINE__
 
-#define MAX_TRACE_LEVEL 6
+#define MAX_TRACE_LEVEL 9
 #define TRACE_DEBUGGING MAX_TRACE_LEVEL
 
 class Mutex; /* Forward */
+class Redis;
 
 /* ******************************* */
 
 class Trace {
  private:
-  u_int8_t traceLevel;
+  char *logFile;
+  FILE *logFd;
+  Redis *traceRedis;
+  int *logFileTracesCount;
+  volatile u_int8_t traceLevel;
 
 #ifdef WIN32
   VOID AddToMessageLog(LPTSTR lpszMsg);
+  Mutex rotate_mutex;
+#else
+  bool logFileMsg;
+  FILE *logFdShadow;
+  int *logFileTracesCountShadow;
 #endif
 
+
  public:
   Trace();
   ~Trace();
 
   void init();
+  void initRedis(const char *redis_host, const char *redis_password, u_int16_t redis_port, u_int8_t _redis_db_id);
+  void rotate_logs(bool force_rotation);
+  void set_log_file(const char *log_file);
   void set_trace_level(u_int8_t id);
   inline u_int8_t get_trace_level() { return(traceLevel); };
   void traceEvent(int eventTraceLevel, const char* file, const int line, const char * format, ...);
