@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,6 @@
 class TrafficStats {
  private:
   u_int64_t numPkts, numBytes;
-  u_int64_t numDeserializedPkts, numDeserializedBytes;
 
  public:
   TrafficStats();
@@ -36,8 +35,6 @@ class TrafficStats {
   inline void incStats(u_int pkt_len)       { numPkts++, numBytes += pkt_len; };
   inline u_int64_t getNumPkts()             { return(numPkts);                };
   inline u_int64_t getNumBytes()            { return(numBytes);               };
-  inline u_int64_t getNumDeserializedPkts() { return(numDeserializedPkts);    };
-  inline u_int64_t getNumDeserializedBytes(){ return(numDeserializedBytes);   };
   void printStats();
 
   char* serialize();
