@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -88,6 +88,7 @@
 #define MAX_TTL   53
 #define DST_TOS   55
 #define IN_SRC_MAC   56
+#define OUT_DST_MAC   57
 #define SRC_VLAN   58
 #define DST_VLAN   59
 #define DOT1Q_SRC_VLAN   243
@@ -108,7 +109,8 @@
 #define MPLS_LABEL_8   77
 #define MPLS_LABEL_9   78
 #define MPLS_LABEL_10   79
-#define OUT_DST_MAC   80
+#define IN_DST_MAC   80
+#define OUT_SRC_MAC  81
 #define APPLICATION_ID   95
 #define PACKET_SECTION_OFFSET   102
 #define SAMPLED_PACKET_SIZE   103
@@ -128,8 +130,18 @@
 #define FLOW_DURATION_MICROSECONDS	162
 #define TCP_SRC_PORT 182
 #define TCP_DST_PORT 183
+#define INITIATOR_OCTETS   231
+#define RESPONDER_OCTETS   232
+#define FIREWALL_EVENT 233
+#define INGRESS_VRFID 234
 #define BIFLOW_DIRECTION   239
+#define POST_NAT_SRC_IPV4_ADDR        225
+#define POST_NAT_DST_IPV4_ADDR        226
+#define POST_NAPT_SRC_TRANSPORT_PORT  227
+#define POST_NAPT_DST_TRANSPORT_PORT  228
 #define OBSERVATION_POINT_TYPE   277
+#define INITIATOR_PKTS  298
+#define RESPONDER_PKTS  299
 #define OBSERVATION_POINT_ID   300
 #define SELECTOR_ID   302
 #define IPFIX_SAMPLING_ALGORITHM   304
@@ -142,7 +154,7 @@
 #define FRAGMENTS   NTOP_BASE_ID+80
 #define SRC_AS_MAP  NTOP_BASE_ID+443
 #define DST_AS_MAP  NTOP_BASE_ID+444
-
+#define NPROBE_IPV4_ADDRESS  NTOP_BASE_ID+471 /* Exported only via ZMQ */
 /* NTOP_BASE_ID+82 ->87  Available (deprecated) */
 #define NUM_PKTS_UP_TO_128_BYTES   NTOP_BASE_ID+88
 #define NUM_PKTS_128_TO_256_BYTES   NTOP_BASE_ID+89
