@@ -24,14 +24,6 @@
 --!  @ingroup Script
 ---------------------------------
 
-
---! @memberof interface
---! @brief Get the default network interface name.
---! @details For more information please read the @ref ntop_get_default_interface_name documentation.
---! @return {string} The default network interface name of ntopng.
-function getDefaultIfName()
-
-
 --! @memberof interface
 --! @brief Sets the network interface name identified by Id as active one on which to perform operations.
 --! @details For more information please read the @ref ntop_set_active_interface_id documentation.
@@ -108,20 +100,6 @@ function restoreHost(host_ip)
 --! @return An hashtable containing the flow information of network interface with minimal details, and number of hits.
 function getFlowsInfo()
 
---! @memberof interface
---! @memberof interface
---! @brief Get the flow information (minimal details) of network interface for local hosts.
---! @details Same as getFlowsInfo() but restricted to local hosts only
---! @return An hashtable containing the flow information of network interface with minimal details, and number of hits.
-function getLocalFlowsInfo()
-
---! @memberof interface
---! @brief Get the flow peers information of network interface.
---! @details For more information please read the @ref ntop_get_interface_flows_peers documentation.
---! @tparam string host_name The host name.
---! @return An hashtable of hashtable containing the flow peers information of network interface.
-function getFlowPeers(host_name)
-
 --! @memberof interface
 --! @brief Get the flow peers information identified by key of network interface.
 --! @details For more information please read the @ref ntop_get_interface_find_flow_by_key documentation.
@@ -142,12 +120,6 @@ function findHost(key)
 --! @return The endpoint of network interface.
 function getEndpoint()
 
---! @memberof interface
---! @brief Increase interface drops.
---! @details For more information please read the @ref ntop_increase_drops documentation.
---! @tparam drops Number of interface drops to be incremented to interface stats.
-function incrDrops(drops)
-
 --! @memberof interface
 --! @brief Check if the network interface is running.
 --! @details For more information please read the @ref ntop_interface_is_running documentation.
@@ -219,7 +191,7 @@ function delCache(key)
 --! @brief Get the members of a redis set identified by key.
 --! @details For more information please read the @ref ntop_get_set_members_redis documentation.
 --! @tparam string key The redis key.
---! @return An hashtabele containing the member of a redis set identified by key if the set exists.
+--! @return An hashtable containing the member of a redis set identified by key if the set exists.
 function getMembersCache(key)
 
 --! @memberof ntop
@@ -330,7 +302,7 @@ function exists(path)
 --! @brief Scan the input directory and return the list of files.
 --! @details For more information please read the @ref ntop_list_dir_files documentation.
 --! @tparam string path The absolute path of directory.
---! @return An hastable containing the list of files of the input directory.
+--! @return An hashtable containing the list of files of the input directory.
 function readdir(path)
 
 ------------------ ZMQ ------------------
@@ -413,7 +385,7 @@ function getUserGroup()
 
 --! @memberof ntop
 --! @brief Get the user allowed networks.
---! @details Return the stored allowed netwroks (i.e. the networks this user can see) for the given user.
+--! @details Return the stored allowed networks (i.e. the networks this user can see) for the given user.
 --! @return The user allowed networks
 function getAllowedNetworks()
 
@@ -462,14 +434,14 @@ function deleteUser(username)
 --! @details For more information please read the @ref ntop_resolve_address documentation.
 --! @tparam string numIP The IP address.
 --! @return The host name of IP address.
-function resolveAddress(numIP)
+function resolveName(numIP)
 
 --! @memberof ntop
 --! @brief Get the resolved address in the redis cache stored at key.
 --! @details For more information please read the @ref ntop_get_resolved_address documentation.
 --! @tparam string key The IP address.
 --! @return The host name of IP address.
-function getResolvedAddress(key)
+function getResolvedName(key)
 
 
 
