@@ -6,10 +6,10 @@ dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 require "lua_utils"
 
-sendHTTPHeader('text/html; charset=iso-8859-1')
+sendHTTPContentTypeHeader('text/html')
 
 if(haveAdminPrivileges()) then
-   username = _GET["username"]
+   username = _POST["username"]
    
    if(username == nil) then
       print ("{ \"result\" : -1, \"message\" : \"Invalid parameters\" }")
