@@ -6,12 +6,12 @@ dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 require "lua_utils"
 
-sendHTTPHeader('text/html; charset=iso-8859-1')
+sendHTTPContentTypeHeader('text/html')
 
-username = _GET["username"]
-old_password = _GET["old_password"]
-new_password = _GET["new_password"]
-confirm_new_password = _GET["confirm_new_password"]
+username = _POST["username"]
+old_password = _POST["old_password"]
+new_password = _POST["new_password"]
+confirm_new_password = _POST["confirm_password"]
 
 user_group = ntop.getUserGroup()
 if(user_group == "administrator") then
@@ -26,12 +26,14 @@ if((username == nil) or (old_password == nil) or (new_password == nil) or (confi
    return
 end
 
+username = string.lower(username)
+
 if(new_password ~= confirm_new_password) then
    print ("{ \"result\" : -1, \"message\" : \"Password don't match\" }")
    return
 end
 
-if(ntop.resetUserPassword(_SESSION["user"], username, old_password, new_password)) then
+if(ntop.resetUserPassword(_SESSION["user"], username, unescapeHTML(old_password), unescapeHTML(new_password))) then
    print ("{ \"result\" : 0, \"message\" : \"Password changed successfully\" }")
 else
    print ("{ \"result\" : -1, \"message\" : \"Unable to set the new user password: perhaps the old password was invalid ?\" }")
