@@ -1,5 +1,5 @@
 --
--- (C) 2013-16 - ntop.org
+-- (C) 2013-17 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -7,7 +7,7 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
 
-sendHTTPHeader('text/html; charset=iso-8859-1')
+sendHTTPContentTypeHeader('text/html')
 
 ntop.dumpFile(dirs.installdir .. "/httpdocs/inc/header.inc")
 
@@ -31,7 +31,7 @@ print [[
 			url: url_update ,
 	 ]]
 
-print('title: "Hosts by Country",\n')
+print('title: "'.. i18n("country_stats.hosts_by_country")..'",\n')
 print ('rowCallback: function ( row ) { return country_table_setID(row); },')
 
 -- Set the preference table
@@ -54,7 +54,7 @@ print [[
                                 }
            },
 	     {
-			     title: "Name",
+			     title: "]] print(i18n("name")) print[[",
 				 field: "column_id",
 				 sortable: true,
                              css: {
@@ -64,8 +64,68 @@ print [[
 				 },
 			  ]]
 
+print [[
+			     {
+			     title: "]] print(i18n("hosts_stats.hosts")) print[[",
+				 field: "column_hosts",
+				 sortable: true,
+                             css: {
+			        textAlign: 'center'
+			     }
 
-ntop.dumpFile(dirs.installdir .. "/httpdocs/inc/country_stats_top.inc")
+				 },
+			     {
+			     title: "]] print(i18n("show_alerts.alerts")) print[[",
+				 field: "column_alerts",
+				 sortable: true,
+                             css: {
+			        textAlign: 'center'
+			     }
+
+				 },
+
+			     {
+			     title: "]] print(i18n("seen_since")) print[[",
+				 field: "column_since",
+				 sortable: true,
+                             css: {
+			        textAlign: 'center'
+			     }
+
+				 },
+]]
+
+print [[
+			     {
+			     title: "]] print(i18n("breakdown")) print[[",
+				 field: "column_breakdown",
+				 sortable: false,
+	 	             css: {
+			        textAlign: 'center'
+			     }
+				 },
+			     {
+			     title: "]] print(i18n("throughput")) print[[",
+				 field: "column_thpt",
+				 sortable: true,
+	 	             css: {
+			        textAlign: 'right'
+			     }
+				 },
+			     {
+			     title: "]] print(i18n("traffic")) print[[",
+				 field: "column_traffic",
+				 sortable: true,
+	 	             css: {
+			        textAlign: 'right'
+			     }
+				 }
+			     ]
+	       });
+
+
+       </script>
+
+]]
 
-ntop.dumpFile(dirs.installdir .. "/httpdocs/inc/hosts_stats_bottom.inc")
 dofile(dirs.installdir .. "/scripts/lua/inc/footer.lua")
