@@ -1,5 +1,5 @@
 --
--- (C) 2013-16 - ntop.org
+-- (C) 2013-17 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -8,48 +8,47 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 require "lua_utils"
 require "graph_utils"
 require "db_utils"
+require "flow_aggregation_utils"
 require "template"
 
-
 interface.select(ifname)
-ifstats = aggregateInterfaceStats(interface.getStats())
+ifstats = interface.getStats()
 
-ifId = _GET["ifId"]
-host = _GET["host"]
-peer = _GET["peer"]
-epoch = _GET["epoch"]
-l7proto = _GET["l7proto"]
+local ifId = _GET["ifid"]
+local host = _GET["peer1"]
+local peer = _GET["peer2"]
+local vlan = _GET["vlan"]
+local epoch = _GET["epoch"]
+local l7proto = _GET["l7proto"]
 if l7proto == nil or l7proto == "" then
    l7proto = _GET["l7_proto_id"]
 end
 
-currentPage = _GET["currentPage"]
-perPage = _GET["perPage"]
-sortColumn = _GET["sortColumn"]
-sortOrder = _GET["sortOrder"]
+local currentPage = _GET["currentPage"]
+local perPage = _GET["perPage"]
+local sortColumn = _GET["sortColumn"]
+local sortOrder = _GET["sortOrder"]
 
-epoch_begin = _GET["epoch_begin"]
-if epoch_begin == nil or epoch_begin == "" then
-   epoch_begin = _GET["epoch_start"]
-end
-epoch_end = _GET["epoch_end"]
+local epoch_begin = _GET["epoch_begin"]
+local epoch_end = _GET["epoch_end"]
 
-l4proto = _GET["l4proto"]
+local l4proto = _GET["l4proto"]
 if l4proto == nil or l4proto == "" then
    l4proto = _GET["l4_proto_id"]
 end
-port = _GET["port"]
-info = _GET["info"]
-profile = _GET["profile"]
-limit = _GET["limit"]
+local port = _GET["port"]
+local info = _GET["info"]
+local vlan = _GET["vlan"]
+local profile = _GET["profile"]
+local limit = _GET["limit"]
 
-format = _GET["format"]
+local format = _GET["format"]
 if(format == nil) then format = "json" end
 
-ip_version = _GET["version"]
+local ip_version = _GET["version"]
 if(ip_version == nil) then ip_version = "4" end
 
-ip_version = tonumber(ip_version)
+local ip_version = tonumber(ip_version)
 
 if((currentPage == nil) or (currentPage == "")) then currentPage = 1 end
 if((perPage == nil) or (perPage == "")) then perPage = 5 end
@@ -60,9 +59,20 @@ if(format == "txt") then
    limit = 99999
    currentPage = 1
    perPage = limit
+else
+   if limit == nil then
+      -- when flow aggregation is used, requests can be made with a nil limit so it is important to re-calculate it
+      local count = getNumFlows(ifId, ip_version, host, (l4proto or ""), (port or ""), (l7proto or ""), (info or ""),
+				(vlan or ""), (profile or ""),
+				epoch_begin, epoch_end, true --[[ force count from the raw flows table --]])
+      if count ~= nil and count[1] ~= nil then
+	 limit = count[1]["TOT_FLOWS"]
+      end
+   end
 end
 
-res = getInterfaceTopFlows(ifId, ip_version, host, peer, (l7proto or ""), (l4proto or ""), (port or ""), (info or ""),
+res = getInterfaceTopFlows(ifId, ip_version, host, peer, (l7proto or ""), (l4proto or ""), (port or ""),
+			   vlan, (profile or ''), (info or ""),
 			   epoch_begin, epoch_end, (currentPage-1)*perPage, perPage, sortColumn or 'BYTES', sortOrder or 'DESC')
 
 if(format == "txt") then
@@ -88,7 +98,7 @@ if(format == "txt") then
       filename = filename.."_to_"..string.gsub(formatEpoch(epoch_end), ' ', '-')
    end
    filename = filename..".txt"
-   sendHTTPHeader('text/plain; charset=iso-8859-1', 'attachment; filename="'..filename..'"')
+   sendHTTPContentTypeHeader('text/plain', 'attachment; filename="'..filename..'"')
    local num = 0
    for _,flow in pairs(res) do
       if(num == 0) then
@@ -120,7 +130,7 @@ if(format == "txt") then
       num = num + 1
    end
 else
-   sendHTTPHeader('text/html; charset=iso-8859-1')
+   sendHTTPContentTypeHeader('text/html')
    -- JSON
    if((res == nil) or (type(res) == "string")) then
       return('{ "currentPage" : 1,  "data" : [], "perPage" : '..perPage..',  "sort" : [ [ "column_", "desc" ] ],"totalRows" : 0 }')
@@ -131,11 +141,14 @@ else
 
       for _,flow in pairs(res) do
 	 local num = 0
-	 local base = "<A HREF='"..ntop.getHttpPrefix().."/lua/pro/db_explorer.lua?ifId="..ifId.."&epoch_begin="..epoch_begin.."&epoch_end="..epoch_end
+	 local base = "<A HREF='"..ntop.getHttpPrefix().."/lua/pro/db_explorer.lua?search_flows=true&ifid="..ifId.."&epoch_begin="..epoch_begin.."&epoch_end="..epoch_end.."&search="
+	 if not isEmptyString(flow["VLAN_ID"]) and tonumber(flow["VLAN_ID"]) > 0 then
+	    base = base.."&vlan="..flow["VLAN_ID"]
+	 end
 
 	 if(flow["L4_SRC_PORT"] ~= nil) then
-	    local base_host_url = base.."&host="
-	    local base_port_url = base.."&port="
+	    local base_host_url = base.."&l4proto=&port=&host="
+	    local base_port_url = base.."&l4proto=&host=&port="
 	    local pname = l4ProtoToName(flow["PROTOCOL"])
 	    local lower_pname = string.lower(pname)
 
@@ -152,25 +165,43 @@ else
 	       flow["CLIENT"] = base_host_url..flow["IP_SRC_ADDR"] .."'>"..client.."</A>"
 	       flow["SERVER"] = base_host_url..flow["IP_DST_ADDR"] .."'>"..server.."</A>"
 
-	       if((sport ~= nil) and (sport ~= "0")) then flow["CLIENT"] = flow["CLIENT"] .. ":"..base_port_url..flow["L4_SRC_PORT"].."'>"..sport.."</A>" end
-	       if((dport ~= nil) and (dport ~= "0")) then flow["SERVER"] = flow["SERVER"] .. ":"..base_port_url..flow["L4_DST_PORT"].."'>"..dport.."</A>" end
+	       if((sport ~= nil) and (sport ~= "0")) then
+		  flow["CLIENT"] = flow["CLIENT"] .. ":"..base_port_url..flow["L4_SRC_PORT"].."'>"..sport.."</A>"
+	       end
+	       if((dport ~= nil) and (dport ~= "0")) then
+		  flow["SERVER"] = flow["SERVER"] .. ":"..base_port_url..flow["L4_DST_PORT"].."'>"..dport.."</A>"
+	       end
+
+	       flow["INFO"] = base.."&l4proto=&port=&host=&info="..flow["INFO"].."'><span title='"..flow["INFO"].."'>"..shortenString(flow["INFO"]).."</span></A>"
+
+	       flow["PROTOCOL"] = base.."&protocol=&port=&host=&l4proto="..flow["PROTOCOL"].."'>"..pname.."</A>"
+	       flow["L7_PROTO"] = base.."&port=&host=&l4proto=&protocol="..flow["L7_PROTO"].."'>"..getApplicationLabel(interface.getnDPIProtoName(tonumber(flow["L7_PROTO"]))).."</A>"
+
+	       -- nProbe/related database doesn't handle idx so we have to disable the info
+	       if flow["idx"] ~= "-1" then
+		  flow["FLOW_URL"] = base.."&row_id="..flow["idx"].."&version="..ip_version.."'><span class='label label-info'>Info</span></A>"
+	       else
+		  flow["FLOW_URL"] = "<span class='label label-default'>Info</span></A>"
+	       end
+
+	       if flow["PROFILE"] ~= nil and flow["PROFILE"] ~="" then
+		  flow["INFO"] = "<span class='label label-primary'>"..flow["PROFILE"].."</span>&nbsp;"..flow["INFO"]
+	       end
 
-	       flow["PROTOCOL"] = base.."&l4proto="..flow["PROTOCOL"].."'>"..pname.."</A>"
-	       flow["L7_PROTO"] = base.."&protocol="..flow["L7_PROTO"].."'>"..getApplicationLabel(interface.getnDPIProtoName(tonumber(flow["L7_PROTO"]))).."</A>"
-	       flow["FLOW_URL"] = base.."&flow_idx="..flow["idx"].."&version="..ip_version.."'><span class='label label-info'>Info</span></A>"
-	       flow["INFO"] = base.."&info="..flow["INFO"].."'>"..flow["INFO"].."</A>"
-               if flow["PROFILE"] ~= nil and flow["PROFILE"] ~="" then
-                   flow["INFO"] = "<span class='label label-primary'>"..flow["PROFILE"].."</span>&nbsp;"..flow["INFO"]
-               end
 	    else
 	       flow["CLIENT"] = client..":"..ntop.getservbyport(tonumber(flow["L4_SRC_PORT"]), lower_pname)
 	       flow["SERVER"] = server..":"..ntop.getservbyport(tonumber(flow["L4_DST_PORT"]), lower_pname)
 	       flow["PROTOCOL"] = pname
 	       flow["L7_PROTO"] = getApplicationLabel(interface.getnDPIProtoName(tonumber(flow["L7_PROTO"])))
 	       flow["FLOW_URL"] = ""
+	       flow["INFO"] = "<span title='"..flow["INFO"].."'>"..shortenString(flow["INFO"]).."</span>"
 	    end
 	 end
 
+	 for _, k in pairs({"CLIENT", "SERVER", "PROTOCOL", "L7_PROTO"}) do
+	    flow[k] = "<i>"..flow[k].."</i>"
+	 end
+
 	 duration = tonumber(flow["LAST_SWITCHED"])-tonumber(flow["FIRST_SWITCHED"])+1
 	 flow["AVG_THROUGHPUT"] = bitsToSize((8*tonumber(flow["BYTES"])) / duration)
 
