@@ -1,5 +1,5 @@
 --
--- (C) 2013-16 - ntop.org
+-- (C) 2013-17 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -10,7 +10,8 @@ require "lua_utils"
 sendHTTPHeader('application/json')
 
 interface.select(ifname)
-hosts_stats,total = aggregateHostsStats(interface.getHostsInfo())
+hosts_stats = interface.getHostsInfo()
+hosts_stats = hosts_stats["hosts"]
 
 print [[
 {
@@ -38,7 +39,7 @@ for key, value in pairs(hosts_stats) do
    if(val > threshold) then
 
    if(num > 0) then print(",\n") else print("\n") end
-   res = ntop.getResolvedAddress(key)
+   res = getResolvedAddress(hostkey2hostinfo(key))
 
    --if(res == nil) then res = "AAA" end
    print("{ \"name\": \"" .. res .. "\", \"size\": " .. (hosts_stats[key]["bytes.sent"]+hosts_stats[key]["bytes.rcvd"]).. "} ")
@@ -77,4 +78,4 @@ print [[
 ]
 }
 
-]]
\ No newline at end of file
+]]
