@@ -1,5 +1,5 @@
 --
--- (C) 2013-16 - ntop.org
+-- (C) 2013-17 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -7,11 +7,11 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
 
-sendHTTPHeader('text/html; charset=iso-8859-1')
+sendHTTPContentTypeHeader('text/html')
 
 interface.select(ifname)
 
-mode = _GET["mode"]
+mode = _GET["direction"]
 type = _GET["distr"]
 host_info = url2hostinfo(_GET)
 host = interface.getHostInfo(host_info["host"],host_info["vlan"])
@@ -62,4 +62,4 @@ end
 
 print "\n]"
 
-end
\ No newline at end of file
+end
