@@ -1,5 +1,5 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-17 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -7,15 +7,21 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
 
-sendHTTPHeader('text/html; charset=iso-8859-1')
+sendHTTPContentTypeHeader('text/html')
 
 interface.select(ifname)
-ifstats = aggregateInterfaceStats(interface.getStats())
+ifstats = interface.getStats()
 
-type = _GET["type"]
+distr_type = _GET["distr"]
 
-if((type == nil) or (type == "size")) then
+if((distr_type == nil) or (distr_type == "size")) then
    what = ifstats["pktSizeDistribution"]
+elseif distr_type == "ipver" then
+   print "[\n"
+   print("\t { \"label\": \"IPv6\", \"value\": ".. ifstats.eth.IPv6_packets .." },")
+   print("\t { \"label\": \"IPv4\", \"value\": ".. ifstats.eth.IPv4_packets .." }")
+   print "\n]"
+   return
 end
 
 local pkt_distribution = {
@@ -43,13 +49,15 @@ num = 0
 sum = 0
 for key, value in pairs(what) do
    if(value > threshold) then
-      if(num > 0) then
-	 print ",\n"
+      if(pkt_distribution[key] ~= nil) then
+	 if(num > 0) then
+	    print ",\n"
+	 end
+	 	 
+	 print("\t { \"label\": \"" .. pkt_distribution[key] .."\", \"value\": ".. value .." }") 
+	 num = num + 1
+	 sum = sum + value
       end
-   
-      print("\t { \"label\": \"" .. pkt_distribution[key] .."\", \"value\": ".. value .." }") 
-      num = num + 1
-      sum = sum + value
    end
 end
 
