@@ -1,5 +1,5 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-17 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -12,18 +12,26 @@ sendHTTPHeader('application/json')
 tracked_host = _GET["host"]
 
 interface.select(ifname)
-peers = interface.getFlowPeers()
-
+max_num_peers = 10
 max_num_links = 32
-max_num_hosts = 8
+
+peers = getTopFlowPeers(tracked_host, max_num_peers, true --[[ high details for cli2srv.last/srv2cli.last fields ]])
 
 local debug = false
 
 -- 1. compute total traffic
 total_traffic = 0
-for key, values in pairs(peers) do
-   total_traffic = total_traffic + values["sent.last"] + values["rcvd.last"]
-   if(debug) then io.write("->"..key.."\t[".. (values["sent.last"] + values["rcvd.last"]) .."][".. values["duration"].."]" .. "\n") end
+
+for _, values in ipairs(peers) do
+   local key
+   if(values["cli.ip"] == tracked_host) then
+      key = hostinfo2hostkey(values, "srv")
+   else
+      key = hostinfo2hostkey(values, "cli")
+   end
+
+   total_traffic = total_traffic + values["bytes.last"]
+   if(debug) then io.write("->"..key.."\t[".. (values["bytes.last"]) .."][".. values["duration"].."]" .. "\n") end
 end
 
 if(debug) then io.write("\n") end
@@ -37,28 +45,38 @@ end
 
 if(debug) then io.write("\nThreshold: "..threshold.."\n") end
 
+-- map host -> incremental number
 hosts = {}
 num = 0
 print '{"nodes":[\n'
 
 -- print(" >>>[" .. tracked_host .. "]\n")
 
+-- fills hosts table with available hosts and prints out some json code
 while(num == 0) do
-   for key, values in pairs(peers) do
+   for _, values in ipairs(peers) do
+      local key
+      if(values["cli.ip"] == tracked_host) then
+	 key = hostinfo2hostkey(values, "srv")
+      else
+	 key = hostinfo2hostkey(values, "cli")
+      end
+
       -- print("[" .. key .. "][" .. values["client"] .. ",".. values["client.vlan_id"] .. "][" .. values["server"] .. ",".. values["client.vlan_id"] .. "]\n")
-      last = values["sent.last"] + values["rcvd.last"]
+      last = values["bytes.last"]
       if((last == 0) and (values.duration < 3)) then
-	 last = values["sent"] + values["rcvd"]
+	 last = values["bytes"]
       end
       if(last > threshold) then
 
-	 if(debug) then io.write("==>"..key.."\t[T:"..tracked_host.. (values["sent.last"] + values["rcvd.last"]) .."][".. values["duration"].."][" .. last.. "]\n") end
+	 if(debug) then io.write("==>"..key.."\t[T:"..tracked_host.. (values["bytes.last"]) .."][".. values["duration"].."][" .. last.. "]\n") end
 	 if((debug) and (findString(key, tracked_host) ~= nil))then io.write("findString(key, tracked_host)==>"..findString(key, tracked_host)) end
 	 if((debug) and (findString(values["cli.ip"], tracked_host) ~= nil)) then io.write("findString(values[cli.ip], tracked_host)==>"..findString(values["cli.ip"], tracked_host)) end
 	 if((debug) and (findString(values["srv.ip"], tracked_host) ~= nil)) then io.write("findString(values[srv.ip], tracked_host)==>"..findString(values["srv.ip"], tracked_host)) end
 
-	 k = string.split(key, " ")
+	 local k = {hostinfo2hostkey(values, "cli"), hostinfo2hostkey(values, "srv")}  --string.split(key, " ")
 
+	 -- Note some checks are redundant here, they are already performed in getFlowPeers
 	 if((tracked_host == nil)
    	    or findString(k[1], tracked_host)
             or findString(k[2], tracked_host)
@@ -67,11 +85,8 @@ while(num == 0) do
 	    -- print("[" .. key .. "]")
 	    -- print("[" .. tracked_host .. "]\n")
 
-	    for key,word in pairs(split(key, " ")) do
-	       if(num >= max_num_hosts) then
-		  break
-	       end
-
+	    -- for each cli, srv
+	    for key,word in pairs(k) do
 	       if(hosts[word] == nil) then
 		  hosts[word] = num
 
@@ -82,7 +97,7 @@ while(num == 0) do
 		  host_info = hostkey2hostinfo(word)
 
 		  -- 3. print nodes
-		  name = shortHostName(ntop.getResolvedAddress(word))
+		  name = shortHostName(getResolvedAddress(hostkey2hostinfo(word)))
 
 		  print ("\t{\"name\": \"" .. name .. "\", \"host\": \"" .. host_info["host"] .. "\", \"vlan\": \"" .. host_info["vlan"] .. "\"}")
 		  num = num + 1
@@ -109,7 +124,8 @@ if ((num == 0) and (tracked_host == nil)) then
    -- 2.1 It looks like in this network there are many flows with no clear predominant traffic
    --     Then we take the host with most traffic and print flows belonging to it
 
-   hosts_stats = interface.getHosts()
+   hosts_stats = interface.getHostsInfo(true, "column_traffic", max_num_peers)
+   hosts_stats = hosts_stats["hosts"]
    for key, value in pairs(hosts_stats) do
       value = hosts_stats[key]["traffic"]
       if((value ~= nil) and (value > top_value)) then
@@ -127,7 +143,14 @@ if ((num == 0) and (tracked_host == nil)) then
       print ("{\"name\": \"" .. top_host .. "\", \"host\": \"" .. host_info["host"] .. "\", \"vlan\": \"" .. host_info["vlan"] .. "\"}")
       num = num + 1
 
-      for key, values in pairs(peers) do
+      for _, values in ipairs(peers) do
+	 local key
+	 if(values["cli.ip"] == tracked_host) then
+	    key = hostinfo2hostkey(values, "srv")
+	 else
+	    key = hostinfo2hostkey(values, "cli")
+	 end
+
 	 if(findString(key, ip) or findString(values["client"], ip) or findString(values["server"], ip)) then
 	    for key,word in pairs(split(key, " ")) do
 	       if(hosts[word] == nil) then
@@ -138,22 +161,9 @@ if ((num == 0) and (tracked_host == nil)) then
 		  -- 3. print nodes
 		  print ("{\"name\": \"" .. word .. "\", \"host\": \"" .. host_info["host"] .. "\", \"vlan\": \"" .. host_info["vlan"] .. "\"}")
 		  num = num + 1
-
-		  if(num >= max_num_hosts) then
-		     break
-		  end
 	       end --if
-
-	       if(num >= max_num_hosts) then
-		  break
-	       end
-
 	    end -- for
 	 end -- if
-
-	 if(num >= max_num_hosts) then
-	    break
-	 end
       end -- for
    end -- if
 end -- if
@@ -168,14 +178,15 @@ num = 0
 
 -- Avoid to have a link A->B, and B->A
 reverse_nodes = {}
-for key, values in pairs(peers) do
-   val = values["sent.last"] + values["rcvd.last"]
+for _, values in ipairs(peers) do
+   key = {hostinfo2hostkey(values, "cli"), hostinfo2hostkey(values, "srv")}  --string.split(key, " ")
+   val = values["bytes.last"]
 
-   if(((val == 0) or (val > threshold)) or ((top_host ~= nil) and (findString(key, top_host) ~= nil)) and (num < max_num_links)) then
+   if(((val == 0) or (val > threshold)) or ((top_host ~= nil) and (findString(table.concat(key, " "), top_host) ~= nil)) and (num < max_num_links)) then
       e = {}
       id = 0
       --print("->"..key.."\n")
-      for key,word in pairs(split(key, " ")) do
+      for key,word in pairs(key) do
 	 --print(word .. "=" .. hosts[word].."\n")
 	 e[id] = hosts[word]
 	 id = id + 1
@@ -188,8 +199,8 @@ for key, values in pairs(peers) do
 
 	 reverse_nodes[e[1]..":"..e[0]] = 1
 
-	 sentv = values["sent.last"]
-	 recvv = values["rcvd.last"]
+	 sentv = values["cli2srv.last"]
+	 recvv = values["srv2cli.last"]
 
 	 if(val == 0) then
 	    val = 1
