@@ -1,5 +1,5 @@
 --
--- (C) 2013-16 - ntop.org
+-- (C) 2013-17 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -7,10 +7,11 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
 
-sendHTTPHeader('text/html; charset=iso-8859-1')
+sendHTTPContentTypeHeader('text/html')
 
 interface.select(ifname)
-hosts_stats,total = aggregateHostsStats(interface.getHostsInfo(false, "column_traffic"))
+hosts_stats = interface.getHostsInfo(false, "column_traffic")
+hosts_stats = hosts_stats["hosts"]
 
 ajax_format = _GET["ajax_format"]
 
@@ -28,7 +29,7 @@ for key, value in pairs(hosts_stats) do
       if(host_info["host"] == "255.255.255.255") then
 	 key = "Broadcast"
       end
-      _hosts_stats[value] = key -- ntop.getResolvedAddress(key)
+      _hosts_stats[value] = key -- ntop.getResolvedName(key)
       if((top_value < value) or (top_key == nil)) then
 	 top_key = key
 	 top_value = value
