@@ -1,5 +1,5 @@
 --
--- (C) 2013-16 - ntop.org
+-- (C) 2013-17 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -8,15 +8,14 @@ if((dirs.scriptdir ~= nil) and (dirs.scriptdir ~= "")) then package.path = dirs.
 
 require "lua_utils"
 
-sendHTTPHeader('text/html; charset=iso-8859-1')
+sendHTTPContentTypeHeader('text/html')
 
-interface.select(_GET["id"])
--- ifstats = aggregateInterfaceStats(interface.getFlowsStatus())
+interface.select(_GET["ifid"])
 ifstats = interface.getFlowsStatus()
 --tprint(ifstats)
 
 tcpFlowStats = ifstats
-
+-- tprint(tcpFlowStats)
 sum = tcpFlowStats["SYN"]+tcpFlowStats["Established"]+tcpFlowStats["RST"]+tcpFlowStats["FIN"]
 
 print("[")
