@@ -7,10 +7,13 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "template"
 require "lua_utils"
+require "flow_aggregation_utils"
 
 local db_debug = false
 
---- ====================================================================
+local db_utils = {}
+
+-- ########################################################
 
 function iptonumber(str)
    local num = 0
@@ -20,23 +23,45 @@ function iptonumber(str)
    return num
 end
 
+-- ########################################################
 
 function expandIpV4Network(net)
-   local prefix = net:match("/(.+)")
-   address = net:gsub("/.+","")
+   local address, prefix = splitNetworkPrefix(net)
 
-   if(prefix == nil) then prefix = 32 end
+   if(prefix == nil or prefix > 32 or prefix <= 0) then prefix = 32 end
 
    local num_hosts = 2^(32-prefix)
    local addr = iptonumber(address)
 
+   for i=1,32-prefix do
+      addr = clearbit(addr, bit(i))
+   end
+
    return({ addr, addr+num_hosts-1 })
 end
 
+-- ########################################################
 
---- ====================================================================
+function flowsTableName(version, force_raw)
+   if tblname_prefs == nil then
+      tblname_prefs = ntop.getPrefs()
+   end
 
-function getInterfaceTopFlows(interface_id, version, host_or_profile, peer, l7proto, l4proto, port, info, begin_epoch, end_epoch, offset, max_num_flows, sort_column, sort_order)
+   local tblname = "flowsv"..version
+
+   if force_raw ~= true and ntop.isEnterprise() == true then
+      if useAggregatedFlows() == true then
+	 tblname = "aggr"..tblname
+      end
+   end
+
+   -- return "flowsv"..version -- FIXX: remove this line when ready
+   return tblname
+end
+
+-- ########################################################
+
+function getInterfaceTopFlows(interface_id, version, host_or_profile, peer, l7proto, l4proto, port, vlan, profile, info, begin_epoch, end_epoch, offset, max_num_flows, sort_column, sort_order, aggregated_flows)
    -- CONVERT(UNCOMPRESS(JSON) USING 'utf8') AS JSON
 
    if(version == 4) then
@@ -47,13 +72,16 @@ function getInterfaceTopFlows(interface_id, version, host_or_profile, peer, l7pr
 
    follow = " ,L4_SRC_PORT,L4_DST_PORT,VLAN_ID,PROTOCOL,FIRST_SWITCHED,LAST_SWITCHED,PACKETS,IN_BYTES + OUT_BYTES as BYTES,IN_BYTES,OUT_BYTES,idx,L7_PROTO,INFO"
    if ntop.isPro() then follow = follow..",PROFILE" end
-   follow = follow.." from flowsv"..version.." where FIRST_SWITCHED <= "..end_epoch.." and FIRST_SWITCHED >= "..begin_epoch
+
+   follow = follow.." from "..flowsTableName(version, true --[[ force raw flows --]]).." where FIRST_SWITCHED <= "..end_epoch.." and FIRST_SWITCHED >= "..begin_epoch
 
    if((l7proto ~= "") and (l7proto ~= "-1")) then follow = follow .." AND L7_PROTO="..l7proto end
    if((l4proto ~= "") and (l4proto ~= "-1")) then follow = follow .." AND PROTOCOL="..l4proto end
    if(port ~= "") then follow = follow .." AND (L4_SRC_PORT="..port.." OR L4_DST_PORT="..port..")" end
-   if(info ~= "") then follow = follow .." AND (INFO='"..info.."')" end
-   follow = follow.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL)"
+   if((vlan ~= nil) and (vlan ~= "")) then follow = follow .." AND VLAN_ID="..vlan end
+   if((profile ~= nil) and (profile ~= "")) then follow = follow .." AND PROFILE='"..profile.."'" end
+   if(info ~= "") then follow = follow .." AND (INFO LIKE '%"..info.."%')" end
+   follow = follow.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL OR NTOPNG_INSTANCE_NAME='')"
    follow = follow.." AND (INTERFACE_ID='"..tonumber(interface_id).."')"
 
    if host_or_profile ~= nil and host_or_profile ~= "" and string.starts(host_or_profile, 'profile:') then
@@ -94,7 +122,7 @@ function getInterfaceTopFlows(interface_id, version, host_or_profile, peer, l7pr
    end
 end
 
---- ====================================================================
+-- ########################################################
 
 function getFlowInfo(interface_id, version, flow_idx)
    version = tonumber(version)
@@ -122,9 +150,9 @@ function getFlowInfo(interface_id, version, flow_idx)
    end
 end
 
---- ====================================================================
+-- ########################################################
 
-function getNumFlows(interface_id, version, host, protocol, port, l7proto, info, begin_epoch, end_epoch)
+function getNumFlows(interface_id, version, host, protocol, port, l7proto, info, vlan, profile, begin_epoch, end_epoch, force_raw_flows)
    if(version == nil) then version = 4 end
 
    if(info == "") then info = nil end
@@ -150,19 +178,30 @@ function getNumFlows(interface_id, version, host, protocol, port, l7proto, info,
       end
    end
 
-   sql = "select COUNT(*) AS TOT_FLOWS, SUM(IN_BYTES + OUT_BYTES) AS TOT_BYTES, SUM(PACKETS) AS TOT_PACKETS FROM flowsv"..version.." where FIRST_SWITCHED <= "..end_epoch.." and FIRST_SWITCHED >= "..begin_epoch
-   sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL)"
+   sql = "select COUNT(*) AS TOT_FLOWS, SUM(IN_BYTES + OUT_BYTES) AS TOT_BYTES, SUM(PACKETS) AS TOT_PACKETS FROM "..flowsTableName(version, force_raw_flows --[[force count from raw flows --]]).." where FIRST_SWITCHED <= "..end_epoch.." and FIRST_SWITCHED >= "..begin_epoch
+   sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL OR NTOPNG_INSTANCE_NAME='')"
    sql = sql.." AND (INTERFACE_ID='"..tonumber(interface_id).."')"
 
    if((l7proto ~= nil) and (l7proto ~= "")) then sql = sql .." AND L7_PROTO="..l7proto end
    if((protocol ~= nil) and (protocol ~= "")) then sql = sql .." AND PROTOCOL="..protocol end
-   if(info ~= nil) then sql = sql .." AND (INFO='"..info.."')" end
+   if((vlan ~= nil) and (vlan ~= "")) then sql = sql .." AND VLAN_ID="..vlan end
+   if((profile ~= nil) and (profile ~= "")) then sql = sql .." AND PROFILE='"..profile.."'" end
+   if(info ~= nil) then sql = sql .." AND (INFO LIKE '%"..info.."%')" end
 
    if((port ~= nil) and (port ~= "")) then sql = sql .." AND (L4_SRC_PORT="..port.." OR L4_DST_PORT="..port..")" end
 
    if((host ~= nil) and (host ~= "")) then
       if(version == 4) then
-	 sql = sql .." AND (IP_SRC_ADDR=INET_ATON('"..host.."') OR IP_DST_ADDR=INET_ATON('"..host.."'))"
+	 local ip_range = expandIpV4Network(host)
+	 local ip_lowest  = ip_range[1]
+	 local ip_highest = ip_range[2]
+
+	 if ip_lowest == ip_highest then
+	    sql = sql .." AND (IP_SRC_ADDR='"..ip_highest.."' OR IP_DST_ADDR='"..ip_highest.."')"
+	 else
+	    sql = sql .." AND ((IP_SRC_ADDR>='"..ip_lowest.."' AND IP_SRC_ADDR<='"..ip_highest.."')"
+	    sql = sql .." OR (IP_DST_ADDR>='"..ip_lowest.."' AND IP_DST_ADDR<='"..ip_highest.."'))"
+	 end
       else
 	 sql = sql .." AND (IP_SRC_ADDR='"..host.."' OR IP_DST_ADDR='"..host.."')"
       end
@@ -182,126 +221,14 @@ function getNumFlows(interface_id, version, host, protocol, port, l7proto, info,
    end
 end
 
+-- ########################################################
 
-function getTopPeers(interface_id, version, host, protocol, port, l7proto, info, begin_epoch, end_epoch)
-   if(host == nil or host == "") then return {} end
-   if(version == nil) then version = 4 end
-
-   if(info == "") then info = nil end
-   if(l7proto == "") then l7proto = nil end
-   if(protocol == "") then protocol = nil end
-
-   sql = " SELECT "
-   if(version == 4) then
-      sql = sql.." CASE WHEN IP_SRC_ADDR = INET_ATON('"..host.."') THEN INET_NTOA(IP_DST_ADDR) ELSE INET_NTOA(IP_SRC_ADDR) END PEER_ADDR, "
-      -- when the selected host is the source, we consider its peer that is a destination an thus RECEIVES bytes and packets
-      -- similarly, when the selected host is the destination, we consider its peer as a source that SENDS bytes and packets
-      sql = sql.." CASE WHEN IP_SRC_ADDR = INET_ATON('"..host.."') THEN BYTES ELSE 0 END peer_cli_bytes, "
-      sql = sql.." CASE WHEN IP_DST_ADDR = INET_ATON('"..host.."') THEN BYTES ELSE 0 END peer_srv_bytes, "
-      sql = sql.." CASE WHEN IP_SRC_ADDR = INET_ATON('"..host.."') THEN PACKETS ELSE 0 END peer_cli_packets, "
-      sql = sql.." CASE WHEN IP_DST_ADDR = INET_ATON('"..host.."') THEN PACKETS ELSE 0 END peer_srv_packets, "
-   else
-      sql = sql.." CASE WHEN IP_SRC_ADDR = '"..host.."' THEN IP_DST_ADDR ELSE IP_SRC_ADDR END PEER_ADDR, "
-      sql = sql.." CASE WHEN IP_SRC_ADDR = '"..host.."' THEN BYTES ELSE 0 END peer_cli_bytes, "
-      sql = sql.." CASE WHEN IP_DST_ADDR = '"..host.."' THEN BYTES ELSE 0 END peer_srv_bytes, "
-      sql = sql.." CASE WHEN IP_SRC_ADDR = '"..host.."' THEN PACKETS ELSE 0 END peer_cli_packets, "
-      sql = sql.." CASE WHEN IP_DST_ADDR = '"..host.."' THEN PACKETS ELSE 0 END peer_srv_packets, "
-   end
-
-   sql = sql.."sum(peer_cli_bytes + peer_srv_bytes) as TOT_BYTES, sum(peer_cli_packets + peer_srv_packets) as TOT_PACKETS, "
-   sql = sql.."sum(peer_cli_bytes) as CLI_BYTES, sum(peer_cli_packets) as CLI_PACKETS, "
-   sql = sql.."sum(peer_srv_bytes) as SRV_BYTES, sum(peer_srv_packets) as SRV_PACKETS, "
-   sql = sql.."count(*) as TOT_FLOWS "
-   sql = sql.." FROM flowsv"..version
-
-   sql = sql.." WHERE FIRST_SWITCHED <= "..end_epoch.." and FIRST_SWITCHED >= "..begin_epoch
-   sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL)"
-   sql = sql.." AND (INTERFACE_ID='"..tonumber(interface_id).."')"
-
-   if((l7proto ~= nil) and (l7proto ~= "")) then sql = sql .." AND L7_PROTO="..l7proto end
-   if((protocol ~= nil) and (protocol ~= "")) then sql = sql .." AND PROTOCOL="..protocol end
-
-   if(info ~= nil) then sql = sql .." AND (INFO='"..info.."')" end
-
-   if((port ~= nil) and (port ~= "")) then sql = sql .." AND (L4_SRC_PORT="..port.." OR L4_DST_PORT="..port..")" end
-
-   if((host ~= nil) and (host ~= "")) then
-      if(version == 4) then
-    sql = sql .." AND (IP_SRC_ADDR=INET_ATON('"..host.."') OR IP_DST_ADDR=INET_ATON('"..host.."'))"
-      else
-    sql = sql .." AND (IP_SRC_ADDR='"..host.."' OR IP_DST_ADDR='"..host.."')"
-      end
-   end
-
-   -- we don't care about the order so we group by least and greatest
-   sql = sql.." group by PEER_ADDR "
-
-   sql = sql.." order by TOT_BYTES desc limit 10"
-
-   if(db_debug == true) then io.write(sql.."\n") end
-
-   res = interface.execSQLQuery(sql)
-   if(type(res) == "string") then
-      if(db_debug == true) then io.write(res.."\n") end
-      return {}
-   elseif res == nil then
-      return {}
-   else
-      return(res)
-   end
-end
-
-function getTopL7Protocols(interface_id, version, host, protocol, port, info, begin_epoch, end_epoch)
-   if(host == nil or host == "") then return {} end
-   if(version == nil) then version = 4 end
-
-   if(info == "") then info = nil end
-   if(protocol == "") then protocol = nil end
-
-   sql = " SELECT L7_PROTO, "
-   sql = sql.."sum(BYTES) as TOT_BYTES, sum(PACKETS) as TOT_PACKETS, count(*) as TOT_FLOWS "
-   sql = sql.." FROM flowsv"..version
-
-   sql = sql.." WHERE FIRST_SWITCHED <= "..end_epoch.." and FIRST_SWITCHED >= "..begin_epoch
-   sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL)"
-   sql = sql.." AND (INTERFACE_ID='"..tonumber(interface_id).."')"
-
-   if((protocol ~= nil) and (protocol ~= "")) then sql = sql .." AND PROTOCOL="..protocol end
-
-   if(info ~= nil) then sql = sql .." AND (INFO='"..info.."')" end
-
-   if((port ~= nil) and (port ~= "")) then sql = sql .." AND (L4_SRC_PORT="..port.." OR L4_DST_PORT="..port..")" end
-
-   if((host ~= nil) and (host ~= "")) then
-      if(version == 4) then
-    sql = sql .." AND (IP_SRC_ADDR=INET_ATON('"..host.."') OR IP_DST_ADDR=INET_ATON('"..host.."'))"
-      else
-    sql = sql .." AND (IP_SRC_ADDR='"..host.."' OR IP_DST_ADDR='"..host.."')"
-      end
-   end
-
-   -- we don't care about the order so we group by least and greatest
-   sql = sql.." group by L7_PROTO "
-
-   sql = sql.." order by TOT_BYTES desc limit 10"
-
-   if(db_debug == true) then io.write(sql.."\n") end
-
-   res = interface.execSQLQuery(sql)
-   if(type(res) == "string") then
-      if(db_debug == true) then io.write(res.."\n") end
-      return {}
-   elseif res == nil then
-      return {}
-   else
-      return(res)
-   end
-end
-
-function getOverallTopTalkersSELECT_FROM_WHERE_clause(src_or_dst, v4_or_v6, begin_epoch, end_epoch, ifid, l4proto, port)
+function getOverallTopTalkersSELECT_FROM_WHERE_clause(src_or_dst, v4_or_v6, begin_epoch, end_epoch, ifid, l4proto, port, vlan, profile)
    local sql = ""
    local sql_bytes_packets = "PACKETS as packets, "
+   local exclude_same_src_dst = false
    if src_or_dst     == "IP_DST_ADDR" then
+      exclude_same_src_dst = true
       -- if this is a destination address, we account it INGRESS traffic
       sql_bytes_packets = sql_bytes_packets .. "OUT_BYTES as bytes_sent, IN_BYTES  as bytes_rcvd, "
    elseif src_or_dst == "IP_SRC_ADDR" then
@@ -314,27 +241,41 @@ function getOverallTopTalkersSELECT_FROM_WHERE_clause(src_or_dst, v4_or_v6, begi
    if v4_or_v6 == 6 then
       sql = " SELECT NULL addrv4, "..src_or_dst.." addrv6, "
       sql = sql..sql_bytes_packets
-      sql = sql.."FIRST_SWITCHED, LAST_SWITCHED FROM flowsv6 "
+      sql = sql.."FIRST_SWITCHED, LAST_SWITCHED FROM "..flowsTableName(6).." "
    elseif v4_or_v6 == 4 then -- ipv4
       sql = " SELECT "..src_or_dst.." addrv4, NULL addrv6, "
       sql = sql..sql_bytes_packets
-      sql = sql.."FIRST_SWITCHED, LAST_SWITCHED FROM flowsv4 "
+      sql = sql.."FIRST_SWITCHED, LAST_SWITCHED FROM "..flowsTableName(4).." "
    else
       sql = ""
    end
    sql = sql.." WHERE FIRST_SWITCHED <= "..end_epoch.." and FIRST_SWITCHED >= "..begin_epoch
-   sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL) "
+   sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL OR NTOPNG_INSTANCE_NAME='') "
    sql = sql.." AND (INTERFACE_ID='"..tonumber(ifid).."') "
+   if(exclude_same_src_dst == true) then
+      -- avoid double counting flows that have source == destination
+      sql = sql.." AND IP_SRC_ADDR <> IP_DST_ADDR"
+   end
    if((l4proto ~= nil) and (l4proto ~= "") and (l4proto ~= "-1")) then
       sql = sql .." AND PROTOCOL="..l4proto
    end
    if((port ~= nil) and (port ~= "")) then
       sql = sql .." AND (L4_SRC_PORT="..port.." OR L4_DST_PORT="..port..")"
    end
+
+   if((vlan ~= nil) and (vlan ~= "")) then
+      sql = sql .." AND VLAN_ID="..vlan
+   end
+
+   if((profile ~= nil) and (profile ~= "")) then
+      sql = sql .." AND PROFILE='"..profile.."'"
+   end
    return sql..'\n'
 end
 
-function getOverallTopTalkers(interface_id, l4proto, port, info, begin_epoch, end_epoch, sort_column, sort_order, offset, limit)
+-- ########################################################
+
+function getOverallTopTalkers(interface_id, l4proto, port, vlan, profile, info, begin_epoch, end_epoch, sort_column, sort_order, offset, limit)
    -- retrieves top talkers in the given time range
 
    if(info == "") then info = nil end
@@ -349,13 +290,13 @@ function getOverallTopTalkers(interface_id, l4proto, port, info, begin_epoch, en
    sql = sql.." FROM "
 
    sql = sql.."("
-   sql = sql..getOverallTopTalkersSELECT_FROM_WHERE_clause('IP_SRC_ADDR', 4, begin_epoch, end_epoch, interface_id, l4proto, port)
+   sql = sql..getOverallTopTalkersSELECT_FROM_WHERE_clause('IP_SRC_ADDR', 4, begin_epoch, end_epoch, interface_id, l4proto, port, vlan, profile)
    sql = sql.." UNION ALL "
-   sql = sql..getOverallTopTalkersSELECT_FROM_WHERE_clause('IP_DST_ADDR', 4, begin_epoch, end_epoch, interface_id, l4proto, port)
+   sql = sql..getOverallTopTalkersSELECT_FROM_WHERE_clause('IP_DST_ADDR', 4, begin_epoch, end_epoch, interface_id, l4proto, port, vlan, profile)
    sql = sql.." UNION ALL "
-   sql = sql..getOverallTopTalkersSELECT_FROM_WHERE_clause('IP_SRC_ADDR', 6, begin_epoch, end_epoch, interface_id, l4proto, port)
+   sql = sql..getOverallTopTalkersSELECT_FROM_WHERE_clause('IP_SRC_ADDR', 6, begin_epoch, end_epoch, interface_id, l4proto, port, vlan, profile)
    sql = sql.." UNION ALL "
-   sql = sql..getOverallTopTalkersSELECT_FROM_WHERE_clause('IP_DST_ADDR', 6, begin_epoch, end_epoch, interface_id, l4proto, port)
+   sql = sql..getOverallTopTalkersSELECT_FROM_WHERE_clause('IP_DST_ADDR', 6, begin_epoch, end_epoch, interface_id, l4proto, port, vlan, profile)
    sql = sql..") talkers"
    sql = sql.." group by addr "
 
@@ -379,7 +320,7 @@ function getOverallTopTalkers(interface_id, l4proto, port, info, begin_epoch, en
 
    -- SLICE
    local slice_offset = 0
-   local sclice_limit = 100
+   local slice_limit = 100
    if tonumber(offset) >= 0 then slice_offset = offset end
    if tonumber(limit) > 0 then slice_limit = limit end
    sql = sql.."limit "..slice_offset..","..slice_limit.." "
@@ -397,8 +338,9 @@ function getOverallTopTalkers(interface_id, l4proto, port, info, begin_epoch, en
    end
 end
 
+-- ########################################################
 
-function getHostTopTalkers(interface_id, host, l7_proto_id, l4_proto_id, port, info, begin_epoch, end_epoch, sort_column, sort_order, offset, limit)
+function getHostTopTalkers(interface_id, host, l7_proto_id, l4_proto_id, port, vlan, profile, info, begin_epoch, end_epoch, sort_column, sort_order, offset, limit)
    -- obtains host top talkers, possibly restricting the range only to l7_proto_id
    if host == nil or host == "" then return {} end
 
@@ -423,20 +365,28 @@ function getHostTopTalkers(interface_id, host, l7_proto_id, l4_proto_id, port, i
    else
       sql = sql.." CASE WHEN IP_SRC_ADDR = '"..host.."' THEN IP_DST_ADDR ELSE IP_SRC_ADDR END addr, "
       sql = sql.." CASE WHEN IP_SRC_ADDR = '"..host.."' THEN OUT_BYTES ELSE IN_BYTES  END peer_bytes_sent, "
-      sql = sql.." CASE WHEN IP_SRC_ADDR = '"..host.."' THEN IN_BYTES  ELSE OUT_BUTES END peer_bytes_rcvd, "
+      sql = sql.." CASE WHEN IP_SRC_ADDR = '"..host.."' THEN IN_BYTES  ELSE OUT_BYTES END peer_bytes_rcvd, "
    end
    sql = sql.." FIRST_SWITCHED, LAST_SWITCHED "
-   sql = sql.." FROM flowsv"..version
+   sql = sql.." FROM "..flowsTableName(version)
 
    sql = sql.." WHERE FIRST_SWITCHED <= "..end_epoch.." and FIRST_SWITCHED >= "..begin_epoch
-   sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL)"
+   sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL OR NTOPNG_INSTANCE_NAME='')"
    sql = sql.." AND (INTERFACE_ID='"..tonumber(interface_id).."')"
 
    if((port ~= nil) and (port ~= "")) then
       sql = sql .." AND (L4_SRC_PORT="..port.." OR L4_DST_PORT="..port..")"
    end
+
+   if((vlan ~= nil) and (vlan ~= "")) then
+      sql = sql .." AND VLAN_ID="..vlan
+   end
+
+   if((profile ~= nil) and (profile ~= "")) then
+      sql = sql .." AND PROFILE='"..profile.."'"
+   end
    if(info ~= nil) then
-      sql = sql .." AND (INFO='"..info.."')"
+      sql = sql .." AND (INFO LIKE '%"..info.."%')"
    end
 
    if l7_proto_id and l7_proto_id ~="" then sql = sql.." AND L7_PROTO = "..tonumber(l7_proto_id) end
@@ -475,7 +425,7 @@ function getHostTopTalkers(interface_id, host, l7_proto_id, l4_proto_id, port, i
 
    -- SLICE
    local slice_offset = 0
-   local sclice_limit = 100
+   local slice_limit = 100
    if tonumber(offset) >= 0 then slice_offset = offset end
    if tonumber(limit) > 0 then slice_limit = limit end
    sql = sql.."limit "..slice_offset..","..slice_limit.." "
@@ -494,11 +444,14 @@ function getHostTopTalkers(interface_id, host, l7_proto_id, l4_proto_id, port, i
    end
 end
 
-function getAppTopTalkersSELECT_FROM_WHERE_clause(src_or_dst, v4_or_v6, begin_epoch, end_epoch, ifid, l7_proto_id, l4_proto_id, port)
+-- ########################################################
+
+function getAppTopTalkersSELECT_FROM_WHERE_clause(src_or_dst, v4_or_v6, begin_epoch, end_epoch, ifid, l7_proto_id, l4_proto_id, port, vlan, profile)
    local sql = ""
    local sql_bytes_packets = "PACKETS as packets, "
-
+   local exclude_same_src_dst = false
    if src_or_dst     == "IP_DST_ADDR" then
+      exclude_same_src_dst = true
       -- if this is a destination address, we account it INGRESS traffic
       sql_bytes_packets = sql_bytes_packets .. "OUT_BYTES as bytes_sent, IN_BYTES  as bytes_rcvd, "
    elseif src_or_dst == "IP_SRC_ADDR" then
@@ -511,29 +464,43 @@ function getAppTopTalkersSELECT_FROM_WHERE_clause(src_or_dst, v4_or_v6, begin_ep
    if v4_or_v6 == 6 then
       sql = " SELECT NULL addrv4, "..src_or_dst.." addrv6, "
       sql = sql..sql_bytes_packets
-      sql = sql.."FIRST_SWITCHED, LAST_SWITCHED FROM flowsv6 "
+      sql = sql.."FIRST_SWITCHED, LAST_SWITCHED FROM "..flowsTableName(6).." "
    elseif v4_or_v6 == 4 then -- ipv4
       sql = " SELECT "..src_or_dst.." addrv4, NULL addrv6, "
       sql = sql..sql_bytes_packets
-      sql = sql.."FIRST_SWITCHED, LAST_SWITCHED FROM flowsv4 "
+      sql = sql.."FIRST_SWITCHED, LAST_SWITCHED FROM "..flowsTableName(4).." "
    else
       sql = ""
    end
 
    sql = sql.." WHERE FIRST_SWITCHED <= "..end_epoch.." and FIRST_SWITCHED >= "..begin_epoch
-   sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL) "
+   sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL OR NTOPNG_INSTANCE_NAME='') "
    sql = sql.." AND (INTERFACE_ID='"..tonumber(ifid).."') "
    sql = sql.." AND L7_PROTO = "..tonumber(l7_proto_id)
+   if(exclude_same_src_dst == true) then
+      -- avoid double counting flows that have source == destination
+      sql = sql.." AND IP_SRC_ADDR <> IP_DST_ADDR"
+   end
    if((l4_proto_id ~= nil) and (l4_proto_id ~= "") and (l4_proto_id ~= "-1")) then
       sql = sql .." AND PROTOCOL="..l4_proto_id
    end
    if((port ~= nil) and (port ~= "")) then
       sql = sql .." AND (L4_SRC_PORT="..port.." OR L4_DST_PORT="..port..")"
    end
+
+   if((vlan ~= nil) and (vlan ~= "")) then
+      sql = sql .." AND VLAN_ID="..vlan
+   end
+
+   if((profile ~= nil) and (profile ~= "")) then
+      sql = sql .." AND PROFILE='"..profile.."'"
+   end
    return sql..'\n'
 end
 
-function getAppTopTalkers(interface_id, l7_proto_id, l4_proto_id, port, info, begin_epoch, end_epoch, sort_column, sort_order, offset, limit)
+-- ########################################################
+
+function getAppTopTalkers(interface_id, l7_proto_id, l4_proto_id, port, vlan, profile, info, begin_epoch, end_epoch, sort_column, sort_order, offset, limit)
    -- retrieves top talkers in the given time range
    if(info == "") then info = nil end
 
@@ -547,13 +514,13 @@ function getAppTopTalkers(interface_id, l7_proto_id, l4_proto_id, port, info, be
    sql = sql.." FROM "
 
    sql = sql.."("
-   sql = sql..getAppTopTalkersSELECT_FROM_WHERE_clause('IP_SRC_ADDR', 4, begin_epoch, end_epoch, interface_id, l7_proto_id, l4_proto_id, port)
+   sql = sql..getAppTopTalkersSELECT_FROM_WHERE_clause('IP_SRC_ADDR', 4, begin_epoch, end_epoch, interface_id, l7_proto_id, l4_proto_id, port, vlan, profile)
    sql = sql.." UNION ALL "
-   sql = sql..getAppTopTalkersSELECT_FROM_WHERE_clause('IP_DST_ADDR', 4, begin_epoch, end_epoch, interface_id, l7_proto_id, l4_proto_id, port)
+   sql = sql..getAppTopTalkersSELECT_FROM_WHERE_clause('IP_DST_ADDR', 4, begin_epoch, end_epoch, interface_id, l7_proto_id, l4_proto_id, port, vlan, profile)
    sql = sql.." UNION ALL "
-   sql = sql..getAppTopTalkersSELECT_FROM_WHERE_clause('IP_SRC_ADDR', 6, begin_epoch, end_epoch, interface_id, l7_proto_id, l4_proto_id, port)
+   sql = sql..getAppTopTalkersSELECT_FROM_WHERE_clause('IP_SRC_ADDR', 6, begin_epoch, end_epoch, interface_id, l7_proto_id, l4_proto_id, port, vlan, profile)
    sql = sql.." UNION ALL "
-   sql = sql..getAppTopTalkersSELECT_FROM_WHERE_clause('IP_DST_ADDR', 6, begin_epoch, end_epoch, interface_id, l7_proto_id, l4_proto_id, port)
+   sql = sql..getAppTopTalkersSELECT_FROM_WHERE_clause('IP_DST_ADDR', 6, begin_epoch, end_epoch, interface_id, l7_proto_id, l4_proto_id, port, vlan, profile)
    sql = sql..") talkers"
    sql = sql.." group by addr "
 
@@ -577,7 +544,7 @@ function getAppTopTalkers(interface_id, l7_proto_id, l4_proto_id, port, info, be
 
    -- SLICE
    local slice_offset = 0
-   local sclice_limit = 100
+   local slice_limit = 100
    if tonumber(offset) >= 0 then slice_offset = offset end
    if tonumber(limit) > 0 then slice_limit = limit end
    sql = sql.."limit "..slice_offset..","..slice_limit.." "
@@ -595,7 +562,9 @@ function getAppTopTalkers(interface_id, l7_proto_id, l4_proto_id, port, info, be
    end
 end
 
-function getTopApplications(interface_id, peer1, peer2, l7_proto_id, l4_proto_id, port, info, begin_epoch, end_epoch, sort_column, sort_order, offset, limit)
+-- ########################################################
+
+function getTopApplications(interface_id, peer1, peer2, l7_proto_id, l4_proto_id, port, vlan, profile, info, begin_epoch, end_epoch, sort_column, sort_order, offset, limit)
    -- if both peers are nil, top applications are overall in the time range
    -- if peer1 is nil nad peer2 is not nil, then top apps are for peer1
    -- if peer2 is nil nad peer1 is not nil, then top apps are for peer2
@@ -610,19 +579,27 @@ function getTopApplications(interface_id, peer1, peer2, l7_proto_id, l4_proto_id
    sql = " SELECT L7_PROTO application, "
    sql = sql.."sum(IN_BYTES + OUT_BYTES) as tot_bytes, sum(PACKETS) as tot_packets, count(*) as tot_flows "
    -- sql = sql.." (sum(LAST_SWITCHED) - sum(FIRST_SWITCHED)) / count(*) as avg_flow_duration "
-   sql = sql.." FROM flowsv"..version
+   sql = sql.." FROM "..flowsTableName(version)
 
    sql = sql.." WHERE FIRST_SWITCHED <= "..end_epoch.." and FIRST_SWITCHED >= "..begin_epoch
-   sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL)"
+   sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL OR NTOPNG_INSTANCE_NAME='')"
    sql = sql.." AND (INTERFACE_ID='"..tonumber(interface_id).."')"
 
-
    if((port ~= nil) and (port ~= "")) then
       sql = sql .." AND (L4_SRC_PORT="..port.." OR L4_DST_PORT="..port..")"
    end
+
+   if((vlan ~= nil) and (vlan ~= "")) then
+      sql = sql .." AND VLAN_ID="..vlan
+   end
+
+   if((profile ~= nil) and (profile ~= "")) then
+      sql = sql .." AND PROFILE='"..profile.."'"
+   end
+
    if l7_proto_id and l7_proto_id ~="" then sql = sql.." AND L7_PROTO = "..tonumber(l7_proto_id) end
    if l4_proto_id and l4_proto_id ~="" then sql = sql.." AND PROTOCOL = "..tonumber(l4_proto_id) end
-   if(info ~= nil) then sql = sql .." AND (INFO='"..info.."')" end
+   if(info ~= nil) then sql = sql .." AND (INFO LIKE '%"..info.."%')" end
 
    if peer1 and peer1 ~= "" then
       if(version == 4) then
@@ -660,7 +637,7 @@ function getTopApplications(interface_id, peer1, peer2, l7_proto_id, l4_proto_id
 
    -- SLICE
    local slice_offset = 0
-   local sclice_limit = 100
+   local slice_limit = 100
    if tonumber(offset) >= 0 then slice_offset = offset end
    if tonumber(limit) > 0 then slice_limit = limit end
    sql = sql.."limit "..slice_offset..","..slice_limit.." "
@@ -678,57 +655,24 @@ function getTopApplications(interface_id, peer1, peer2, l7_proto_id, l4_proto_id
    end
 end
 
+-- ########################################################
 
-function getPeersTrafficHistogram(interface_id, peer1, peer2, info, begin_epoch, end_epoch)
-   if peer1 == nil or peer1 == "" or peer2 == nil or peer2 == "" then return {} end
+function db_utils.harverstExpiredMySQLFlows(ifname, mysql_retention, verbose)
+   interface.select(ifname)
 
-   local max_bins = 2000  -- do not return more than 2k datapoints
-   local interval = end_epoch - begin_epoch  -- the larger the interval the coarser the aggregation
-   local bin_width = math.floor(interval / max_bins)
-
-   local version = 4
-   if isIPv6(peer1) then version = 6 end
-   if(info == "") then info = nil end
-
-   if(version == 4) then
-      sql = " SELECT INET_NTOA(least(IP_SRC_ADDR, IP_DST_ADDR)) peer1_addr, INET_NTOA(greatest(IP_SRC_ADDR, IP_DST_ADDR)) peer2_addr, "
-   else
-      sql = " SELECT least(IP_SRC_ADDR, IP_DST_ADDR) peer1_addr, greatest(IP_SRC_ADDR, IP_DST_ADDR) peer2_addr, "
-   end
-   sql = sql.." MIN(FIRST_SWITCHED) first_switched_bin, " -- the oldest datapoint in each bin
-   sql = sql.." sum(BYTES) as tot_bytes, sum(PACKETS) as tot_packets, count(*) as tot_flows, "
-   sql = sql.." (sum(FIRST_SWITCHED) - sum(LAST_SWITCHED)) / count(*) as avg_flow_duration "
-
-   sql = sql.." FROM flowsv"..version
-
-   sql = sql.." WHERE FIRST_SWITCHED <= "..end_epoch.." and FIRST_SWITCHED >= "..begin_epoch
-   sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."'OR NTOPNG_INSTANCE_NAME IS NULL)"
-   sql = sql.." AND (INTERFACE_ID='"..tonumber(interface_id).."')"
-
-   if(info ~= nil) then sql = sql .." AND (INFO='"..info.."')" end
-
-   if(version == 4) then
-      sql = sql .." AND (IP_SRC_ADDR=INET_ATON('"..peer1.."') OR IP_DST_ADDR=INET_ATON('"..peer2.."'))"
-      sql = sql .." AND (IP_SRC_ADDR=INET_ATON('"..peer2.."') OR IP_DST_ADDR=INET_ATON('"..peer2.."'))"
-   else
-      sql = sql .." AND (IP_SRC_ADDR='"..peer1.."' OR IP_DST_ADDR='"..peer2.."')"
-      sql = sql .." AND (IP_SRC_ADDR='"..peer2.."' OR IP_DST_ADDR='"..peer2.."')"
+   local dbtables = {"flowsv4", "flowsv6"}
+   if useAggregatedFlows() then
+      dbtables[#dbtables+1] = "aggrflowsv4"
+      dbtables[#dbtables+1] = "aggrflowsv6"
    end
 
-   -- we don't care about the order so we group by least and greatest
-   sql = sql.." group by least(IP_SRC_ADDR, IP_DST_ADDR), greatest(IP_SRC_ADDR, IP_DST_ADDR), FIRST_SWITCHED DIV ("..bin_width..")"
-
-   sql = sql.." order by TOT_BYTES desc limit 100"
-
-   if(db_debug == true) then io.write(sql.."\n") end
-
-   res = interface.execSQLQuery(sql)
-   if(type(res) == "string") then
-      if(db_debug == true) then io.write(res.."\n") end
-      return {}
-   elseif res == nil then
-      return {}
-   else
-      return(res)
+   for _, tb in pairs(dbtables) do
+      local sql = "DELETE FROM "..tb.." where FIRST_SWITCHED < "..mysql_retention
+      sql = sql.." AND (INTERFACE_ID = "..getInterfaceId(ifname)..")"
+      sql = sql.." AND (NTOPNG_INSTANCE_NAME='"..ntop.getPrefs()["instance_name"].."' OR NTOPNG_INSTANCE_NAME IS NULL OR NTOPNG_INSTANCE_NAME='')"
+      interface.execSQLQuery(sql)
+      if(verbose) then io.write(sql.."\n") end
    end
 end
+
+return db_utils
