@@ -1,12 +1,12 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-17 - ntop.org
 --
 
 dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
-
+local os_utils = require "os_utils"
 
 function cleanName(name)
    n = string.gsub(name, "'", "_")
@@ -46,7 +46,7 @@ end
 
 -- #########################
 
-sendHTTPHeader('text/html; charset=iso-8859-1')
+sendHTTPContentTypeHeader('text/html')
 
 begin = os.clock()
 t = os.time() -- -86400
@@ -58,7 +58,7 @@ key_name = when..".keys"
 local debug = true
 local delete_keys = true
 
-dump_dir = fixPath(dirs.workingdir .. "/datadump/")
+dump_dir = os_utils.fixPath(dirs.workingdir .. "/datadump/")
 ntop.mkdir(dump_dir)
 
 fname = dump_dir .. "20".. when
