@@ -1,5 +1,5 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-17 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -11,6 +11,7 @@ sendHTTPHeader('application/json')
 
 interface.select(ifname)
 flows_stats = interface.getFlowsInfo()
+flows_stats = flows_stats["flows"]
 
 procs = {}
 hosts = {}
@@ -24,7 +25,7 @@ for key, value in pairs(flows_stats) do
    if(flow["cli.host"] ~= nil) then 
       c_sym = flow["cli.host"] 
    else
-      c_sym = ntop.getResolvedAddress(flow["cli.ip"])
+      c_sym = getResolvedAddress(hostkey2hostinfo(flow["cli.ip"]))
 
       if(c_sym ~= flow["cli.ip"]) then
 	 c_sym = c_sym .." (".. flow["cli.ip"] ..")"
@@ -35,7 +36,7 @@ for key, value in pairs(flows_stats) do
    if(flow["srv.host"] ~= nil) then 
       s_sym = flow["srv.host"] 
    else
-      s_sym = ntop.getResolvedAddress(flow["srv.ip"])
+      s_sym = getResolvedAddress(hostkey2hostinfo(flow["srv.ip"]))
       
       if(s_sym ~= flow["srv.ip"]) then
 	 s_sym = s_sym .." (".. flow["srv.ip"] .. ")" 
@@ -111,7 +112,7 @@ for key, value in pairs(hosts) do
        m = m + 1
        v = _v[1]
        -- Process
-       link = ntop.getHttpPrefix().."/lua/get_process_info.lua?pid="..v["pid"].."&name="..k.."&host=".. key .."&page=Flows"
+       link = ntop.getHttpPrefix().."/lua/get_process_info.lua?pid="..v["pid"].."&pid_name="..k.."&host=".. key .."&page=Flows"
        print('\n\t\t{ "name": "'..k..' (pid '.. v["pid"]..')", "link": "'.. link ..'", "type": "proc", "children": [ ')
        o = 0
        for peer,_ in pairs(_v[2]) do
@@ -128,7 +129,7 @@ for key, value in pairs(hosts) do
 	     if(o > 0) then print(",") end
 	     o = o + 1
 	     link = ntop.getHttpPrefix().."/lua/host_details.lua?host="..pid .."&page=flows"
-	     link = ntop.getHttpPrefix().."/lua/get_process_info.lua?pid="..pid.."&name="..k.."&host=".. key .."&page=Flows"
+	     link = ntop.getHttpPrefix().."/lua/get_process_info.lua?pid="..pid.."&pid_name="..k.."&host=".. key .."&page=Flows"
 	     print('\n\t\t\t{ "name": "'..p_val["name"]..' (pid '.. pid..')", "link": "'.. link ..'", "type": "proc", "children": [ ] } ')
 	  end
        end
