@@ -67,7 +67,7 @@ else
 	 
 	 if(add_init or (proc.father_pid ~= 1)) then
 	    -- No link for father
-	    -- link = ntop.getHttpPrefix().."/lua/get_process_info.lua?pid="..proc.father_pid.."&name="..proc.father_name.."&host=".. host .."&page=flows"
+	    -- link = ntop.getHttpPrefix().."/lua/get_process_info.lua?pid="..proc.father_pid.."&pid_name="..proc.father_name.."&host=".. host .."&page=flows"
 	    nest2tab(nest)
 	    if(add_comma) then print(',') add_comma = false end
 	    print(' { "name": "'..proc.father_name..' (pid '.. proc.father_pid..')", "type": "proc", "children": [ ')
@@ -75,7 +75,7 @@ else
 	 end
       end
 
-      link = ntop.getHttpPrefix().."/lua/get_process_info.lua?pid="..proc.pid.."&name="..proc.name.."&host=".. host .."&page=Flows"
+      link = ntop.getHttpPrefix().."/lua/get_process_info.lua?pid="..proc.pid.."&pid_name="..proc.name.."&host=".. host .."&page=Flows"
       nest2tab(nest)
       if(add_comma) then print(',') add_comma = false end
       print('{ "name": "'..proc.name..' (pid '.. proc.pid..')", "link": "'.. link ..'", "type": "proc", "children": [ ] }')
@@ -106,12 +106,12 @@ else
       -- nest = nest + 1
       nest = displayProc(nest, flow.client_process, 
 			 flow["cli.ip"], 
-			 flowinfo2hostname(flow, "cli", flow["vlan"]),
+			 flowinfo2hostname(flow, "cli"),
 			 true,  true, true, true, last, false)
       
       displayProc(nest, flow.server_process, 
 		  flow["srv.ip"], 
-		  flowinfo2hostname(flow, "srv", flow["vlan"]),
+		  flowinfo2hostname(flow, "srv"),
 		  (flow["cli.ip"] ~= flow["srv.ip"]), 
 		  ((flow.client_process.father_pid ~= flow.server_process.father_pid) or (flow["cli.ip"] ~= flow["srv.ip"])),
 		  false,
@@ -121,11 +121,11 @@ else
       end
    elseif(flow.client_process ~= nil) then
       nest = displayProc(nest, flow.client_process, 
-			 flow["cli.ip"], flowinfo2hostname(flow, "cli", flow["vlan"]),
+			 flow["cli.ip"], flowinfo2hostname(flow, "cli"),
 			 true, true, true, true, true, false)
    elseif(flow.server_process ~= nil) then
       nest = displayProc(nest, flow.server_process,
-			 flow["srv.ip"], flowinfo2hostname(flow, "srv", flow["vlan"]),
+			 flow["srv.ip"], flowinfo2hostname(flow, "srv"),
 			 true, true, true, true, true, false)
    end
 end
