@@ -8,7 +8,7 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 require "lua_utils"
 
 if(mode ~= "embed") then
-   sendHTTPHeader('text/html; charset=iso-8859-1')
+   sendHTTPContentTypeHeader('text/html')
    ntop.dumpFile(dirs.installdir .. "/httpdocs/inc/header.inc")
    active_page = "hosts"
    dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
@@ -104,13 +104,6 @@ function getJSONData(url, params, error_message) {
   return jsonData;
 }
 
-function readableBytes(bytes) {
-  var sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB'];
-  if (bytes == 0) return '0 Bytes';
-  var i = parseInt(Math.floor(Math.log(bytes) / Math.log(1024)));
-  return (bytes / Math.pow(1024, i)).toFixed(2) + ' ' + sizes[i];
-};
-
 var colorGen = d3.scale.category20();
 var explode_process = '';
 
@@ -203,7 +196,7 @@ print [[/lua/sprobe_hosts_interactions_data.lua");
         label: nodes[node_id].name,
         ip: nodes[node_id].system_id,
         ondoubleclick: function(n) {
-	  //TODO /lua/sprobe_host_process.lua?host=" + link.source + "&id=" + link.source_system_id + "&name="+link.source_name
+	  //TODO /lua/sprobe_host_process.lua?host=" + link.source + "&pid=" + link.source_system_id + "&pid_name="+link.source_name
           //refreshGraph();
         },
         image: image
@@ -236,7 +229,7 @@ print [[/lua/sprobe_hosts_interactions_data.lua");
       {
         color: color,
         weight: getWeight(link.cli2srv_bytes),
-        label: readableBytes(link.cli2srv_bytes)
+        label: bytesToSize(link.cli2srv_bytes)
       }
     );
     graph.newEdge(
@@ -245,7 +238,7 @@ print [[/lua/sprobe_hosts_interactions_data.lua");
       {
         color: color,
         weight: getWeight(link.srv2cli_bytes),
-        label: readableBytes(link.srv2cli_bytes)
+        label: bytesToSize(link.srv2cli_bytes)
       }
     );
   });
