@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,32 +25,39 @@
 
 AddressResolution::AddressResolution() {
   num_resolved_addresses = num_resolved_fails = 0;
+  num_resolvers =
+#ifdef NTOPNG_EMBEDDED_EDITION
+      1
+#else
+      CONST_NUM_RESOLVERS
+#endif
+      ;
+
+  if(!(resolveThreadLoop = (pthread_t*)calloc(num_resolvers, sizeof(pthread_t))))
+    throw 2;
 }
 
 /* ******************************************* */
 
 /* Format: 131.114.21.0/24,10.0.0.0/255.0.0.0 */
-bool AddressResolution::setLocalNetworks(char *rule) {
-  char *net = strtok(rule, ",");
-
-  while(net != NULL) {
-    int16_t rc;
-
-    if((rc = localNetworks.addAddress(net)) < 0) return false;
-    net = strtok(NULL, ",");
-  }
-  return true;
-}
+bool AddressResolution::setLocalNetworks(char *rule) { return(localNetworks.addAddresses(rule)); }
 
 /* ******************************************* */
 
-int16_t AddressResolution::findAddress(int family, void *addr) {
-  return(localNetworks.findAddress(family, addr));
+int16_t AddressResolution::findAddress(int family, void *addr, u_int8_t *network_mask_bits) {
+  return(localNetworks.findAddress(family, addr, network_mask_bits));
 }
 
 /* **************************************** */
 
 AddressResolution::~AddressResolution() {
+  if(ntop->getPrefs()->is_dns_resolution_enabled()) {
+    for(int i = 0; i < num_resolvers; i++)
+      pthread_join(resolveThreadLoop[i], NULL);
+  }
+
+  free(resolveThreadLoop);
+
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "Address resolution stats [%u resolved][%u failures]",
 			       num_resolved_addresses, num_resolved_fails);
 }
@@ -62,7 +69,7 @@ void AddressResolution::resolveHostName(char *_numeric_ip, char *symbolic, u_int
   u_int numeric_ip_len;
 
   snprintf(query, sizeof(query), "%s", _numeric_ip);
-  if((at = strchr(query, '@')) != '\0') at[0] = '\0';
+  if((at = strchr(query, '@')) != NULL) at[0] = '\0';
   numeric_ip = query;
   numeric_ip_len = strlen(numeric_ip)-1;
 
@@ -140,6 +147,8 @@ void AddressResolution::resolveHostName(char *_numeric_ip, char *symbolic, u_int
 static void* resolveLoop(void* ptr) {
   AddressResolution *a = (AddressResolution*)ptr;
   Redis *r = ntop->getRedis();
+  u_int no_resolution_loops = 0;
+  const u_int max_num_idle_loops = 3;
 
   while(!ntop->getGlobals()->isShutdown()) {
     char numeric_ip[64];
@@ -148,8 +157,12 @@ static void* resolveLoop(void* ptr) {
     if(rc == 0) {
       if(numeric_ip[0] != '\0')
 	a->resolveHostName(numeric_ip);
-    } else
-      sleep(1);    
+
+      no_resolution_loops = 0;
+    } else {
+      if(no_resolution_loops < max_num_idle_loops) no_resolution_loops++;
+      sleep(no_resolution_loops);
+    }
   }
 
   return(NULL);
@@ -159,16 +172,9 @@ static void* resolveLoop(void* ptr) {
 
 void AddressResolution::startResolveAddressLoop() {
   if(ntop->getPrefs()->is_dns_resolution_enabled()) {
-    int num_resolvers =
-#ifdef NTOPNG_EMBEDDED_EDITION
-      1
-#else
-      CONST_NUM_RESOLVERS
-#endif
-      ;
 
-    for(int i=0; i<num_resolvers; i++)
-      pthread_create(&resolveThreadLoop, NULL, resolveLoop, (void*)this);
+    for(int i = 0; i < num_resolvers; i++)
+      pthread_create(&resolveThreadLoop[i], NULL, resolveLoop, (void*)this);
   }
 }
 
