@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,21 +21,29 @@
 
 #include "ntop_includes.h"
 
+#ifndef HAVE_NEDGE
+
 /* **************************************************** */
 
 CollectorInterface::CollectorInterface(const char *_endpoint) : ParserInterface(_endpoint) {
-  char *tmp, *e;
-  const char *topics[] = { "flow", "event", NULL };
+  char *tmp, *e, *t;
+  const char *topics[] = { "flow", "event", "counter", NULL };
 
-  num_drops = 0, num_subscribers = 0;
+  memset(&recvStats, 0, sizeof(recvStats));
+  num_subscribers = 0;
 
   context = zmq_ctx_new();
 
   if((tmp = strdup(_endpoint)) == NULL) throw("Out of memory");
 
-  e = strtok(tmp, ",");
+  is_collector = false;
+  
+  e = strtok_r(tmp, ",", &t);
   while(e != NULL) {
-    if(num_subscribers == CONST_MAX_NUM_ZMQ_SUBSCRIBERS) {
+    int l = strlen(e)-1, val;
+    char last_char = e[l];
+
+    if(num_subscribers == MAX_ZMQ_SUBSCRIBERS) {
       ntop->getTrace()->traceEvent(TRACE_ERROR,
 				   "Too many endpoints defined %u: skipping those in excess",
 				   num_subscribers);
@@ -44,11 +52,18 @@ CollectorInterface::CollectorInterface(const char *_endpoint) : ParserInterface(
 
     subscriber[num_subscribers].socket = zmq_socket(context, ZMQ_SUB);
 
-    if(ntop->getPrefs()->is_zmq_collector_mode()) {     
+    val = 131072;
+    if(zmq_setsockopt(subscriber[num_subscribers].socket, ZMQ_RCVBUF, &val, sizeof(val)) != 0)
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to enlarge ZMQ buffer size");
+
+    if(last_char == 'c')
+      is_collector = true, e[l] = '\0';
+
+    if(is_collector) {
       if(zmq_bind(subscriber[num_subscribers].socket, e) != 0) {
 	zmq_close(subscriber[num_subscribers].socket);
 	zmq_ctx_destroy(context);
-	ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to bind to ZMQ endpoint %s", e);
+	ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to bind to ZMQ endpoint %s [collector]", e);
 	free(tmp);
 	throw("Unable to bind to the specified ZMQ endpoint");
       }
@@ -56,7 +71,7 @@ CollectorInterface::CollectorInterface(const char *_endpoint) : ParserInterface(
       if(zmq_connect(subscriber[num_subscribers].socket, e) != 0) {
 	zmq_close(subscriber[num_subscribers].socket);
 	zmq_ctx_destroy(context);
-	ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to connect to ZMQ endpoint %s", e);
+	ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to connect to ZMQ endpoint %s [probe]", e);
 	free(tmp);
 	throw("Unable to connect to the specified ZMQ endpoint");
       }
@@ -75,7 +90,8 @@ CollectorInterface::CollectorInterface(const char *_endpoint) : ParserInterface(
     subscriber[num_subscribers].endpoint = strdup(e);
 
     num_subscribers++;
-    e = strtok(NULL, ",");
+
+    e = strtok_r(NULL, ",", &t);
   }
 
   free(tmp);
@@ -98,13 +114,12 @@ void CollectorInterface::collect_flows() {
   struct zmq_msg_hdr h;
   char payload[8192];
   u_int payload_len = sizeof(payload)-1;
-  zmq_pollitem_t items[CONST_MAX_NUM_ZMQ_SUBSCRIBERS];
+  zmq_pollitem_t items[MAX_ZMQ_SUBSCRIBERS];
+  u_int32_t zmq_max_num_polls_before_purge = MAX_ZMQ_POLLS_BEFORE_PURGE;
+  u_int32_t now, next_purge_idle = (u_int32_t)time(NULL) + FLOW_PURGE_FREQUENCY;
   int rc, size;
 
-  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Collecting flows on %s [%s]",
-			       ifname,
-			       ntop->getPrefs()->is_zmq_collector_mode() ? 
-			       "nprobe->ntopng" : "ntopng->nprobe");
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Collecting flows on %s", ifname);
 
   while(isRunning()) {
     while(idle()) {
@@ -117,28 +132,62 @@ void CollectorInterface::collect_flows() {
       items[i].socket = subscriber[i].socket, items[i].fd = 0, items[i].events = ZMQ_POLLIN, items[i].revents = 0;
 
     do {
-      rc = zmq_poll(items, num_subscribers, 1000 /* 1 sec */);
+      rc = zmq_poll(items, num_subscribers,  MAX_ZMQ_POLL_WAIT_MS);
+
+      now = (u_int32_t)time(NULL);
+      zmq_max_num_polls_before_purge--;
+
       if((rc < 0) || (!isRunning())) return;
-      if(rc == 0) purgeIdle(time(NULL));
+
+      if(rc == 0 || now >= next_purge_idle || zmq_max_num_polls_before_purge == 0) {
+	purgeIdle(now);
+	next_purge_idle = now + FLOW_PURGE_FREQUENCY;
+	zmq_max_num_polls_before_purge = MAX_ZMQ_POLLS_BEFORE_PURGE;
+      }
     } while(rc == 0);
 
     for(int source_id=0; source_id<num_subscribers; source_id++) {
+      u_int32_t msg_id;
+	
       if(items[source_id].revents & ZMQ_POLLIN) {
 	size = zmq_recv(items[source_id].socket, &h, sizeof(h), 0);
 
-	if((size != sizeof(h)) || (h.version != MSG_VERSION)) {
+	if(size == sizeof(struct zmq_msg_hdr_v0)) {
+	  /* Legacy version */
+	  msg_id = 0;
+	} else if((size != sizeof(h)) || (h.version != ZMQ_MSG_VERSION)) {
 	  ntop->getTrace()->traceEvent(TRACE_WARNING,
-				       "Unsupported publisher version [%d]: your nProbe sender is outdated?",
-				       h.version);
+				       "Unsupported publisher version: your nProbe sender is outdated? [%u][%u]",
+				       sizeof(struct zmq_msg_hdr), sizeof(h));
 	  continue;
-	}
+	} else
+	  msg_id = h.msg_id;
 
+	if((!is_collector) && (msg_id > 0)) {
+	  /* 
+	     TODO
+	     Develop logic for computing drops in collector mode
+	  */
+	  if(msg_id < recvStats.last_zmq_msg_id) {
+	    /* Start over */
+	  } else if(recvStats.last_zmq_msg_id > 0) {
+	    u_int32_t diff = msg_id - recvStats.last_zmq_msg_id;
+
+	    if(diff != 1) {
+	      recvStats.zmq_msg_drops += diff;
+	      ntop->getTrace()->traceEvent(TRACE_INFO, "msg_id=%u, drops=%u", msg_id, recvStats.zmq_msg_drops);
+	    }
+	  }
+
+	  recvStats.last_zmq_msg_id = msg_id;
+	}       
+	
 	size = zmq_recv(items[source_id].socket, payload, payload_len, 0);
 
 	if(size > 0) {
 	  char *uncompressed = NULL;
 	  u_int uncompressed_len;
-
+	  
 	  payload[size] = '\0';
 
 	  if(payload[0] == 0 /* Compressed traffic */) {
@@ -146,41 +195,52 @@ void CollectorInterface::collect_flows() {
 	    int err;
 	    uLongf uLen;
 
-	    uLen = uncompressed_len = 3*size;
+	    uLen = uncompressed_len = max(3*size, MAX_ZMQ_FLOW_BUF);
 	    uncompressed = (char*)malloc(uncompressed_len+1);
 	    if((err = uncompress((Bytef*)uncompressed, &uLen, (Bytef*)&payload[1], size-1)) != Z_OK) {
-	      ntop->getTrace()->traceEvent(TRACE_ERROR, "Uncompress error [%d]", err);
-	      return;
+	      ntop->getTrace()->traceEvent(TRACE_ERROR, "Uncompress error [%d][len: %u]", err, size);
+	      continue;
 	    }
 
 	    uncompressed_len = uLen, uncompressed[uLen] = '\0';
 #else
 	    static bool once = false;
-	    
+
 	    if(!once)
 	      ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to uncompress ZMQ traffic: ntopng compiled without zlib"), once = true;
 
-	    return;
+	    continue;
 #endif
 	  } else
 	    uncompressed = payload, uncompressed_len = size;
-	
-	  if(ntop->getPrefs()->get_zmq_encryption_pwd())
-	    Utils::xor_encdec((u_char*)uncompressed, uncompressed_len, (u_char*)ntop->getPrefs()->get_zmq_encryption_pwd());	  
-	  
-	  /* ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s", uncompressed); */
 
-	  if(strcmp(h.url, "event") == 0)
+	  if(ntop->getPrefs()->get_zmq_encryption_pwd())
+	    Utils::xor_encdec((u_char*)uncompressed, uncompressed_len, (u_char*)ntop->getPrefs()->get_zmq_encryption_pwd());
+	  
+	  ntop->getTrace()->traceEvent(TRACE_INFO, "%s [msg_id=%u]", uncompressed, msg_id);
+
+	  switch(h.url[0]) {
+	  case 'e': /* event */
+	    recvStats.num_events++;
 	    parseEvent(uncompressed, uncompressed_len, source_id, this);
-	  else
-	    parseFlow(uncompressed, uncompressed_len, source_id, this);
+	    break;
+
+	  case 'f': /* flow */
+	    recvStats.num_flows += parseFlow(uncompressed, uncompressed_len, source_id, this);
+	    break;
+
+	  case 'c': /* counter */
+	    recvStats.num_counters++;
+	    parseCounter(uncompressed, uncompressed_len, source_id, this);
+	    break;
+	  }
+
+	  /* ntop->getTrace()->traceEvent(TRACE_INFO, "[%s] %s", h.url, uncompressed); */
 
 #ifdef HAVE_ZLIB
 	  if(payload[0] == 0 /* only if the traffic was actually compressed */)
 	    if(uncompressed) free(uncompressed);
 #endif
-
-	  ntop->getTrace()->traceEvent(TRACE_INFO, "[%u] %s", uncompressed_len, uncompressed);
 	} /* size > 0 */
       }
     } /* for */
@@ -229,3 +289,30 @@ bool CollectorInterface::set_packet_filter(char *filter) {
 }
 
 /* **************************************************** */
+
+void CollectorInterface::lua(lua_State* vm) {
+  ParserInterface::lua(vm);
+
+  lua_newtable(vm);
+  lua_push_int_table_entry(vm, "flows", recvStats.num_flows);
+  lua_push_int_table_entry(vm, "events", recvStats.num_events);
+  lua_push_int_table_entry(vm, "counters", recvStats.num_counters);
+  lua_push_int_table_entry(vm, "zmq_msg_drops", recvStats.zmq_msg_drops);
+  lua_pushstring(vm, "zmqRecvStats");
+  lua_insert(vm, -2);
+  lua_settable(vm, -3);
+}
+
+/* **************************************************** */
+
+void CollectorInterface::purgeIdle(time_t when) {
+  NetworkInterface::purgeIdle(when);
+
+  if(flowHashing) {
+    FlowHashing *current, *tmp;
+    HASH_ITER(hh, flowHashing, current, tmp)
+      static_cast<NetworkInterface*>(current->iface)->purgeIdle(when);
+  }
+}
+
+#endif
