@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -38,14 +38,14 @@ DB::~DB() {
 
 /* ******************************************* */
 
-bool DB::dumpFlow(time_t when, bool partial_dump, Flow *f, char *json) {
+bool DB::dumpFlow(time_t when, Flow *f, char *json) {
   ntop->getTrace()->traceEvent(TRACE_WARNING, "Internal error");
   return(false);
 }
 
 /* ******************************************* */
 
-int DB::exec_sql_query(lua_State *vm, char *sql, bool limit_rows) {
+int DB::exec_sql_query(lua_State *vm, char *sql, bool limit_rows, bool wait_for_db_created) {
   ntop->getTrace()->traceEvent(TRACE_WARNING, "Internal error");
   return(false);
 }
@@ -53,5 +53,15 @@ int DB::exec_sql_query(lua_State *vm, char *sql, bool limit_rows) {
 /* ******************************************* */
 
 void DB::startDBLoop() {
-  ntop->getTrace()->traceEvent(TRACE_WARNING, "Internal error");
+  ntop->getTrace()->traceEvent(TRACE_WARNING, "*** Internal error ***");
 }
+
+/* ******************************************* */
+
+#ifdef NTOPNG_PRO
+bool DB::dumpAggregatedFlow(AggregatedFlow *f) {
+  ntop->getTrace()->traceEvent(TRACE_WARNING, "Internal error");
+  return(false);
+}
+#endif
+
