@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,18 +24,18 @@
 /* *************************************** */
 
 EthStats::EthStats() {
-  eth_IPv4.reset();
-  eth_IPv6.reset();
-  eth_ARP.reset();
-  eth_MPLS.reset();
-  eth_other.reset();
+  cleanup();
 }
 
 /* *************************************** */
 
-void EthStats::incStats(u_int16_t proto, u_int32_t num_pkts,
+void EthStats::incStats(bool ingressPacket,
+			u_int16_t proto, u_int32_t num_pkts,
 			u_int32_t num_bytes, u_int pkt_overhead) {
-  raw.inc(num_pkts, num_bytes+pkt_overhead*num_pkts);
+  if(ingressPacket)
+    rawIngress.inc(num_pkts, num_bytes+pkt_overhead*num_pkts);
+  else
+    rawEgress.inc(num_pkts, num_bytes+pkt_overhead*num_pkts);
 
   switch(proto) {
   case ETHERTYPE_ARP:
@@ -61,6 +61,7 @@ void EthStats::incStats(u_int16_t proto, u_int32_t num_pkts,
 
 void EthStats::lua(lua_State *vm) {
   lua_newtable(vm);
+
   eth_IPv4.lua(vm,  "IPv4_");
   eth_IPv6.lua(vm,  "IPv6_");
   eth_ARP.lua(vm,   "ARP_");
@@ -68,10 +69,18 @@ void EthStats::lua(lua_State *vm) {
   eth_other.lua(vm, "other_");
   lua_push_int_table_entry(vm, "bytes",   getNumBytes());
   lua_push_int_table_entry(vm, "packets", getNumPackets());
-  lua_pushstring(vm, "eth");
-  lua_insert(vm, -2);
-  lua_settable(vm, -3);
 
+  lua_newtable(vm);
+  lua_push_int_table_entry(vm, "bytes",   getNumIngressBytes());
+  lua_push_int_table_entry(vm, "packets", getNumIngressPackets());
+  lua_pushstring(vm, "ingress"); lua_insert(vm, -2); lua_settable(vm, -3);
+
+  lua_newtable(vm);
+  lua_push_int_table_entry(vm, "bytes",   getNumEgressBytes());
+  lua_push_int_table_entry(vm, "packets", getNumEgressPackets());
+  lua_pushstring(vm, "egress"); lua_insert(vm, -2); lua_settable(vm, -3);
+
+  lua_pushstring(vm, "eth"); lua_insert(vm, -2); lua_settable(vm, -3);
 }
 
 /* *************************************** */
@@ -87,7 +96,7 @@ void EthStats::print() {
 /* *************************************** */
 
 void EthStats::cleanup() {
-  raw.reset();
+  rawIngress.reset(), rawEgress.reset(); 
   eth_IPv4.reset();
   eth_IPv6.reset();
   eth_ARP.reset();
