@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 #include "ntop_includes.h"
 
+#ifndef HAVE_NEDGE
 
 /* **************************************************** */
 
@@ -66,7 +67,7 @@ void ExportInterface::export_data(char *json) {
     struct zmq_msg_hdr msg_hdr;
 
     snprintf(msg_hdr.url, sizeof(msg_hdr.url), "%s", topic);
-    msg_hdr.version = 0, msg_hdr.size = (u_int32_t)strlen(json);
+    msg_hdr.version = ZMQ_MSG_VERSION, msg_hdr.size = (u_int32_t)strlen(json);
 
     zmq_send(publisher, &msg_hdr, sizeof(msg_hdr), ZMQ_SNDMORE);
     zmq_send(publisher, json, msg_hdr.size, 0);
@@ -75,3 +76,5 @@ void ExportInterface::export_data(char *json) {
 }
 
 /* **************************************************** */
+
+#endif
