@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -42,7 +42,7 @@ Flow* FlowHash::find(IpAddress *src_ip, IpAddress *dst_ip,
   u_int16_t num_loops = 0;
   
   while(head) {
-    if((!head->idle())
+    if((!head->idle() && !head->is_ready_to_be_purged())
        && head->equal(src_ip, dst_ip, src_port, dst_port, vlanId, protocol, src2dst_direction)) {
       if(num_loops > max_num_loops) {
 	ntop->getTrace()->traceEvent(TRACE_INFO, "DEBUG: [Num loops: %u][hashId: %u]", num_loops, hash);
