@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,8 @@
 
 GenericHash::GenericHash(NetworkInterface *_iface, u_int _num_hashes, u_int _max_hash_size) {
   num_hashes = _num_hashes, max_hash_size = _max_hash_size, current_size = 0;
+  purge_step = max_val(num_hashes / PURGE_FRACTION, 1);
+  
   iface = _iface;
   table = new GenericHashEntry*[num_hashes];
   for(u_int i = 0; i < num_hashes; i++)
@@ -133,37 +135,73 @@ bool GenericHash::remove(GenericHashEntry *h) {
 
 /* ************************************ */
 
-void GenericHash::walk(bool (*walker)(GenericHashEntry *h, void *user_data), void *user_data) {
+bool GenericHash::walk(u_int32_t *begin_slot,
+		       bool walk_all,
+		       bool (*walker)(GenericHashEntry *h, void *user_data, bool *entryMatched), void *user_data) {
   bool found = false;
-
+  u_int16_t tot_matched = 0;
+  
   if(ntop->getGlobals()->isShutdown())     
-    return;
+    return(found);
 
-  for(u_int hash_id = 0; hash_id < num_hashes; hash_id++) {
+  for(u_int hash_id = *begin_slot; hash_id < num_hashes; hash_id++) {    
     if(table[hash_id] != NULL) {
       GenericHashEntry *head;
 
-      //ntop->getTrace()->traceEvent(TRACE_NORMAL, "[walk] Locking %d [%p]", hash_id, locks[hash_id]);
+#if WALK_DEBUG
+      ntop->getTrace()->traceEvent(TRACE_NORMAL, "[walk] Locking %d [%p]", hash_id, locks[hash_id]);
+#endif
+      
       locks[hash_id]->lock(__FILE__, __LINE__);
       head = table[hash_id];
 
       while(head) {
 	GenericHashEntry *next = head->next();
 
-	if((!head->idle()) && walker(head, user_data)) {
-	  found = true;
-	  break;
+	if(!head->idle() && !head->is_ready_to_be_purged()) {
+	  bool matched = false;
+	  bool rc = walker(head, user_data, &matched);
+
+	  if(matched) tot_matched++;
+	  
+	  if(rc) {
+	    found = true;
+	    break;
+	  }
 	}
+	
 	head = next;
       } /* while */
 
       locks[hash_id]->unlock(__FILE__, __LINE__);
       // ntop->getTrace()->traceEvent(TRACE_NORMAL, "[walk] Unlocked %d", hash_id);
 
+      if((tot_matched >= MIN_NUM_HASH_WALK_ELEMS) /* At least a few entries have been returned */
+	 && (!walk_all)) {
+	u_int32_t next_slot  = (hash_id == (num_hashes-1)) ? 0 /* start over */ : (hash_id+1);
+	
+	*begin_slot = next_slot;
+#if WALK_DEBUG
+	ntop->getTrace()->traceEvent(TRACE_NORMAL, "[walk] Over [nextSlot: %u][hash_id: %u][tot_matched: %u]",
+				     next_slot, hash_id, tot_matched);
+#endif
+	
+	return(found);
+      }
+      
       if(found)
-	break;
+	break;      
     }
   }
+
+  if(!found)
+    *begin_slot = 0 /* start over */;
+
+#if WALK_DEBUG
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "[walk] Over [tot_matched: %u]", tot_matched);
+#endif
+  
+  return(found);
 }
 
 /* ************************************ */
@@ -183,7 +221,7 @@ u_int GenericHash::purgeIdle() {
 
   disablePurge();
 
-  for(u_int j = 0; j < num_hashes / PURGE_FRACTION; j++) {
+  for(u_int j = 0; j < purge_step; j++) {
     if(++last_purged_hash == num_hashes) last_purged_hash = 0;
     i = last_purged_hash;
 
@@ -197,7 +235,7 @@ u_int GenericHash::purgeIdle() {
       while(head) {
 	GenericHashEntry *next = head->next();
 
-	if(head->idle()) {
+	if(head->is_ready_to_be_purged()) {
 	  if(prev == NULL) {
 	    table[i] = next;
 	  } else {
@@ -208,6 +246,13 @@ u_int GenericHash::purgeIdle() {
 	  delete(head);
 	  head = next;
 	} else {
+	  /* Do the chores */
+	  head->housekeep();
+
+	  /* Purge at the next run */
+	  if(head->idle())
+	    head->set_to_purge();
+	  
 	  prev = head;
 	  head = next;
 	}
@@ -232,7 +277,7 @@ GenericHashEntry* GenericHash::findByKey(u_int32_t key) {
 
   locks[hash]->lock(__FILE__, __LINE__);
   while(head != NULL) {
-    if((!head->idle()) && (head->key() == key))
+    if((!head->idle()) && (!head->is_ready_to_be_purged()) && (head->key() == key))
       break;
     else
       head = head->next();
