@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -105,24 +105,31 @@ void Geolocation::getAS(IpAddress *addr, u_int32_t *asn, char **asname) {
   if(rsp != NULL) {
     char *space = strchr(rsp, ' ');
 
-    *asn = atoi(&rsp[2]);
+    if(asn)
+      *asn = atoi(&rsp[2]);
 
-    if(space)
-      *asname = strdup(&space[1]);
-    else
-      *asname = strdup(rsp);
+    if(asname) {
+      if(space)
+	*asname = strdup(&space[1]);
+      else
+	*asname = strdup(rsp);
+    }
 
     free(rsp);
     return;
   }
 #endif
 
-  *asn = 0, *asname = NULL;
+  if(asn)
+    *asn = 0;
+  if(asname)
+    *asname = NULL;
 }
 
 /* *************************************** */
 
-void Geolocation::getInfo(IpAddress *addr, char **country_code, char **city, float *latitude, float *longitude) {
+void Geolocation::getInfo(IpAddress *addr, char **continent_code, char **country_code,
+			  char **city, float *latitude, float *longitude) {
 #ifdef HAVE_GEOIP
   GeoIPRecord *geo = NULL;
   struct ipAddress *ip = addr->getIP();
@@ -143,12 +150,19 @@ void Geolocation::getInfo(IpAddress *addr, char **country_code, char **city, flo
   }
 
   if(geo != NULL) {
-    *country_code = geo->country_code ? strdup(geo->country_code) : NULL;
+    *continent_code = 
+#ifdef WIN32
+		""
+#else
+		geo->continent_code
+#endif
+		;
+    *country_code = geo->country_code;
     *city = geo->city ? strdup(geo->city) : NULL;
     *latitude = geo->latitude, *longitude = geo->longitude;
     GeoIPRecord_delete(geo);
   } else
-    *country_code = NULL, *city = NULL, *latitude = *longitude = 0;
 #endif
+    *country_code = (char*)UNKNOWN_COUNTRY, *city = NULL, *latitude = *longitude = 0, *continent_code = (char*)"";
 }
 
