@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 /* **************************************** */
 
 HTTPBL::HTTPBL(char *_api_key) {
-  api_key = _api_key ? strdup(_api_key) : NULL;
+  api_key = _api_key ? _api_key : NULL;
   num_httpblized_categorizations = num_httpblized_fails = 0;
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "Enable http:dl with API key %s", api_key);
 }
@@ -185,7 +185,7 @@ static int dns_query_execute(char* query, char* resp, u_int resp_len) {
 /* **************************************************** */
 
 void HTTPBL::queryHTTPBL(char* numeric_ip) {
-  char dns_query_str[256] = { 0 }, query_resp[256], alert_msg[512], *iface;
+  char dns_query_str[256] = { 0 }, query_resp[256], *iface;
   int rc;
 
   /* Format numeric_ip@interface_name */
@@ -222,11 +222,8 @@ void HTTPBL::queryHTTPBL(char* numeric_ip) {
       /* https://www.projecthoneypot.org/httpbl_api.php */
 
       /* We need to figure out the current list of peers speaking with this host */
-      snprintf(alert_msg, sizeof(alert_msg),
-	       "Host <A HREF='/lua/host_details.lua?host=%s&ifname=%s'>%s</A> blacklisted on HTTP:BL [code=%s]",
-	       numeric_ip, iface, numeric_ip, query_resp);
+      /* TODO: possibly generate an alert */
 
-      ntop->getRedis()->queueAlert(alert_level_warning, alert_permanent, alert_dangerous_host, alert_msg);
       break;
   }
 
