@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,8 @@
 
 /* ************************************ */
 
-HostHash::HostHash(NetworkInterface *_iface, u_int _num_hashes, u_int _max_hash_size) : GenericHash(_iface, _num_hashes, _max_hash_size) {
+HostHash::HostHash(NetworkInterface *_iface, u_int _num_hashes, u_int _max_hash_size)
+  : GenericHash(_iface, _num_hashes, _max_hash_size) {
   num_http_hosts = 0;
 }
 
@@ -42,6 +43,7 @@ Host* HostHash::get(u_int16_t vlanId, IpAddress *key) {
     
     while(head != NULL) {      
       if((!head->idle())
+	 && (!head->is_ready_to_be_purged())
 	 && (head->get_vlan_id() == vlanId)
 	 && (head->get_ip() != NULL)
 	 && (head->get_ip()->compare(key) == 0))
@@ -57,39 +59,6 @@ Host* HostHash::get(u_int16_t vlanId, IpAddress *key) {
 
 /* ************************************ */
 
-Host* HostHash::get(u_int16_t vlanId, const u_int8_t mac[6]) {
-  u_int32_t hash = 0;
-
-  for(int i=0; i<6; i++) hash += mac[i] << (i+1);
-  hash %= num_hashes;
-
-  if(table[hash] == NULL) {
-    return(NULL);
-  } else {
-    Host *head;
-
-    locks[hash]->lock(__FILE__, __LINE__);
-    head = (Host*)table[hash];
-
-    while(head != NULL) {
-      if((!head->idle())
-	 && (head->get_ip() == NULL /* This is not a L2 host */)
-         // && (vlanId == 0 /* any vlan */ || head->get_vlan_id() == vlanId)
-         && (head->get_vlan_id() == vlanId)
-         && (memcmp(mac, head->get_mac(), 6) == 0))
-	break;
-      else
-        head = (Host*)head->next();
-    }
-    
-    locks[hash]->unlock(__FILE__, __LINE__);
-    
-    return(head);
-  }
-}
-
-/* ************************************ */
-
 void HostHash::incNumHTTPEntries() {
   m.lock(__FILE__, __LINE__);
   num_http_hosts++; 
