@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,47 +31,7 @@ IpAddress::IpAddress() {
 
 /* ******************************************* */
 
-IpAddress::IpAddress(char *string) {
-  ip_key = 0;
-  set_from_string(string);
-  compute_key();
-}
-
-/* ******************************************* */
-
-IpAddress::IpAddress(IpAddress *ip) {
-  ip_key = 0;
-  set(ip);
-}
-
-/* ******************************************* */
-
-IpAddress::IpAddress(u_int32_t _ipv4) {
-  ip_key = 0;
-  set_ipv4(_ipv4);
-  compute_key();
-}
-
-/* ******************************************* */
-
-IpAddress::IpAddress(struct ndpi_in6_addr *_ipv6) {
-  ip_key = 0;
-  set_ipv6(_ipv6);
-  addr.privateIP = false;
-  compute_key();
-}
-
-/* ******************************************* */
-
-void IpAddress::set(IpAddress *ip) {
-  memcpy(&addr, &ip->addr, sizeof(struct ipAddress));
-  ip_key = ip->ip_key;
-  compute_key();
-}
-
-/* ******************************************* */
-
-void IpAddress::set_from_string(char *sym_addr) {
+void IpAddress::set(char *sym_addr) {
   if(strchr(sym_addr, '.')) {
     addr.ipVersion = 4, addr.localHost = 0, addr.ipType.ipv4 = inet_addr(sym_addr);
   } else {
@@ -90,16 +50,23 @@ void IpAddress::set_from_string(char *sym_addr) {
 
 bool IpAddress::isEmpty() {
   if((addr.ipVersion == 0)
-     || ((addr.ipVersion == 4) && (addr.ipType.ipv4 == 0)))
-    return(true);
-  else
-    return(false);
+     || ((addr.ipVersion == 4) && (addr.ipType.ipv4 == 0))) {
+    return true;
+  }else if(addr.ipVersion == 6) {
+    struct ndpi_in6_addr empty_ipv6;
+    memset(&empty_ipv6, 0, sizeof(empty_ipv6));
+    return memcmp((void*)&empty_ipv6, (void*)&addr.ipType.ipv6, sizeof(empty_ipv6)) == 0 ? true : false;
+  }
+  return false;
 }
 
 /* ******************************************* */
 
 void IpAddress::checkIP() {
   u_int32_t a;
+  int16_t local_network_id;
+  u_int32_t nmask;
+  u_int8_t nmask_bits;
 
   addr.privateIP = false; /* Default */
 
@@ -115,6 +82,7 @@ void IpAddress::checkIP() {
     172.16.0.0      -   172.31.255.255  (172.16/12 prefix)
     192.168.0.0     -   192.168.255.255 (192.168/16 prefix)
   */
+
   a = ntohl(addr.ipType.ipv4);
 
   if(((a & 0xFF000000) == 0x0A000000 /* 10.0.0.0/8 */)
@@ -127,6 +95,16 @@ void IpAddress::checkIP() {
     addr.multicastIP = true;
   else if((a == 0xFFFFFFFF) || (a == 0))
     addr.broadcastIP = true;
+
+  if(ntop->isLocalAddress(AF_INET, &addr.ipType.ipv4, &local_network_id, &nmask_bits)) {
+    if(nmask_bits < 31) { /* /32 is just an host, /31 is a point-to-point */
+      nmask = ~((1 << (32 - nmask_bits)) - 1);
+      if(a == (a | ~nmask) || a == (a & nmask))
+	addr.broadcastIP = true;
+    }
+
+  }
+
 }
 
 /* ******************************************* */
@@ -143,7 +121,7 @@ int IpAddress::compare(IpAddress *ip) {
 
     if(a < b) return(1); else if(b < a) return(-1); else return(0);
 #else
-    return(memcmp(&ip->addr.ipType.ipv4, &addr.ipType.ipv4, sizeof(u_int32_t)));
+    return(memcmp(&addr.ipType.ipv4, &ip->addr.ipType.ipv4, sizeof(u_int32_t)));
 #endif
   } else
     return(memcmp(&addr.ipType.ipv6, &ip->addr.ipType.ipv6, sizeof(struct ndpi_in6_addr)));
@@ -193,11 +171,22 @@ void IpAddress::compute_key() {
 /* ******************************************* */
 
 char* IpAddress::print(char *str, u_int str_len, u_int8_t bitmask) {
+  str[0] = '\0';
   return(intoa(str, str_len, bitmask));
 }
 
 /* ******************************************* */
 
+char* IpAddress::printMask(char *str, u_int str_len, bool isLocalIP) {
+  if(Utils::maskHost(isLocalIP)) {
+    snprintf(str, str_len, (addr.ipVersion == 4) ? "0.0.0.0" : "::");
+    return(str);
+  } else
+    return(intoa(str, str_len, 0xFF /* bitmask */));
+}
+
+/* ******************************************* */
+
 bool IpAddress::isLocalHost(int16_t *network_id) {
   if(addr.ipVersion == 4) {
     u_int32_t v = /* htonl */(addr.ipType.ipv4);
@@ -210,16 +199,16 @@ bool IpAddress::isLocalHost(int16_t *network_id) {
 
 /* ******************************************* */
 
-void* IpAddress::findAddress(patricia_tree_t *ptree) {
+void* IpAddress::findAddress(AddressTree *ptree) {
   if(ptree == NULL)
     return(NULL);
   else {
     void *ret;
 
     if(addr.ipVersion == 4)
-      ret = ptree_match(ptree, AF_INET, &addr.ipType.ipv4, 32);
+      ret = Utils::ptree_match(ptree->getTree(true), AF_INET, &addr.ipType.ipv4, 32);
     else
-      ret = ptree_match(ptree, AF_INET6, (void*)&addr.ipType.ipv6, 128);
+      ret = Utils::ptree_match(ptree->getTree(false), AF_INET6, (void*)&addr.ipType.ipv6, 128);
 
     return(ret);
   }
@@ -239,26 +228,6 @@ char* IpAddress::serialize() {
 
 /* ******************************************* */
 
-void IpAddress::deserialize(json_object *o) {
-  json_object *obj;
-
-  if(!o) return;
-
-  /* Reset all */
-  memset(&addr, 0, sizeof(addr));
-
-  if(json_object_object_get_ex(o, "ipVersion", &obj))
-    addr.ipVersion = json_object_get_int(obj);
-
-  if(json_object_object_get_ex(o, "localHost", &obj))
-    addr.localHost = json_object_get_boolean(obj);
-
-  if(json_object_object_get_ex(o, "ip", &obj))
-    set_from_string((char*)json_object_get_string(obj));
-}
-
-/* ******************************************* */
-
 json_object* IpAddress::getJSONObject() {
   json_object *my_object;
   char buf[64];
@@ -275,38 +244,64 @@ json_object* IpAddress::getJSONObject() {
 /* ******************************************* */
 
 /**
- * @brief Check if the host matches the specifed host tree
+ * @brief Check if the host matches the specified host tree
  *
  * @param ptree     The hosts allowed to be accessed.
  * @return true if the host matches the ptre, false otherwise.
  */
-bool IpAddress::match(patricia_tree_t *ptree) {
-  patricia_node_t *node;
+bool IpAddress::match(AddressTree *tree) {
+  if(tree == NULL)
+    return(true);
+  else {
+    patricia_tree_t *ptree = tree->getTree((addr.ipVersion == 4) ? true : false);
+    patricia_node_t *node;
 
-  if(ptree == NULL) return(true);
+    if(ptree == NULL) return(true);
 
-  if(addr.ipVersion == 4)
-    node = ptree_match(ptree, AF_INET, (void*)&addr.ipType.ipv4, 32);
-  else
-    node = ptree_match(ptree, AF_INET6, (void*)&addr.ipType.ipv6, 128);
+    if(addr.ipVersion == 4)
+      node = Utils::ptree_match(ptree, AF_INET, (void*)&addr.ipType.ipv4, 32);
+    else
+      node = Utils::ptree_match(ptree, AF_INET6, (void*)&addr.ipType.ipv6, 128);
 
-  return((node == NULL) ? false : true);
+    return((node == NULL) ? false : true);
+  }
 }
 
 /* ****************************** */
 
 char* IpAddress::intoa(char* buf, u_short bufLen, u_int8_t bitmask) {
   if((addr.ipVersion == 4) || (addr.ipVersion == 0 /* Misconfigured */)) {
+    bitmask = bitmask <= 32 ? bitmask : 32;
     u_int32_t a = ntohl(addr.ipType.ipv4);
 
     if(bitmask > 0) {
+      /* bitmask 0 here causes integer overflow */
       u_int32_t netmask = ~((1 << (32 - bitmask)) - 1);
       a &= netmask;
+    } else {
+      /* bitmask is 0 */
+      a = 0;
     }
 
     return(Utils::intoaV4(a, buf, bufLen));
   } else {
+    bitmask = bitmask <= 128 ? bitmask : 128;
     return(Utils::intoaV6(addr.ipType.ipv6, bitmask, buf, bufLen));
   }
 }
+/* ****************************** */
 
+void IpAddress::dump() {
+  int16_t network_id;
+  char buf[48];
+  const char *local, *system;
+
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "-------------------- [ Local ]");
+  local  =  isLocalHost(&network_id)  ? "Yes" : "No";
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "-------------------- [ System ]");
+  system =  isLocalInterfaceAddress() ? "Yes" : "No";
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "--------------------");
+
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s [Local: %s][SystemHost: %s]",
+			       print(buf, sizeof(buf)), local, system);
+}
