@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 #include "ntop_includes.h"
 
+#ifdef HAVE_MYSQL
 /* **************************************************** */
 
 static void* queryLoop(void* ptr) {
@@ -34,7 +35,12 @@ void* MySQLDB::queryLoop() {
   MYSQL mysql_alt;
   char sql[CONST_MAX_SQL_QUERY_LEN];
 
-  if(!connectToDB(&mysql_alt, true))
+  while(!ntop->getGlobals()->isShutdown()
+	&& !MySQLDB::isDbCreated() /* wait until the db has been created */) {
+    sleep(1);
+  }
+
+  if(ntop->getGlobals()->isShutdown() || !connectToDB(&mysql_alt, true))
     return(NULL);
 
   while(!ntop->getGlobals()->isShutdown()) {
@@ -46,40 +52,46 @@ void* MySQLDB::queryLoop() {
 				     "Tried to execute a query longer than %u. Skipping.",
 				     CONST_MAX_SQL_QUERY_LEN - 2);
 	continue;  // prevents overflown queries to generate mysql errors
-      } else if(exec_sql_query(&mysql_alt, sql, true, true, false) < 0) {
-	ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s", get_last_db_error(&mysql_alt));
+      } else if((rc = exec_sql_query(&mysql_alt, sql, true /* Attempt to reconnect */, true /* Don't print errors */, false)) < 0) {
+	ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s [rc=%d]", get_last_db_error(&mysql_alt), rc);
 	ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", sql);
-	mysql_close(&mysql_alt);
-	if(!connectToDB(&mysql_alt, true))
-	  return(NULL);
+
+	/* Don't give up, manually re-connect */
+	disconnectFromDB(&mysql_alt);
+	if(!connectToDB(&mysql_alt, true)) _usleep(100);
+      } else {
+	mysqlExportedFlows++;
       }
     } else
       sleep(1);    
   }
 
-  mysql_close(&mysql_alt);
+  disconnectFromDB(&mysql_alt);
   return(NULL);
 }
 
 /* ******************************************* */
-
-MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
+volatile bool MySQLDB::db_created = false;
+bool MySQLDB::createDBSchema(bool set_db_created) {
   char sql[CONST_MAX_SQL_QUERY_LEN];
 
-  if(connectToDB(&mysql, false) == false)
-    return;
-
   if(iface) {
+    disconnectFromDB(&mysql);
+    if(connectToDB(&mysql, false) == false){
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to connect: %s\n", get_last_db_error(&mysql));
+      return false;
+    }
+
     /* 1 - Create database if missing */
-    snprintf(sql, sizeof(sql), "CREATE DATABASE IF NOT EXISTS %s", ntop->getPrefs()->get_mysql_dbname());
+    snprintf(sql, sizeof(sql), "CREATE DATABASE IF NOT EXISTS `%s`", ntop->getPrefs()->get_mysql_dbname());
     if(exec_sql_query(&mysql, sql, true) < 0) {
       ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s\n", get_last_db_error(&mysql));
-      return;
+      return false;
     }
 
     if(mysql_select_db(&mysql, ntop->getPrefs()->get_mysql_dbname())) {
       ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s\n", get_last_db_error(&mysql));
-      return;
+      return false;
     }
 
     /* 2.1 - Create table if missing [IPv6] */
@@ -116,7 +128,7 @@ MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
 
     if(exec_sql_query(&mysql, sql, true) < 0) {
       ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s\n", get_last_db_error(&mysql));
-      return;
+      return false;
     }
 
     /* 2.2 - Create table if missing [IPv4] */
@@ -153,7 +165,7 @@ MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
 
     if(exec_sql_query(&mysql, sql, true) < 0) {
       ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s\n", get_last_db_error(&mysql));
-      return;
+      return false;
     }
 
     // the remainder of this method has the purpose of MIGRATING old table structures to
@@ -208,7 +220,7 @@ MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
 	     ntop->getPrefs()->get_mysql_tablename(), iface->get_id());
     exec_sql_query(&mysql, sql, true, true);
 
-    // We trasfer old table contents into the new schema
+    // We transfer old table contents into the new schema
     snprintf(sql, sizeof(sql), "INSERT IGNORE INTO `%sv4` "
 	     "SELECT * FROM `%sv4_%u`",
 	     ntop->getPrefs()->get_mysql_tablename(),
@@ -241,7 +253,7 @@ MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
 	   ntop->getPrefs()->get_mysql_tablename());
   exec_sql_query(&mysql, sql, true, true);
 
-  // Add an extra column with the interface id to speed up certain quer
+  // Add an extra column with the interface id to speed up certain query
   snprintf(sql, sizeof(sql), "ALTER TABLE `%sv4` ADD COLUMN INTERFACE_ID SMALLINT(5) DEFAULT NULL",
 	   ntop->getPrefs()->get_mysql_tablename());
   exec_sql_query(&mysql, sql, true, true);
@@ -289,9 +301,16 @@ MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
   // Move column BYTES to BYTES_IN and add BYTES_OUT
   // note that this operation will arbitrarily move the old BYTES contents to BYTES_IN
   const u_int16_t ipvers[2] = {4, 6};
-  for (u_int16_t i = 0; i < sizeof(ipvers); i++){
-    snprintf(sql, sizeof(sql), "SHOW COLUMNS FROM `%sv%hu` LIKE 'BYTES'",
-	     ntop->getPrefs()->get_mysql_tablename(), ipvers[i]);
+  for (u_int16_t i = 0; i < sizeof(ipvers) / sizeof(u_int16_t); i++){
+    snprintf(sql, sizeof(sql),
+	     "SELECT 1 "
+	     "FROM information_schema.COLUMNS "
+	     "WHERE TABLE_SCHEMA='%s' "
+	     "AND TABLE_NAME='%sv%hu' "
+	     "AND COLUMN_NAME='BYTES' ",
+	     ntop->getPrefs()->get_mysql_dbname(),
+	     ntop->getPrefs()->get_mysql_tablename(),
+	     ipvers[i]);
     if(exec_sql_query(&mysql, sql, true, true) > 0){
       // if here, the column BYTES exists so we want to alter the table
       ntop->getTrace()->traceEvent(TRACE_NORMAL,
@@ -309,7 +328,7 @@ MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
   }
 
   // Modify database engine to MyISAM (that is much faster in non-transactional environments)
-  for (u_int16_t i = 0; i < sizeof(ipvers); i++){
+  for (u_int16_t i = 0; i < sizeof(ipvers) / sizeof(u_int16_t); i++){
     snprintf(sql, sizeof(sql),
 	     "SELECT 1 "
 	     "FROM information_schema.TABLES "
@@ -320,7 +339,8 @@ MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
 	     ntop->getPrefs()->get_mysql_tablename(),
 	     ipvers[i]);
     if(exec_sql_query(&mysql, sql, true, true) > 0){
-      // if here, the table has enging InnoDB so we want to modify that to MyISAM
+      // if here, the table has engine InnoDB so we want to modify that to MyISAM
+      ntop->getTrace()->traceEvent(TRACE_INFO, "%s", sql);
       ntop->getTrace()->traceEvent(TRACE_NORMAL,
 				   "MySQL schema update. Altering table %sv%hu: "
 				   "changing engine from InnoDB to MyISAM.",
@@ -332,12 +352,138 @@ MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
       exec_sql_query(&mysql, sql, true, true);
     }
   }
+
+  // Modify the number of partitions from 32 to 8
+  for (u_int16_t i = 0; i < sizeof(ipvers) / sizeof(u_int16_t); i++){
+    snprintf(sql, sizeof(sql),
+	     "SELECT 1 "
+	     "FROM information_schema.PARTITIONS "
+	     "WHERE TABLE_SCHEMA='%s' "
+	     "AND TABLE_NAME='%sv%hu' "
+	     "GROUP BY TABLE_NAME HAVING COUNT(*) = 32 ",
+	     ntop->getPrefs()->get_mysql_dbname(),
+	     ntop->getPrefs()->get_mysql_tablename(),
+	     ipvers[i]);
+    if(exec_sql_query(&mysql, sql, true, true) > 0){
+      // if here, the table has 32 partitions and we want to convert them to 8
+      ntop->getTrace()->traceEvent(TRACE_INFO, "%s", sql);
+      ntop->getTrace()->traceEvent(TRACE_NORMAL,
+				   "MySQL schema update. Altering table %sv%hu: "
+				   "changing the number of partitions to 8.",
+				   ntop->getPrefs()->get_mysql_tablename(), ipvers[i]);
+
+      snprintf(sql, sizeof(sql),
+	       "ALTER TABLE `%sv%hu` PARTITION BY HASH(FIRST_SWITCHED) PARTITIONS 8",
+	       ntop->getPrefs()->get_mysql_tablename(), ipvers[i]);
+      ntop->getTrace()->traceEvent(TRACE_INFO, "%s", sql);
+      exec_sql_query(&mysql, sql, true, true);
+    }
+  }
+
+  // make counter fields as unsigned so they can store 2x values
+  const char *counter_fields[3] = {"IN_BYTES", "OUT_BYTES", "PACKETS"};
+  for (u_int16_t i = 0; i < sizeof(ipvers) / sizeof(u_int16_t); i++){
+    for (u_int16_t j = 0; j < sizeof(counter_fields) / sizeof(char*); j++){
+      snprintf(sql, sizeof(sql),
+	       "SELECT 1 "
+	       "FROM information_schema.COLUMNS "
+	       "WHERE TABLE_SCHEMA='%s' "
+	       "AND TABLE_NAME='%sv%hu' "
+	       "AND COLUMN_NAME='%s' "
+	       "AND COLUMN_TYPE NOT LIKE '%%UNSIGNED' ",
+	       ntop->getPrefs()->get_mysql_dbname(),
+	       ntop->getPrefs()->get_mysql_tablename(),
+	       ipvers[i], counter_fields[j]);
+      if(exec_sql_query(&mysql, sql, true, true) > 0){
+	// if here we have to convert the type to unsigned
+	ntop->getTrace()->traceEvent(TRACE_NORMAL,
+				     "MySQL schema update. Altering table %sv%hu: "
+				     "changing %s data type to unsigned int.",
+				     ntop->getPrefs()->get_mysql_tablename(),
+				     ipvers[i],
+				     counter_fields[j]);
+
+	snprintf(sql, sizeof(sql),
+		 "ALTER TABLE `%sv%hu` MODIFY COLUMN `%s` int(10) unsigned",
+		 ntop->getPrefs()->get_mysql_tablename(), ipvers[i], counter_fields[j]);
+	exec_sql_query(&mysql, sql, true, true);
+      }
+    }
+  }
+
+  if(set_db_created)
+    db_created = true;
+  return true;
+}
+
+/* ******************************************* */
+
+bool MySQLDB::createNprobeDBView() {
+  char sql[CONST_MAX_SQL_QUERY_LEN];
+  const u_int16_t ipvers[2] = {4, 6};
+  u_int16_t i = 0;
+
+  if(mysql_select_db(&mysql, ntop->getPrefs()->get_mysql_dbname())) {
+    goto err;
+    return false;
+  }
+
+  for(; i < sizeof(ipvers) / sizeof(u_int16_t); i++){
+    snprintf(sql, sizeof(sql), MYSQL_DROP_NPROBE_VIEW, ipvers[i]);
+
+    ntop->getTrace()->traceEvent(TRACE_INFO,
+				 "Deleting existing nProbe views for IPV%hu:\n"
+				 "[%s]",
+				 ipvers[i],
+				 sql);
+
+    if(exec_sql_query(&mysql, sql, true) < 0)
+      goto err;
+
+    snprintf(sql, sizeof(sql), MYSQL_CREATE_NPROBE_VIEW,
+	     ipvers[i], ipvers[i], ipvers[i], iface->get_id(),
+	     ntop->getPrefs()->get_mysql_tablename(),
+	     ipvers[i]);
+
+    ntop->getTrace()->traceEvent(TRACE_INFO,
+				 "Creating nProbe view on table %sflows for IPV%hu:\n"
+				 "[%s]",
+				 ntop->getPrefs()->get_mysql_tablename(),
+				 ipvers[i],
+				 sql);
+
+    if(exec_sql_query(&mysql, sql, true) < 0) {
+    err:
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s\n", get_last_db_error(&mysql));
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "Before starting ntopng, make sure to start nprobe with option --mysql and template @NTOPNG@.");
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "Example:");
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "./nprobe -i eno1 -T \"@NTOPNG@\" --mysql=\"localhost:ntopng:nf:root:root\" --zmq \"tcp://127.0.0.1:5556\" --zmq-probe-mode");
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "./ntopng  -i \"tcp://*:5556c\" -F \"mysql-nprobe;localhost;ntopng;nf;root;root\"");
+      return false;
+    }
+
+  }
+
+  db_created = true;
+
+  return true;
+}
+
+/* ******************************************* */
+
+MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
+  mysqlDroppedFlows = 0;
+  mysqlExportedFlows = 0, mysqlLastExportedFlows = 0;
+  mysqlExportRate = 0;
+  checkpointDroppedFlows = checkpointExportedFlows = 0;
+  lastUpdateTime.tv_sec = 0, lastUpdateTime.tv_usec = 0;
+  connectToDB(&mysql, false);
 }
 
 /* ******************************************* */
 
 MySQLDB::~MySQLDB() {
-  mysql_close(&mysql);
+  disconnectFromDB(&mysql);
 }
 
 /* ******************************************* */
@@ -346,145 +492,186 @@ void MySQLDB::startDBLoop() {
   pthread_create(&queryThreadLoop, NULL, ::queryLoop, (void*)this);
 }
 
+
 /* ******************************************* */
 
-#ifdef NOTUSED
-char* MySQLDB::get_insert_into_values(IPVersion vers) {
-  char sql[CONST_MAX_SQL_QUERY_LEN];
+void MySQLDB::updateStats(const struct timeval *tv) {
+  if(tv == NULL) return;
 
-  snprintf(sql,
-	   sizeof(sql),
-	   "INSERT INTO `%sv%i` (VLAN_ID,L7_PROTO,IP_SRC_ADDR,L4_SRC_PORT,IP_DST_ADDR,L4_DST_PORT,PROTOCOL,"
-	   "BYTES,PACKETS,FIRST_SWITCHED,LAST_SWITCHED,INFO,JSON,NTOPNG_INSTANCE_NAME,INTERFACE_ID"
-#ifdef NTOPNG_PRO
-	   ",PROFILE"
-#endif
-	   ") "
-	   "VALUES ",
-	   ntop->getPrefs()->get_mysql_tablename(),
-	   vers);
+  if(lastUpdateTime.tv_sec > 0) {
+    float tdiffMsec = ((float)(tv->tv_sec-lastUpdateTime.tv_sec)*1000)+((tv->tv_usec-lastUpdateTime.tv_usec)/(float)1000);
+    if(tdiffMsec >= 1000) { /* al least one second */
+      u_int64_t diffFlows = mysqlExportedFlows - mysqlLastExportedFlows;
+      mysqlLastExportedFlows = mysqlExportedFlows;
 
-  return strdup(sql);
+      mysqlExportRate = ((float)(diffFlows * 1000)) / tdiffMsec;
+      if (mysqlExportRate < 0) mysqlExportRate = 0;
+    }
+  }
+
+  memcpy(&lastUpdateTime, tv, sizeof(struct timeval));
 }
-#endif
 
 /* ******************************************* */
 
-bool MySQLDB::dumpFlow(time_t when, bool partial_dump, Flow *f, char *json) {
-  char sql[CONST_MAX_SQL_QUERY_LEN], cli_str[64], srv_str[64], *json_buf;
+void MySQLDB::lua(lua_State *vm, bool since_last_checkpoint) const {
+  lua_push_int_table_entry(vm, "flow_export_count",
+			   mysqlExportedFlows - (since_last_checkpoint ? checkpointExportedFlows : 0));
+  lua_push_int32_table_entry(vm, "flow_export_drops",
+			     mysqlDroppedFlows - (since_last_checkpoint ? checkpointDroppedFlows : 0));
+  lua_push_float_table_entry(vm, "flow_export_rate",
+			     mysqlExportRate >= 0 ? mysqlExportRate : 0);
+}
+
+/* ******************************************* */
+
+char* MySQLDB::escapeAphostrophes(const char *unescaped) {
+  char *buf;
+  int l, i, j;
+
+  if(!unescaped)
+    return NULL;
+
+  l = strlen(unescaped);
+
+  if((buf = (char*)malloc(2*l + 1)) == NULL) {
+    ntop->getTrace()->traceEvent(TRACE_WARNING, "Not enough memory");
+    return NULL;
+  }
+
+  for(i = 0, j = 0; i<l; i++) {
+    /* http://stackoverflow.com/questions/9596652/how-to-escape-apostrophe-in-mysql */
+    if(unescaped[i] == '\'')
+	buf[j++] = '\'';
+
+      buf[j++] = unescaped[i];
+    }
+
+  buf[j] = '\0';
+
+  return buf;
+}
+
+/* ******************************************* */
+
+int MySQLDB::flow2InsertValues(Flow *f, char *json,
+			       char *values_buf, size_t values_buf_len) const {
+  char cli_str[64], srv_str[64], *json_buf, *info_buf;
   u_int32_t packets, first_seen, last_seen;
   u_int32_t bytes_cli2srv, bytes_srv2cli;
+  size_t len;
 
-  if((f->get_cli_host() == NULL) || (f->get_srv_host() == NULL))
-    return(false);
+  if(!values_buf || !values_buf_len || !f)
+    return -1;
 
-  if(json == NULL)
-    json_buf = strdup("");
-  else {
-    int l, i, j;
+  json_buf = escapeAphostrophes(json);
+  info_buf = escapeAphostrophes(f->getFlowInfo());
 
-    l = strlen(json);
+  /* Prevents ERROR 1406 (22001): Data too long for column 'INFO' at row 1 */
+  if(info_buf && strlen(info_buf) > 254)
+    info_buf[255] = '\0';
 
-    if((json_buf = (char*)malloc(2*l + 1)) == NULL) {
-      ntop->getTrace()->traceEvent(TRACE_WARNING, "Not enough memory");
-      return(false);
-    }
-
-    for(i = 0, j = 0; i<l; i++) {
-      /* http://stackoverflow.com/questions/9596652/how-to-escape-apostrophe-in-mysql */
-      if(json[i] == '\'')
-	json_buf[j++] = '\'';
-
-      json_buf[j++] = json[i];
-    }
-
-    json_buf[j] = '\0';
-  }
-
-  if(partial_dump) {
-    bytes_cli2srv = f->get_partial_bytes_cli2srv();
-    bytes_srv2cli = f->get_partial_bytes_srv2cli();
-    packets = f->get_partial_packets();
-    first_seen = f->get_partial_first_seen();
-    last_seen = f->get_partial_last_seen();
-  } else {
-    bytes_cli2srv = f->get_bytes_cli2srv();
-    bytes_srv2cli = f->get_bytes_srv2cli();
-    packets = f->get_packets();
-    first_seen = f->get_first_seen();
-    last_seen = f->get_last_seen();
-  }
+  /* Use of partial_ functions is safe as they will deal with partial dumps automatically */
+  bytes_cli2srv = f->get_partial_bytes_cli2srv();
+  bytes_srv2cli = f->get_partial_bytes_srv2cli();
+  packets = f->get_partial_packets();
+  first_seen = f->get_partial_first_seen();
+  last_seen = f->get_partial_last_seen();
 
   if(f->get_cli_host()->get_ip()->isIPv4()) {
-    snprintf(sql, sizeof(sql),
-	     "INSERT INTO `%sv4` (VLAN_ID,L7_PROTO,IP_SRC_ADDR,L4_SRC_PORT,IP_DST_ADDR,L4_DST_PORT,PROTOCOL,"
-	     "IN_BYTES,OUT_BYTES,PACKETS,FIRST_SWITCHED,LAST_SWITCHED,INFO,JSON,NTOPNG_INSTANCE_NAME,INTERFACE_ID"
+    len = snprintf(values_buf, values_buf_len,
+		   MYSQL_INSERT_VALUES_V4,
+		   f->get_vlan_id(),
+		   f->get_detected_protocol().app_protocol,
+		   htonl(f->get_cli_host()->get_ip()->get_ipv4()),
+		   f->get_cli_port(),
+		   htonl(f->get_srv_host()->get_ip()->get_ipv4()),
+		   f->get_srv_port(),
+		   f->get_protocol(),
+		   bytes_cli2srv, bytes_srv2cli,
+		   packets, first_seen, last_seen,
+		   info_buf ? info_buf : "",
+		   json_buf ? json_buf : "",
+		   ntop->getPrefs()->get_instance_name(),
+		   iface->get_id()
 #ifdef NTOPNG_PRO
-	     ",PROFILE"
+		   ,f->get_profile_name()
 #endif
-	     ") VALUES ('%u','%u','%u','%u','%u','%u','%u','%u','%u','%u','%u','%u','%s',COMPRESS('%s'), '%s', '%u'"
-#ifdef NTOPNG_PRO
-	     ",'%s'"  // this is the string for traffic profile
-#endif
-	     ") ",
-	     ntop->getPrefs()->get_mysql_tablename(),
-	     f->get_vlan_id(),
-	     f->get_detected_protocol().protocol,
-	     htonl(f->get_cli_host()->get_ip()->get_ipv4()),
-	     f->get_cli_port(),
-	     htonl(f->get_srv_host()->get_ip()->get_ipv4()),
-	     f->get_srv_port(),
-	     f->get_protocol(),
-	     bytes_cli2srv, bytes_srv2cli,
-	     packets, first_seen, last_seen,
-	     f->getFlowServerInfo() ? f->getFlowServerInfo() : "",
-	     json_buf,
-	     ntop->getPrefs()->get_instance_name(),
-	     iface->get_id()
-#ifdef NTOPNG_PRO
-	     ,f->get_profile_name()
-#endif
-	     );
+		   );
   }  else {
-    snprintf(sql, sizeof(sql),
-	     "INSERT INTO `%sv6` (VLAN_ID,L7_PROTO,IP_SRC_ADDR,L4_SRC_PORT,IP_DST_ADDR,L4_DST_PORT,PROTOCOL,"
-	     "IN_BYTES,OUT_BYTES,PACKETS,FIRST_SWITCHED,LAST_SWITCHED,INFO,JSON,NTOPNG_INSTANCE_NAME,INTERFACE_ID"
+    len = snprintf(values_buf, values_buf_len,
+		   MYSQL_INSERT_VALUES_V6,
+		   f->get_vlan_id(),
+		   f->get_detected_protocol().app_protocol,
+		   f->get_cli_host()->get_ip()->print(cli_str, sizeof(cli_str)),
+		   f->get_cli_port(),
+		   f->get_srv_host()->get_ip()->print(srv_str, sizeof(srv_str)),
+		   f->get_srv_port(),
+		   f->get_protocol(),
+		   bytes_cli2srv, bytes_srv2cli,
+		   packets, first_seen, last_seen,
+		   info_buf ? info_buf : "",
+		   json_buf ? json_buf : "",
+		   ntop->getPrefs()->get_instance_name(),
+		   iface->get_id()
 #ifdef NTOPNG_PRO
-	     ",PROFILE"
+		   ,f->get_profile_name()
 #endif
-	     ") VALUES ('%u','%u','%s','%u','%s','%u','%u','%u','%u','%u','%u','%u','%s',COMPRESS('%s'), '%s', '%u'"
-#ifdef NTOPNG_PRO
-	     ",'%s'"  // this is the string for traffic profile
-#endif
-	     ") ",
-	     ntop->getPrefs()->get_mysql_tablename(),
-	     f->get_vlan_id(),
-	     f->get_detected_protocol().protocol,
-	     f->get_cli_host()->get_ip()->print(cli_str, sizeof(cli_str)),
-	     f->get_cli_port(),
-	     f->get_srv_host()->get_ip()->print(srv_str, sizeof(srv_str)),
-	     f->get_srv_port(),
-	     f->get_protocol(),
-	     bytes_cli2srv, bytes_srv2cli,
-	     packets, first_seen, last_seen,
-	     f->getFlowServerInfo() ? f->getFlowServerInfo() : "",
-	     json_buf,
-	     ntop->getPrefs()->get_instance_name(),
-	     iface->get_id()
-#ifdef NTOPNG_PRO
-	     ,f->get_profile_name()
-#endif
-	     );
+		   );
   }
-  free(json_buf);
 
-  ntop->getRedis()->lpush(CONST_SQL_QUEUE, sql, CONST_MAX_MYSQL_QUEUE_LEN);
+  if(json_buf)
+    free(json_buf);
+  if(info_buf)
+    free(info_buf);
+
+  return len;
+}
+
+/* ******************************************* */
+
+bool MySQLDB::dumpFlow(time_t when, Flow *f, char *json) {
+  char sql[CONST_MAX_SQL_QUERY_LEN];
+
+  if((f->get_cli_host() == NULL) || (f->get_srv_host() == NULL) || !MySQLDB::db_created)
+    return(false);
+
+  if(f->get_cli_host()->get_ip()->isIPv4())
+    snprintf(sql, sizeof(sql), "INSERT INTO `%sv4` " MYSQL_INSERT_FIELDS " VALUES ",
+	     ntop->getPrefs()->get_mysql_tablename());
+  else
+    snprintf(sql, sizeof(sql), "INSERT INTO `%sv6` " MYSQL_INSERT_FIELDS " VALUES ",
+	     ntop->getPrefs()->get_mysql_tablename());
+
+  /* do the actual flow insertion as a tuple */
+  flow2InsertValues(f, json, &sql[strlen(sql)], sizeof(sql) - strlen(sql) - 1);
+
+  if (ntop->getRedis()->llen(CONST_SQL_QUEUE) < CONST_MAX_MYSQL_QUEUE_LEN) {
+    /* We know that we should have locked before evaluating the condition
+       above as another thread, via the lpush below, can invalidate the condition.
+       However, we prefer to avoid an additional lock as the lpush guarantees
+       that no more than CONST_MAX_MYSQL_QUEUE_LEN will ever be in the queue.
+       The drawback is that the counter mysqlDroppedFlows
+       is not guaranteed to be 100% accurate but we can tolerate this.
+    */
+    ntop->getRedis()->lpush(CONST_SQL_QUEUE, sql, CONST_MAX_MYSQL_QUEUE_LEN);
+  } else {
+    mysqlDroppedFlows++;
+  }
 
   return(true);
 }
 
 /* ******************************************* */
 
+void MySQLDB::disconnectFromDB(MYSQL *conn) {
+  mysql_close(conn);
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Disconnected from MySQL for interface %s...",
+			       iface->get_name() ? iface->get_name() : "<unknown>");
+}
+
+/* ******************************************* */
+
 bool MySQLDB::connectToDB(MYSQL *conn, bool select_db) {
   MYSQL *rc;
   unsigned long flags = CLIENT_COMPRESS;
@@ -503,7 +690,7 @@ bool MySQLDB::connectToDB(MYSQL *conn, bool select_db) {
     return(false);
   }
 
-  if(ntop->getPrefs()->get_mysql_host()[0] == '/') /* Use socket */
+  if(ntop->getPrefs()->get_mysql_host()[0] == '/') /* Use socketD */
     rc = mysql_real_connect(conn,
 			    NULL, /* Host */
 			    ntop->getPrefs()->get_mysql_user(),
@@ -517,14 +704,15 @@ bool MySQLDB::connectToDB(MYSQL *conn, bool select_db) {
 			    ntop->getPrefs()->get_mysql_user(),
 			    ntop->getPrefs()->get_mysql_pw(),
 			    dbname,
-			    3306 /* port */,
+			    ntop->getPrefs()->get_mysql_port(),
 			    NULL /* socket */, flags);
 
   if(rc == NULL) {
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Failed to connect to MySQL: %s [%s:%s]\n",
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "Failed to connect to MySQL: %s [%s@%s:%i]\n",
 				 mysql_error(conn),
+				 ntop->getPrefs()->get_mysql_user(),
 				 ntop->getPrefs()->get_mysql_host(),
-				 ntop->getPrefs()->get_mysql_user());
+                 ntop->getPrefs()->get_mysql_port());
 
     if(m) m->unlock(__FILE__, __LINE__);
     return(false);
@@ -532,9 +720,10 @@ bool MySQLDB::connectToDB(MYSQL *conn, bool select_db) {
 
   db_operational = true;
 
-  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Succesfully connected to MySQL [%s:%s] for interface %s",
-			       ntop->getPrefs()->get_mysql_host(),
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Successfully connected to MySQL [%s@%s:%i] for interface %s",
 			       ntop->getPrefs()->get_mysql_user(),
+			       ntop->getPrefs()->get_mysql_host(),
+			       ntop->getPrefs()->get_mysql_port(),
 			       iface->get_name());
 
   if(m) m->unlock(__FILE__, __LINE__);
@@ -545,14 +734,17 @@ bool MySQLDB::connectToDB(MYSQL *conn, bool select_db) {
 
 /*
   Locking is necessary when multiple queries are executed
-  simulatenously (e.g. via Lua)
+  simultaneously (e.g. via Lua)
 */
-int MySQLDB::exec_sql_query(MYSQL *conn, char *sql,
+int MySQLDB::exec_sql_query(MYSQL *conn, const char *sql,
 			    bool doReconnect, bool ignoreErrors,
 			    bool doLock) {
   int rc;
   MYSQL_RES *result;
 
+  /* Don't check db_created here. This method is private
+     so hopefully we know what we're doing.
+   */
   if(!db_operational)
     return(-2);
 
@@ -566,18 +758,18 @@ int MySQLDB::exec_sql_query(MYSQL *conn, char *sql,
     case CR_SERVER_GONE_ERROR:
     case CR_SERVER_LOST:
       if(doReconnect) {
-	mysql_close(conn);
+	disconnectFromDB(conn);
 	if(doLock && m) m->unlock(__FILE__, __LINE__);
 
 	connectToDB(conn, true);
 
 	return(exec_sql_query(conn, sql, false));
       } else
-	ntop->getTrace()->traceEvent(TRACE_INFO, "%s", sql);
+	ntop->getTrace()->traceEvent(TRACE_INFO, "[%s][%s]", get_last_db_error(conn), sql);
       break;
 
     default:
-      ntop->getTrace()->traceEvent(TRACE_INFO, "%s", sql);
+      ntop->getTrace()->traceEvent(TRACE_INFO, "[%s][%s]", get_last_db_error(conn), sql);
       break;
     }
 
@@ -587,10 +779,13 @@ int MySQLDB::exec_sql_query(MYSQL *conn, char *sql,
     // we want to return the number of rows which is more informative
     // than a simple 0
     if((result = mysql_store_result(&mysql)) == NULL)
-      rc = 0;  // unable to retrieve the result but still the query succeded
-    else{
-      mysql_free_result(result);
+      rc = 0;  // unable to retrieve the result but still the query succeeded
+    else {
       rc = mysql_num_rows(result);
+      ntop->getTrace()->traceEvent(TRACE_INFO,
+				   "Current result set has %lu rows",
+				   (unsigned long)rc);
+      mysql_free_result(result);
     }
   }
 
@@ -601,24 +796,21 @@ int MySQLDB::exec_sql_query(MYSQL *conn, char *sql,
 
 /* ******************************************* */
 
-int MySQLDB::exec_sql_query(lua_State *vm, char *sql, bool limitRows) {
+int MySQLDB::exec_sql_query(lua_State *vm, char *sql, bool limitRows, bool wait_for_db_created) {
   MYSQL_RES *result;
   MYSQL_ROW row;
   char *fields[MYSQL_MAX_NUM_FIELDS] = { NULL };
   int num_fields, rc, num = 0;
 
-  if(!db_operational)
+  if((wait_for_db_created && !MySQLDB::db_created /* Make sure the db exists before doing queries */)
+     || !db_operational)
     return(-2);
 
   if(m) m->lock(__FILE__, __LINE__);
 
   if((rc = mysql_query(&mysql, sql)) != 0) {
-    rc = mysql_errno(&mysql);
-
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: [%s][%d]",
-				 get_last_db_error(&mysql), rc);
-
-    mysql_close(&mysql);
+    /* retry */
+    disconnectFromDB(&mysql);
     if(m) m->unlock(__FILE__, __LINE__);
     connectToDB(&mysql, true);
 
@@ -629,40 +821,57 @@ int MySQLDB::exec_sql_query(lua_State *vm, char *sql, bool limitRows) {
     rc = mysql_query(&mysql, sql);
   }
 
-  if((rc != 0) || ((result = mysql_store_result(&mysql)) == NULL)) {
-    lua_pushstring(vm, get_last_db_error(&mysql));
+  if((rc != 0)
+     || (((result = mysql_store_result(&mysql)) == NULL)
+	 && mysql_field_count(&mysql) != 0 /* mysql_store_result() returned nothing; should it have? */)) {
+    rc = mysql_errno(&mysql);
+
+    if(rc) {
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: [%s][%d][%s]",
+				   get_last_db_error(&mysql), rc, sql);
+      lua_pushstring(vm, get_last_db_error(&mysql));
+    } else {
+      lua_pushnil(vm);
+    }
+
     if(m) m->unlock(__FILE__, __LINE__);
     return(rc);
   }
 
-  num_fields = min_val(mysql_num_fields(result), MYSQL_MAX_NUM_FIELDS);
-  lua_newtable(vm);
-
-  num = 0;
-  while((row = mysql_fetch_row(result))) {
+  if((result == NULL) || (mysql_field_count(&mysql) == 0)) {
+    lua_pushnil(vm);
+  } else {
+    num_fields = min_val(mysql_num_fields(result), MYSQL_MAX_NUM_FIELDS);
     lua_newtable(vm);
 
-    if(num == 0) {
-      for(int i = 0; i < num_fields; i++) {
-	MYSQL_FIELD *field = mysql_fetch_field(result);
+    num = 0;
+    while((row = mysql_fetch_row(result))) {
+      lua_newtable(vm);
 
-	fields[i] = field->name;
+      if(num == 0) {
+	for(int i = 0; i < num_fields; i++) {
+	  MYSQL_FIELD *field = mysql_fetch_field(result);
+
+	  fields[i] = field->name;
+	}
       }
+
+      for(int i = 0; i < num_fields; i++)
+	lua_push_str_table_entry(vm, (const char*)fields[i], row[i] ? row[i] : (char*)"");
+
+      lua_pushnumber(vm, ++num);
+      lua_insert(vm, -2);
+      lua_settable(vm, -3);
+
+      if(limitRows && num >= MYSQL_MAX_NUM_ROWS) break;
     }
 
-    for(int i = 0; i < num_fields; i++)
-      lua_push_str_table_entry(vm, (const char*)fields[i], row[i] ? row[i] : (char*)"");
-
-    lua_pushnumber(vm, ++num);
-    lua_insert(vm, -2);
-    lua_settable(vm, -3);
-
-    if(limitRows && num >= MYSQL_MAX_NUM_ROWS) break;
+    mysql_free_result(result);
   }
-
-  mysql_free_result(result);
-
+  
   if(m) m->unlock(__FILE__, __LINE__);
 
   return(0);
 }
+
+#endif
