@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,7 @@ NtopGlobals::NtopGlobals() {
   file_id = 0;
   trace = new Trace();  
   mutex = new Mutex();
-  is_shutdown = false, do_decode_tunnels = true;
+  is_shutdown = shutdown_requested = false, do_decode_tunnels = true;
 };
 
 /* **************************************** */
