@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -57,18 +57,29 @@ PF_RINGInterface::PF_RINGInterface(const char *name) : NetworkInterface(name) {
 
   pcap_datalink_type = DLT_EN10MB;
 
-  pfring_set_direction(pfring_handle, rx_only_direction);
   pfring_set_poll_watermark(pfring_handle, 8);
   pfring_set_application_name(pfring_handle, (char*)"ntopng");
-
+  pfring_enable_rss_rehash(pfring_handle);
+  
   switch(ntop->getPrefs()->getCaptureDirection()) {
   case PCAP_D_INOUT: direction = rx_and_tx_direction; break;
-  case PCAP_D_IN:    direction = rx_only_direction; break;
-  case PCAP_D_OUT:   direction = tx_only_direction; break;
+  case PCAP_D_IN:    direction = rx_only_direction;   break;
+  case PCAP_D_OUT:   direction = tx_only_direction;   break;
   }
 
   if(pfring_set_direction(pfring_handle, direction) != 0)
     ntop->getTrace()->traceEvent(TRACE_WARNING, "Unable to set packet capture direction");
+
+  if(pfring_set_socket_mode(pfring_handle, recv_only_mode) != 0)
+    ntop->getTrace()->traceEvent(TRACE_WARNING, "Unable to set socket mode");
+  
+  memset(&last_pfring_stat, 0, sizeof(last_pfring_stat));
+
+  /* 
+     We need to enable the ring here and not in packetPollLoop() as otherwise
+     with ZC we cannot allocate hugepages after we switched to nobody
+  */
+  pfring_enable_ring(pfring_handle);
 }
 
 /* **************************************************** */
@@ -85,13 +96,15 @@ PF_RINGInterface::~PF_RINGInterface() {
 static void* packetPollLoop(void* ptr) {
   PF_RINGInterface *iface = (PF_RINGInterface*)ptr;
   pfring  *pd = iface->get_pfring_handle();
-
+  u_int sleep_time, max_sleep = 1000, step_sleep = 100;
+  
   /* Wait until the initialization completes */
   while(!iface->isRunning()) sleep(1);
-  pfring_enable_ring(pd);
 
   while(iface->idle()) { iface->purgeIdle(time(NULL)); sleep(1); }
 
+  sleep_time = step_sleep;
+  
   while(iface->isRunning()) {
     if(pfring_is_pkt_available(pd)) {
       u_char *buffer;
@@ -99,12 +112,17 @@ static void* packetPollLoop(void* ptr) {
 
       if(pfring_recv(pd, &buffer, 0, &hdr, 0 /* wait_for_packet */) > 0) {
 	try {
-	  bool shaped;
 	  u_int16_t p;
+	  Host *srcHost = NULL, *dstHost = NULL;
+	  Flow *flow = NULL;
 
 	  if(hdr.ts.tv_sec == 0) gettimeofday(&hdr.ts, NULL);
-	  iface->dissectPacket((const struct pcap_pkthdr *) &hdr, buffer,
-			       &shaped, &p);
+	  iface->dissectPacket(DUMMY_BRIDGE_INTERFACE_ID,
+			       (hdr.extended_hdr.rx_direction == 1) ? 
+			       true /* ingress */ : false /* egress */,
+			       NULL, (const struct pcap_pkthdr *) &hdr, buffer,
+			       &p, &srcHost, &dstHost, &flow);
+	  sleep_time = step_sleep;
 	} catch(std::bad_alloc& ba) {
 	  static bool oom_warning_sent = false;
 
@@ -115,7 +133,8 @@ static void* packetPollLoop(void* ptr) {
 	}
       }
     } else {
-      usleep(1);
+      if(sleep_time < max_sleep) sleep_time += step_sleep;
+      usleep(sleep_time);
       iface->purgeIdle(time(NULL));
     }
   }
@@ -147,7 +166,7 @@ void PF_RINGInterface::shutdown() {
 
 /* **************************************************** */
 
-u_int PF_RINGInterface::getNumDroppedPackets() {
+u_int32_t PF_RINGInterface::getNumDroppedPackets() {
   pfring_stat stats;
 
   if(pfring_stats(pfring_handle, &stats) >= 0) {
@@ -158,7 +177,7 @@ u_int PF_RINGInterface::getNumDroppedPackets() {
     return(stats.drop);
   }
 
-  return(0);
+  return 0;
 }
 
 /* **************************************************** */
