@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,8 @@
 #include <uuid/uuid.h>
 #endif
 
+#ifndef HAVE_NEDGE
+
 /* **************************************************** */
 
 PcapInterface::PcapInterface(const char *name) : NetworkInterface(name) {
@@ -32,6 +34,7 @@ PcapInterface::PcapInterface(const char *name) : NetworkInterface(name) {
   struct stat buf;
 
   pcap_handle = NULL, pcap_list = NULL;
+  memset(&last_pcap_stat, 0, sizeof(last_pcap_stat));
   
   if((stat(name, &buf) == 0) || (name[0] == '-') || !strncmp(name, "stdin", 5)) {
     /*
@@ -65,7 +68,7 @@ PcapInterface::PcapInterface(const char *name) : NetworkInterface(name) {
   } else {
     pcap_handle = pcap_open_live(ifname, ntop->getGlobals()->getSnaplen(),
 				 ntop->getPrefs()->use_promiscuous(),
-				 500, pcap_error_buffer);  
+				 1000 /* 1 sec */, pcap_error_buffer);  
 
     if(pcap_handle) {
       char *bl = strrchr(ifname, 
@@ -136,7 +139,7 @@ static void* packetPollLoop(void* ptr) {
 	  ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to open file '%s': %s", 
 				       path, pcap_error_buffer);
 	} else {
-	  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Reading packes from pcap file %s", path);
+	  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Reading packets from pcap file %s", path);
 	  iface->set_pcap_handle(pcap_handle);
 	  break;
 	}
@@ -162,9 +165,10 @@ static void* packetPollLoop(void* ptr) {
       while(iface->idle()) { iface->purgeIdle(time(NULL)); sleep(1); }
 
       if((rc = pcap_next_ex(pd, &hdr, &pkt)) > 0) {
-	if((rc > 0) && (pkt != NULL) && (hdr->caplen > 0)) {
+	if((pkt != NULL) && (hdr->caplen > 0)) {
 	  u_int16_t p;
-	  bool shaped;
+	  Host *srcHost = NULL, *dstHost = NULL;
+	  Flow *flow = NULL;
 
 #ifdef WIN32
 	  /*
@@ -181,10 +185,14 @@ static void* packetPollLoop(void* ptr) {
 	  hdr_copy.len = min(hdr->len, sizeof(pkt_copy) - 1);
 	  hdr_copy.caplen = min(hdr_copy.len, hdr_copy.caplen);
 	  memcpy(pkt_copy, pkt, hdr_copy.len);
-	  iface->dissectPacket(&hdr_copy, (const u_char*)pkt_copy, &shaped, &p);
+	  iface->dissectPacket(DUMMY_BRIDGE_INTERFACE_ID,
+			       true /* ingress - TODO: see if we pass the real packet direction */,
+			       NULL, &hdr_copy, (const u_char*)pkt_copy, &p, &srcHost, &dstHost, &flow);
 #else
 	  hdr->caplen = min_val(hdr->caplen, iface->getMTU());
-	  iface->dissectPacket(hdr, pkt, &shaped, &p);
+	  iface->dissectPacket(DUMMY_BRIDGE_INTERFACE_ID,
+			       true /* ingress - TODO: see if we pass the real packet direction */,
+			       NULL, hdr, pkt, &p, &srcHost, &dstHost, &flow);
 #endif
 	}
       } else if(rc < 0) {
@@ -227,13 +235,13 @@ void PcapInterface::shutdown() {
 
 /* **************************************************** */
 
-u_int PcapInterface::getNumDroppedPackets() {
+u_int32_t PcapInterface::getNumDroppedPackets() {
   struct pcap_stat pcapStat;
 
   if(pcap_handle && (pcap_stats(pcap_handle, &pcapStat) >= 0)) {
     return(pcapStat.ps_drop);
   } else
-    return(0);
+    return 0;
 }
 
 /* **************************************************** */
@@ -248,10 +256,12 @@ bool PcapInterface::set_packet_filter(char *filter) {
 
   if((pcap_compile(pcap_handle, &fcode, filter, 1, netmask.s_addr) < 0)
      || (pcap_setfilter(pcap_handle, &fcode) < 0)) {
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to set on %s filter %s. Filter ignored.\n", ifname, filter);
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to set on %s filter %s. Filter ignored.", ifname, filter);
     return(false);
   } else {
     ntop->getTrace()->traceEvent(TRACE_NORMAL, "Packet capture filter on %s set to \"%s\"", ifname, filter);
     return(true);
   }
 };
+
+#endif
