@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,32 +27,39 @@
 #include "third-party/hiredis/sds.c"
 #endif
 
+// #define CACHE_DEBUG 1
+
 /* **************************************** */
 
-Redis::Redis(char *_redis_host, u_int16_t _redis_port, u_int8_t _redis_db_id) {
-  redis_host = _redis_host, redis_port= _redis_port, redis_db_id = _redis_db_id;
+Redis::Redis(const char *_redis_host, const char *_redis_password, u_int16_t _redis_port, u_int8_t _redis_db_id) {
+  redis_host = _redis_host ? strdup(_redis_host) : NULL;
+  redis_password = _redis_password ? strdup(_redis_password) : NULL;
+  redis_port = _redis_port, redis_db_id = _redis_db_id;
 
+  num_requests = num_reconnections = 0;
   redis = NULL, operational = false;
   reconnectRedis();
-
+  stringCache = NULL, numCached = 0;
   l = new Mutex();
-  setDefaults();
 }
 
 /* **************************************** */
 
 Redis::~Redis() {
+  flushCache();
   redisFree(redis);
+  // flushCache();
   delete l;
+  if(redis_host)     free(redis_host);
+  if(redis_password) free(redis_password);
 }
 
-
 /* **************************************** */
 
 void Redis::reconnectRedis() {
   struct timeval timeout = { 1, 500000 }; // 1.5 seconds
   redisReply *reply;
-  u_int num_attemps = 10;
+  u_int num_attempts = 10;
 
   operational = false;
 
@@ -63,12 +70,23 @@ void Redis::reconnectRedis() {
 
   redis = redisConnectWithTimeout(redis_host, redis_port, timeout);
 
-  while(num_attemps > 0) {
-    if(redis) reply = (redisReply*)redisCommand(redis, "PING"); else reply = NULL;
+  if(redis_password) {
+    num_requests++;
+    reply = (redisReply*)redisCommand(redis, "AUTH %s", redis_password);
+    if(reply && (reply->type == REDIS_REPLY_ERROR)) {
+      ntop->getTrace()->traceEvent(TRACE_ERROR,
+				   "Redis authentication failed: %s", reply->str ? reply->str : "???");
+      goto redis_error_handler;
+    }
+  }
+
+  while(redis && num_attempts > 0) {
+    num_requests++;
+    reply = (redisReply*)redisCommand(redis, "PING");
     if(reply && (reply->type == REDIS_REPLY_ERROR)) {
       ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
       sleep(1);
-      num_attemps--;
+      num_attempts--;
     } else
       break;
   }
@@ -83,6 +101,7 @@ void Redis::reconnectRedis() {
   } else {
     freeReplyObject(reply);
 
+    num_requests++;
     reply = (redisReply*)redisCommand(redis, "SELECT %u", redis_db_id);
     if(reply && (reply->type == REDIS_REPLY_ERROR)) {
       ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
@@ -96,16 +115,24 @@ void Redis::reconnectRedis() {
     }
   }
 
+  num_reconnections++;
 }
 
 /* **************************************** */
 
-int Redis::expire(char *key, u_int expire_sec) {
+int Redis::expire(char *key, u_int expire_secs) {
   int rc;
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
-  reply = (redisReply*)redisCommand(redis, "EXPIRE %s %u", key, expire_sec);
+
+  if(expireCache(key, expire_secs)) {
+    l->unlock(__FILE__, __LINE__);
+    return(0);
+  }
+
+  num_requests++;
+  reply = (redisReply*)redisCommand(redis, "EXPIRE %s %u", key, expire_secs);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
     ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
@@ -117,24 +144,117 @@ int Redis::expire(char *key, u_int expire_sec) {
 
 /* **************************************** */
 
-int Redis::get(char *key, char *rsp, u_int rsp_len) {
+bool Redis::isCacheable(char *key) {
+  if((strstr(key, "ntopng.cache."))
+     || (strstr(key, "ntopng.user.") && (!strstr(key, ".password"))))
+    return(true);
+
+  return(false);
+}
+
+/* **************************************** */
+
+bool Redis::expireCache(char *key, u_int expire_secs) {
+  StringCache_t *cached = NULL;
+
+#ifdef CACHE_DEBUG
+  printf("**** Setting cache expire for %s [%u sec]\n", key, expire_secs);
+#endif
+
+  HASH_FIND_STR(stringCache, key, cached);
+
+  if(cached) {
+    cached->expire = expire_secs ? time(NULL)+expire_secs : 0;
+    return(true);
+  }
+
+  return(false);
+}
+
+/* **************************************** */
+
+/* NOTE: We assume that the addToCache() caller locks this instance */
+void Redis::addToCache(char *key, char *value, u_int expire_secs) {
+  StringCache_t *cached = NULL;
+
+#ifdef CACHE_DEBUG
+  printf("**** Caching %s=%s\n", key, value ? value : "<NULL>");
+#endif
+
+  HASH_FIND_STR(stringCache, key, cached);
+
+  if(cached) {
+    if(cached->value) free(cached->value);
+    cached->value = strdup(value);
+    cached->expire = expire_secs ? time(NULL)+expire_secs : 0;
+    return;
+  }
+
+  cached = (StringCache_t*)malloc(sizeof(StringCache_t));
+  if(cached) {
+    cached->key = strdup(key), cached->value = strdup(value);
+    cached->expire = expire_secs ? time(NULL)+expire_secs : 0;
+
+    if(cached->key && cached->value) {
+      HASH_ADD_STR(stringCache, key, cached);
+      numCached++;
+    } else {
+      if(cached->key)   free(cached->key);
+      if(cached->value) free(cached->value);
+      free(cached);
+    }
+  } else
+    ntop->getTrace()->traceEvent(TRACE_WARNING, "Not enough memory");
+}
+
+/* **************************************** */
+
+int Redis::get(char *key, char *rsp, u_int rsp_len, bool cache_it) {
   int rc;
+  bool cacheable = false;
   redisReply *reply;
+  StringCache_t *cached = NULL;
 
   l->lock(__FILE__, __LINE__);
+
+  HASH_FIND_STR(stringCache, key, cached);
+  if(cached) {
+    snprintf(rsp, rsp_len, "%s", cached->value);
+
+#ifdef CACHE_DEBUG
+    printf("**** Read from cache %s=%s\n", key, rsp);
+#endif
+    l->unlock(__FILE__, __LINE__);
+    return(rsp[0] == '\0' ? -1 : 0);
+  } else {
+#ifdef CACHE_DEBUG
+    printf("**** Unable to find on cache %s\n", key);
+#endif
+  }
+
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "GET %s", key);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
     ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
 
+  cacheable = isCacheable(key);
   if(reply && reply->str) {
     snprintf(rsp, rsp_len, "%s", reply->str), rc = 0;
   } else
     rsp[0] = 0, rc = -1;
 
+  if(cache_it || cacheable)
+    addToCache(key, rsp, 0);
+
   if(reply) freeReplyObject(reply);
   l->unlock(__FILE__, __LINE__);
 
+  if(cacheable && (rc == -1)) {
+    /* Add default */
+    set(key, (char*)"", 0);
+  }
+
   return(rc);
 }
 /* **************************************** */
@@ -142,8 +262,20 @@ int Redis::get(char *key, char *rsp, u_int rsp_len) {
 int Redis::del(char *key){
   int rc;
   redisReply *reply;
+  StringCache_t *cached = NULL;
 
   l->lock(__FILE__, __LINE__);
+
+  HASH_FIND_STR(stringCache, key, cached);
+
+  if(cached) {
+    HASH_DEL(stringCache, cached);
+    if(cached->key)   free(cached->key);
+    if(cached->value) free(cached->value);
+    free(cached);
+  }
+
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "DEL %s", key);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR)){
@@ -166,6 +298,7 @@ int Redis::hashGet(char *key, char *field, char *rsp, u_int rsp_len) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "HGET %s %s", key, field);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -188,6 +321,7 @@ int Redis::hashSet(char *key, char *field, char *value) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "HSET %s %s %s", key, field, value);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -205,6 +339,7 @@ int Redis::hashDel(char *key, char *field) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "HDEL %s %s", key, field);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -226,6 +361,11 @@ int Redis::set(char *key, char *value, u_int expire_secs) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+
+  if(isCacheable(key))
+    addToCache(key, value, expire_secs);
+
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "SET %s %s", key, value);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -233,6 +373,7 @@ int Redis::set(char *key, char *value, u_int expire_secs) {
   if(reply) freeReplyObject(reply), rc = 0; else rc = -1;
 
   if((rc == 0) && (expire_secs != 0)) {
+    num_requests++;
     reply = (redisReply*)redisCommand(redis, "EXPIRE %s %u", key, expire_secs);
     if(!reply) reconnectRedis();
     if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -250,6 +391,7 @@ char* Redis::popSet(char *pop_name, char *rsp, u_int rsp_len) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "SPOP %s", pop_name);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -276,6 +418,7 @@ u_int32_t Redis::incrKey(char *key) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "INCR %s", key);
   if(reply) {
     if(reply->type == REDIS_REPLY_ERROR)
@@ -304,6 +447,7 @@ int Redis::zIncr(char *key, char *member) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "ZINCRBY %s 1 %s", key, member);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -324,6 +468,7 @@ int Redis::zTrim(char *key, u_int trim_len) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "ZREMRANGEBYRANK %s 0 %d", key, -1*trim_len);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -342,6 +487,7 @@ int Redis::zRevRange(const char *pattern, char ***keys_p) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "ZREVRANGE %s 0 -1 WITHSCORES", pattern);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -370,6 +516,7 @@ int Redis::keys(const char *pattern, char ***keys_p) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "KEYS %s", pattern);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -398,6 +545,7 @@ int Redis::hashKeys(const char *pattern, char ***keys_p) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "HKEYS %s", pattern);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -425,11 +573,56 @@ int Redis::hashKeys(const char *pattern, char ***keys_p) {
 
 /* **************************************** */
 
+int Redis::hashGetAll(const char *key, char ***keys_p, char ***values_p) {
+  int rc = 0;
+  int i, j;
+  redisReply *reply;
+
+  l->lock(__FILE__, __LINE__);
+  num_requests++;
+  reply = (redisReply*)redisCommand(redis, "HGETALL %s", key);
+  if(!reply) reconnectRedis();
+  if(reply && (reply->type == REDIS_REPLY_ERROR))
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s [HGETALL %s]", reply->str ? reply->str : "???", key);
+
+  (*keys_p) = NULL;
+
+  if(reply && (reply->type == REDIS_REPLY_ARRAY) && (reply->elements % 2 == 0)) {
+    rc = (int)reply->elements / 2;
+
+    if(rc > 0) {
+      if(((*keys_p) = (char**)malloc(rc * sizeof(char*))) != NULL) {
+        if(((*values_p) = (char**)malloc(rc * sizeof(char*))) != NULL) {
+
+          i = 0;
+          for(j = 0; j < rc; j++) {
+            /* Keys and values are interleaved */
+            (*keys_p)[j] = strdup(reply->element[i]->str);
+            (*values_p)[j] = strdup(reply->element[i+1]->str);
+            i += 2;
+          }
+        } else {
+          free(*keys_p);
+          *keys_p = NULL;
+        }
+      }
+    }
+  }
+
+  if(reply) freeReplyObject(reply);
+  l->unlock(__FILE__, __LINE__);
+
+  return(rc);
+}
+
+/* **************************************** */
+
 int Redis::oneOperator(const char *operation, char *key) {
   int rc;
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "%s %s", operation, key);
   if(!reply) reconnectRedis();
 
@@ -449,6 +642,7 @@ int Redis::twoOperators(const char *operation, char *op1, char *op2) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "%s %s %s", operation, op1, op2);
   if(!reply) reconnectRedis();
 
@@ -465,26 +659,41 @@ int Redis::twoOperators(const char *operation, char *op1, char *op2) {
 
 /* **************************************** */
 
-int Redis::pushHostToTrafficFiltering(char *hostname, bool dont_check_for_existance, bool localHost) {
+int Redis::pushHostToTrafficFiltering(char *hostname, bool dont_check_for_existence, bool localHost) {
   if(ntop->getPrefs()->is_httpbl_enabled()) {
     if(hostname == NULL) return(-1);
-    return(pushHost(TRAFFIC_FILTERING_CACHE, TRAFFIC_FILTERING_TO_RESOLVE, hostname, dont_check_for_existance, localHost));
+    return(pushHost(TRAFFIC_FILTERING_CACHE, TRAFFIC_FILTERING_TO_RESOLVE,
+		    hostname, dont_check_for_existence, localHost));
   } else
     return(0);
 }
 
 /* **************************************** */
 
-int Redis::pushHostToResolve(char *hostname, bool dont_check_for_existance, bool localHost) {
+int Redis::pushHostToResolve(char *hostname, bool dont_check_for_existence, bool localHost) {
   if(!ntop->getPrefs()->is_dns_resolution_enabled()) return(0);
   if(hostname == NULL) return(-1);
-  return(pushHost(DNS_CACHE, DNS_TO_RESOLVE, hostname, dont_check_for_existance, localHost));
+
+  if(!ntop->getPrefs()->is_dns_resolution_enabled_for_all_hosts()) {
+    /*
+      In case only local addresses need to be resolved, skip
+      remote hosts
+    */
+    IpAddress ip;
+    int16_t network_id;
+
+    ip.set(hostname);
+    if(!ip.isLocalHost(&network_id))
+      return(-1);
+  }
+
+  return(pushHost(DNS_CACHE, DNS_TO_RESOLVE, hostname, dont_check_for_existence, localHost));
 }
 
 /* **************************************** */
 
 int Redis::pushHost(const char* ns_cache, const char* ns_list, char *hostname,
-		    bool dont_check_for_existance, bool localHost) {
+		    bool dont_check_for_existence, bool localHost) {
   int rc = 0;
   char key[CONST_MAX_LEN_REDIS_KEY];
   bool found;
@@ -496,13 +705,14 @@ int Redis::pushHost(const char* ns_cache, const char* ns_list, char *hostname,
 
   l->lock(__FILE__, __LINE__);
 
-  if(dont_check_for_existance)
+  if(dont_check_for_existence)
     found = false;
   else {
     /*
       Add only if the address has not been resolved yet
     */
 
+    num_requests++;
     reply = (redisReply*)redisCommand(redis, "GET %s", key);
     if(!reply) reconnectRedis();
 
@@ -560,8 +770,7 @@ char* Redis::getTrafficFilteringCategory(char *numeric_ip, char *buf,
   char key[CONST_MAX_LEN_REDIS_KEY];
   redisReply *reply;
 
-  if(!ntop->getPrefs()->is_httpbl_enabled()
-     && (!ntop->getPrefs()->is_flashstart_enabled()))
+  if(!ntop->getPrefs()->is_httpbl_enabled())
     return(NULL);
 
   buf[0] = '\0';
@@ -572,6 +781,7 @@ char* Redis::getTrafficFilteringCategory(char *numeric_ip, char *buf,
   /*
     Add only if the ip has not been checked against the blacklist
   */
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "GET %s", key);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -584,11 +794,15 @@ char* Redis::getTrafficFilteringCategory(char *numeric_ip, char *buf,
     buf[0] = '\0';
 
     if(categorize_if_unknown) {
-      reply = (redisReply*)redisCommand(redis, "RPUSH %s %s", TRAFFIC_FILTERING_TO_RESOLVE, numeric_ip);
-      if(!reply) reconnectRedis();
-      if(reply && (reply->type == REDIS_REPLY_ERROR))
-	ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-      if(reply) freeReplyObject(reply);
+      num_requests++;
+
+      if(ntop->getPrefs()->is_httpbl_enabled()) {
+	reply = (redisReply*)redisCommand(redis, "RPUSH %s %s", TRAFFIC_FILTERING_TO_RESOLVE, numeric_ip);
+	if(!reply) reconnectRedis();
+	if(reply && (reply->type == REDIS_REPLY_ERROR))
+	  ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+	if(reply) freeReplyObject(reply);
+      }
     }
   }
 
@@ -608,19 +822,53 @@ int Redis::popDomainToCategorize(char *domainname, u_int domainname_len) {
 /* **************************************** */
 
 void Redis::setDefaults() {
-  char value[CONST_MAX_LEN_REDIS_VALUE];
+  char *admin_md5 = (char*)"21232f297a57a5a743894a0e4a801fc3";
+  char *value;
 
+  if((value = (char*)malloc(CONST_MAX_LEN_REDIS_VALUE)) == NULL)
+    return;
+  
   setResolvedAddress((char*)"127.0.0.1", (char*)"localhost");
   setResolvedAddress((char*)"::1", (char*)"localhostV6");
   setResolvedAddress((char*)"255.255.255.255", (char*)"Broadcast");
   setResolvedAddress((char*)"0.0.0.0", (char*)"NoIP");
 
-  if(get((char*)"ntopng.user.admin.password", value, sizeof(value)) < 0) {
-    set((char*)"ntopng.user.admin.password", (char*)"21232f297a57a5a743894a0e4a801fc3");
+  if(get((char*)"ntopng.user.admin.password", value,
+	 CONST_MAX_LEN_REDIS_VALUE) < 0) {
+    set((char*)"ntopng.user.admin.password", admin_md5);
     set((char*)"ntopng.user.admin.full_name", (char*)"ntopng Administrator");
-    set((char*)"ntopng.user.admin.group", (char*)"administrator");
+    set((char*)"ntopng.user.admin.group", (char*)CONST_USER_GROUP_ADMIN);
     set((char*)"ntopng.user.admin.allowed_nets", (char*)"0.0.0.0/0,::/0");
+  } else if(strncmp(value, admin_md5, strlen(admin_md5))) {
+    set((char*)CONST_DEFAULT_PASSWORD_CHANGED, (char*)"1");
   }
+
+  free(value);
+}
+
+/* **************************************** */
+
+int Redis::flushDb() {
+  int rc;
+  redisReply *reply;
+
+  l->lock(__FILE__, __LINE__);
+
+  num_requests++;
+  reply = (redisReply*)redisCommand(redis, "FLUSHDB");
+  if(!reply) reconnectRedis();
+  if(reply && (reply->type == REDIS_REPLY_ERROR))
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+  if(reply) freeReplyObject(reply), rc = 0; else rc = -1;
+
+  l->unlock(__FILE__, __LINE__);
+
+  if (rc == 0) {
+    flushCache();
+    setDefaults();
+  }
+
+  return(rc);
 }
 
 /* **************************************** */
@@ -647,7 +895,7 @@ int Redis::getAddressTrafficFiltering(char *numeric_ip,
   } else {
     /* We need to extend expire */
 
-    expire(numeric_ip, TRAFFIC_FILTERING_CACHE_DURATIION /* expire */);
+    expire(numeric_ip, TRAFFIC_FILTERING_CACHE_DURATION /* expire */);
   }
 
   return(rc);
@@ -683,7 +931,7 @@ int Redis::setTrafficFilteringAddress(char *numeric_ip, char *httpbl) {
   char key[CONST_MAX_LEN_REDIS_KEY];
 
   snprintf(key, sizeof(key), "%s.%s", TRAFFIC_FILTERING_CACHE, numeric_ip);
-  return(set(key, httpbl, TRAFFIC_FILTERING_CACHE_DURATIION));
+  return(set(key, httpbl, TRAFFIC_FILTERING_CACHE_DURATION));
 }
 
 /* **************************************** */
@@ -716,6 +964,7 @@ char* Redis::getVersion(char *str, u_int str_len) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "INFO");
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -756,6 +1005,7 @@ int Redis::hashIncr(char *key, char *field, u_int32_t value) {
   if(key == NULL || field == NULL) return 0;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "HINCRBY %s %s %u", key, field, value);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -783,6 +1033,18 @@ int Redis::addHostToDBDump(NetworkInterface *iface, IpAddress *ip, char *name) {
   return(host_to_id(iface, daybuf, what, &new_key));
 }
 
+/* ******************************************* */
+
+int Redis::sadd(const char *set_name, char *item) {
+  return(msg_push("SADD", set_name, item, 0 /* do not trim, this is not a queue */));
+}
+
+/* ******************************************* */
+
+int Redis::srem(const char *set_name, char *item) {
+  return(msg_push("SREM", set_name, item, 0 /* do not trim, this is not a queue */));
+}
+
 /* **************************************** */
 
 int Redis::smembers(lua_State* vm, char *setName) {
@@ -792,6 +1054,7 @@ int Redis::smembers(lua_State* vm, char *setName) {
   lua_newtable(vm);
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "SMEMBERS %s", setName);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -815,6 +1078,45 @@ int Redis::smembers(lua_State* vm, char *setName) {
   return(rc);
 }
 
+/* **************************************** */
+
+int Redis::smembers(const char *set_name, char ***members) {
+  int rc = -1;
+  u_int i;
+  redisReply *reply = NULL;
+
+  l->lock(__FILE__, __LINE__);
+  num_requests++;
+  reply = (redisReply*)redisCommand(redis, "SMEMBERS %s", set_name);
+
+  if(!reply) reconnectRedis();
+
+  if(reply && (reply->type == REDIS_REPLY_ERROR)) {
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s [SMEMBERS %s]", reply->str ? reply->str : "???", set_name);
+    goto out;
+  }
+
+  (*members) = NULL;
+
+  if(reply && (reply->type == REDIS_REPLY_ARRAY)) {
+    rc = (int)reply->elements;
+
+    if(rc > 0) {
+      if(((*members) = (char**)malloc(reply->elements * sizeof(char*))) != NULL) {
+
+	for(i = 0; i < reply->elements; i++)
+	  (*members)[i] = strdup(reply->element[i]->str);
+      }
+    }
+  }
+
+ out:
+  if(reply) freeReplyObject(reply);
+  l->unlock(__FILE__, __LINE__);
+
+  return(rc);
+}
+
 /* *************************************** */
 
 void Redis::setHostId(NetworkInterface *iface, char *daybuf, char *host_name, u_int32_t id) {
@@ -839,6 +1141,7 @@ void Redis::setHostId(NetworkInterface *iface, char *daybuf, char *host_name, u_
 
 #if 0
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "SADD %s.keys %s", daybuf, value);
   if(!reply) reconnectRedis();
 
@@ -862,18 +1165,22 @@ void Redis::setHostId(NetworkInterface *iface, char *daybuf, char *host_name, u_
 
 /* *************************************** */
 
-u_int32_t Redis::host_to_id(NetworkInterface *iface, char *daybuf, char *host_name, bool *new_key) {
+u_int32_t Redis::host_to_id(NetworkInterface *iface, char *daybuf,
+			    char *host_name, bool *new_key) {
   u_int32_t id;
   int rc;
-  char buf[32], keybuf[384], rsp[CONST_MAX_LEN_REDIS_VALUE];
+  char buf[32], keybuf[384], *rsp;
 
   if(iface == NULL) return(-1);
 
+  if((rsp = (char*)malloc(CONST_MAX_LEN_REDIS_VALUE)) == NULL)
+    return(CONST_LUA_PARAM_ERROR);
+  
   snprintf(keybuf, sizeof(keybuf), "%s|%s", iface->get_name(), host_name);
 
   /* Add host key if missing */
   snprintf(buf, sizeof(buf), "ntopng.%s.hostkeys", daybuf);
-  rc = hashGet(buf, keybuf, rsp, sizeof(rsp));
+  rc = hashGet(buf, keybuf, rsp, CONST_MAX_LEN_REDIS_VALUE);
 
   if(rc == -1) {
     /* Not found */
@@ -885,6 +1192,8 @@ u_int32_t Redis::host_to_id(NetworkInterface *iface, char *daybuf, char *host_na
   } else
     id = atol(rsp), *new_key = false;
 
+  free(rsp);
+  
   return(id);
 }
 
@@ -900,8 +1209,8 @@ int Redis::id_to_host(char *daybuf, char *host_idx, char *buf, u_int buf_len) {
 
 /* ******************************************* */
 
-int Redis::lpush(const char *queue_name, char *msg, u_int queue_trim_size) {
-  return(msg_push("LPUSH", queue_name, msg, queue_trim_size));
+int Redis::lpush(const char *queue_name, char *msg, u_int queue_trim_size, bool trace_errors) {
+  return(msg_push("LPUSH", queue_name, msg, queue_trim_size, trace_errors));
 }
 
 /* ******************************************* */
@@ -912,26 +1221,30 @@ int Redis::rpush(const char *queue_name, char *msg, u_int queue_trim_size) {
 
 /* ******************************************* */
 
-int Redis::msg_push(const char *cmd, const char *queue_name, char *msg, u_int queue_trim_size) {
+int Redis::msg_push(const char *cmd, const char *queue_name, char *msg, u_int queue_trim_size, bool trace_errors) {
   redisReply *reply;
   int rc = 0;
 
   l->lock(__FILE__, __LINE__);
   /* Put the latest messages on top so old messages (if any) will be discarded */
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "%s %s %s", cmd,  queue_name, msg);
 
   if(!reply) reconnectRedis();
   if(reply) {
-    if(reply->type == REDIS_REPLY_ERROR)
+    if(reply->type == REDIS_REPLY_ERROR && trace_errors)
       ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???"), rc = -1;
+    else
+      rc = reply->integer;
 
     freeReplyObject(reply);
 
     if(queue_trim_size > 0) {
-      reply = (redisReply*)redisCommand(redis, "LTRIM %s 0 %u", queue_name, queue_trim_size);
+      num_requests++;
+      reply = (redisReply*)redisCommand(redis, "LTRIM %s 0 %u", queue_name, queue_trim_size - 1);
       if(!reply) reconnectRedis();
       if(reply) {
-	if(reply->type == REDIS_REPLY_ERROR)
+	if(reply->type == REDIS_REPLY_ERROR && trace_errors)
 	  ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???"), rc = -1;
 
 	freeReplyObject(reply);
@@ -947,35 +1260,12 @@ int Redis::msg_push(const char *cmd, const char *queue_name, char *msg, u_int qu
 
 /* ******************************************* */
 
-void Redis::queueAlert(AlertLevel level, AlertStatus s, AlertType t, char *msg) {
-  char what[1024];
-
-  if(ntop->getPrefs()->are_alerts_disabled()) return;
-
-  snprintf(what, sizeof(what), "%u|%u|%u|%u|%s",
-	   (unsigned int)time(NULL), (unsigned int)level,
-	   (unsigned int)s, (unsigned int)t, msg);
-
-#ifndef WIN32
-  // Print alerts into syslog
-  if(ntop->getRuntimePrefs()->are_alerts_syslog_enabled()) {
-    if(alert_level_info == level) syslog(LOG_INFO, "%s", what);
-    else if(alert_level_warning == level) syslog(LOG_WARNING, "%s", what);
-    else if(alert_level_error == level) syslog(LOG_ALERT, "%s", what);
-  }
-#endif
-
-  lpush(CONST_ALERT_MSG_QUEUE, what, CONST_MAX_ALERT_MSG_QUEUE_LEN);
-
-}
-
-/* ******************************************* */
-
 u_int Redis::llen(const char *queue_name) {
   redisReply *reply;
   u_int num = 0;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "LLEN %s", queue_name);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -990,11 +1280,50 @@ u_int Redis::llen(const char *queue_name) {
 
 /* ******************************************* */
 
+int Redis::lset(const char *queue_name, u_int32_t idx, const char *value) {
+  redisReply *reply;
+
+  l->lock(__FILE__, __LINE__);
+  num_requests++;
+  reply = (redisReply*)redisCommand(redis, "LSET %s %u %s", queue_name, idx, value);
+  if(!reply) reconnectRedis();
+  if(reply && (reply->type == REDIS_REPLY_ERROR))
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+
+  l->unlock(__FILE__, __LINE__);
+
+  if(reply) freeReplyObject(reply);
+
+  return 0;
+}
+
+/* ******************************************* */
+
+int Redis::lrem(const char *queue_name, const char *value) {
+  redisReply *reply;
+
+  l->lock(__FILE__, __LINE__);
+  num_requests++;
+  reply = (redisReply*)redisCommand(redis, "LREM %s 0 %s", queue_name, value);
+  if(!reply) reconnectRedis();
+  if(reply && (reply->type == REDIS_REPLY_ERROR))
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+
+  l->unlock(__FILE__, __LINE__);
+
+  if(reply) freeReplyObject(reply);
+
+  return 0;
+}
+
+/* ******************************************* */
+
 int Redis::lpop(const char *queue_name, char *buf, u_int buf_len) {
   int rc;
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "LPOP %s", queue_name);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -1018,6 +1347,7 @@ int Redis::lindex(const char *queue_name, int idx, char *buf, u_int buf_len) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
+  num_requests++;
   reply = (redisReply*)redisCommand(redis, "LINDEX %s %d", queue_name, idx);
 
   if(!reply) reconnectRedis();
@@ -1043,12 +1373,13 @@ int Redis::lpop(const char *queue_name, char ***elements, u_int num_elements) {
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
-
-  // make a redis pipeline that pops multile elements
+  num_requests++;
+  // make a redis pipeline that pops multiple elements
   // with just 1 redis command (so we pay only one RTT
   // and the operation is atomic)
   /*
-  reply = (redisReply*)redisCommand(redis,
+
+    reply = (redisReply*)redisCommand(redis,
 				    "LRANGE %s -%u -1 \r\n LTRIM %s 0 -%u \r\n",
 				    queue_name, num_elements, queue_name, num_elements + 1);
   */
@@ -1086,55 +1417,158 @@ int Redis::lpop(const char *queue_name, char ***elements, u_int num_elements) {
   return(rc);
 }
 
-/* ******************************************* */
+/* **************************************** */
 
-void Redis::deleteQueuedAlert(u_int32_t idx_to_delete) {
+int Redis::lrange(const char *list_name, char ***elements, int start_offset, int end_offset) {
+  int rc = 0;
+  u_int i;
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
-  reply = (redisReply*)redisCommand(redis, "LSET %s %u __deleted__", CONST_ALERT_MSG_QUEUE, idx_to_delete);
+  num_requests++;
+  reply = (redisReply*)redisCommand(redis, "LRANGE %s %i %i", list_name, start_offset, end_offset);
+
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
     ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-  if(reply) freeReplyObject(reply);
 
-  reply = (redisReply*)redisCommand(redis, "LREM %s 0 __deleted__", CONST_ALERT_MSG_QUEUE, idx_to_delete);
-  if(!reply) reconnectRedis();
-  if(reply && (reply->type == REDIS_REPLY_ERROR))
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-  l->unlock(__FILE__, __LINE__);
-  if(reply) freeReplyObject(reply);
-}
+  if(reply && (reply->type == REDIS_REPLY_ARRAY)) {
+    (*elements) = (char**) malloc(reply->elements * sizeof(char*));
+    rc = (int)reply->elements;
 
-/* **************************************** */
-
-u_int Redis::getQueuedAlerts(patricia_tree_t *allowed_hosts, char **alerts, u_int start_idx, u_int num) {
-  u_int i = 0;
-  redisReply *reply = NULL;
-
-  // TODO - We need to filter events that belong to allowed_hosts only
-
-  l->lock(__FILE__, __LINE__);
-  while(i < num) {
-    reply = (redisReply*)redisCommand(redis, "LINDEX %s %u", CONST_ALERT_MSG_QUEUE, start_idx++);
-    if(!reply) reconnectRedis();
-    if(reply && (reply->type == REDIS_REPLY_ERROR)) {
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-      break;
+    for(i = 0; i < reply->elements; i++) {
+      (*elements)[i] = strdup(reply->element[i]->str);
     }
-
-    if(reply && reply->str) {
-      alerts[i++] = strdup(reply->str);
-      freeReplyObject(reply);
-      reply = NULL;
-    } else
-      break;
   }
 
   if(reply) freeReplyObject(reply);
   l->unlock(__FILE__, __LINE__);
 
-  return(i);
+  return(rc);
 }
 
+/* **************************************** */
 
+void Redis::lua(lua_State *vm) {
+  lua_newtable(vm);
+
+  lua_push_int_table_entry(vm, "num_requests", num_requests);
+  lua_push_int_table_entry(vm, "num_reconnections", num_reconnections);
+
+  lua_pushstring(vm, "redis");
+  lua_insert(vm, -2);
+  lua_settable(vm, -3);
+}
+
+/* **************************************** */
+
+void Redis::flushCache() {
+  StringCache_t *sd, *current, *tmp;
+
+  l->lock(__FILE__, __LINE__);
+  sd = stringCache;
+
+  HASH_ITER(hh, sd, current, tmp) {
+    HASH_DEL(sd, current);
+    if(current->key)   free(current->key);
+    if(current->value) free(current->value);
+    free(current);
+  }
+
+  numCached = 0;
+  l->unlock(__FILE__, __LINE__);
+
+#ifdef CACHE_DEBUG
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "**** Successfully flushed cache\n");
+#endif
+}
+
+/* **************************************** */
+
+/* https://github.com/nullptr-cc/redis-extra-tools */
+static void bin2hex(char * in, int len, char * out) {
+  int i;
+
+  char table[] = "0123456789ABCDEF";
+
+  for (i = 0; i < len; ++i) {
+    out[i*2+0] = table[in[i] >> 4 & 0x0F];
+    out[i*2+1] = table[in[i] & 0x0F];
+  }
+
+  out[len*2] = 0;
+}
+
+static char hdig2bin(char c) {
+  if (c >= '0' && c <= '9') {
+    return c - '0';
+  } else {
+    return c - 'A' + 0x0A;
+  }
+}
+
+static void hex2bin(char * in, char * out) {
+  int unsigned i, len = strlen(in);
+
+  for (i = 0; i < len; i += 2) {
+    out[i / 2] = (hdig2bin(in[i]) << 4) + hdig2bin(in[i+1]);
+  }
+}
+
+/* **************************************** */
+
+char* Redis::dump(char *key) {
+  char *rsp = NULL;
+  redisReply *reply;
+
+  l->lock(__FILE__, __LINE__);
+  num_requests++;
+  reply = (redisReply*)redisCommand(redis, "DUMP %s", key);
+  if(!reply) reconnectRedis();
+  if(reply && (reply->type == REDIS_REPLY_ERROR))
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+
+  if(reply) {
+    if((rsp = (char*)malloc(1 + reply->len * 2)) != NULL)
+      bin2hex(reply->str, reply->len, rsp);
+
+    freeReplyObject(reply);
+  }
+
+  l->unlock(__FILE__, __LINE__);
+
+  return(rsp);
+}
+
+/* **************************************** */
+
+int Redis::restore(char *key, char *buf) {
+  int rc;
+  redisReply *reply;
+  char *buf_bin = (char*)malloc(strlen(buf));
+  const char * argv[4] = {"RESTORE", key, "0", buf_bin};
+  size_t argvlen[4] = {7, 0, 0, 0};
+
+  if(buf_bin == NULL)
+    return(-1);
+
+  hex2bin(buf, buf_bin);
+
+  l->lock(__FILE__, __LINE__);
+  num_requests++;
+
+  argvlen[1] = strlen(argv[1]);
+  argvlen[2] = strlen(argv[2]);
+  argvlen[3] = strlen(buf) / 2;
+
+  reply = (redisReply*)redisCommandArgv(redis, 4, argv, argvlen);
+
+  rc = reply ? 0 : -1;
+
+  if(reply) freeReplyObject(reply);
+  l->unlock(__FILE__, __LINE__);
+
+  free(buf_bin);
+
+  return(rc);
+}
