@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2016 - ntop.org
+ * (C) 2016-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,9 @@
 
 #include "ntop_includes.h"
 
-#if defined(HAVE_PF_RING) && (!defined(__mips)) && (!defined(__arm__))
+#ifndef HAVE_NEDGE
+
+#if defined(HAVE_PF_RING) && (!defined(__mips)) && (!defined(__arm__)) && (!defined(__i686__)) && (!defined(__ARM_ARCH))
 
 /* **************************************************** */
 
@@ -42,8 +44,6 @@ ZCCollectorInterface::ZCCollectorInterface(const char *name) : ParserInterface(n
 
   cluster_id = atoi(ifname);
 
-  num_drops = 0;
-
   zq = pfring_zc_ipc_attach_queue(cluster_id, queue_id, rx_only);
 
   if(zq == NULL)
@@ -58,6 +58,8 @@ ZCCollectorInterface::ZCCollectorInterface(const char *name) : ParserInterface(n
 
   if(buffer == NULL)
     throw("pfring_zc_get_packet_handle_from_pool error");
+
+  memset(&last_pfring_zc_stat, 0, sizeof(last_pfring_zc_stat));
 }
 
 /* **************************************************** */
@@ -87,12 +89,20 @@ void ZCCollectorInterface::collect_flows() {
 
     if(rc > 0) {
       u_char *json = pfring_zc_pkt_buff_data(buffer, zq);
-
+      const char *master = "{ \"if.name\"";
+      
       ntop->getTrace()->traceEvent(TRACE_INFO, "%s", json);
-      parseFlow((char*)json, buffer->len, 0, (void*)this);
+      // fprintf(stdout, "+"); fflush(stdout);
+
+      if(strncmp((char*)json, master, strlen(master)) == 0) {
+	parseEvent((char*)json, buffer->len, 0, (void*)this);
+      } else
+	parseFlow((char*)json, buffer->len, 0, (void*)this);
+      // fprintf(stdout, "."); fflush(stdout);
     } else if(rc == 0) {
       usleep(1);
       purgeIdle(time(NULL));
+      // fprintf(stdout, "*"); fflush(stdout);
     } else {
       /* rc < 0 */
       break;
@@ -146,5 +156,24 @@ bool ZCCollectorInterface::set_packet_filter(char *filter) {
 
 /* **************************************************** */
 
+u_int32_t ZCCollectorInterface::getNumDroppedPackets() {
+  pfring_zc_stat stats;
+
+  if(pfring_zc_stats(zq, &stats) >= 0) {
+#if 0
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "[%s][Sent: %llu]"
+				 "[Rcvd: %llu][Drops: %llu][QueueLen: %d]",
+				 ifname, stats.sent, stats.recv, stats.drop,
+				 stats.sent-stats.recv);
+#endif
+    return(stats.drop);
+  }
+
+  return 0;
+}
+
+/* **************************************************** */
+
 #endif
 
+#endif
