@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-17 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,11 +25,20 @@ extern "C" {
   extern char* rrd_strversion(void);
 };
 
+AfterShutdownAction afterShutdownAction = after_shutdown_nop;
+
+/* ******************************** */
+
+void sighup(int sig) {
+  ntop->reloadInterfacesLuaInterpreter();
+}
+
 /* ******************************** */
 
 void sigproc(int sig) {
   static int called = 0;
-
+  ThreadedActivity *shutdown_activity;
+  
   if(called) {
     ntop->getTrace()->traceEvent(TRACE_NORMAL, "Ok I am leaving now");
     _exit(0);
@@ -38,6 +47,13 @@ void sigproc(int sig) {
     called = 1;
   }
 
+  /* Exec shutdown script before shutting down ntopng */
+  if((shutdown_activity = new ThreadedActivity(SHUTDOWN_SCRIPT_PATH))) {
+    /* Don't call run() as by the time the script will be run the delete below will free the memory */
+    shutdown_activity->runScript();
+    delete shutdown_activity;
+  }    
+
   ntop->getGlobals()->shutdown();
   sleep(2); /* Wait until all threads know that we're shutting down... */
   ntop->shutdown();
@@ -45,12 +61,24 @@ void sigproc(int sig) {
 #ifndef WIN32
   if(ntop->getPrefs()->get_pid_path() != NULL) {
     int rc = unlink(ntop->getPrefs()->get_pid_path());
-    ntop->getTrace()->traceEvent(TRACE_NORMAL, "Deleted PID %s [rc: %d]",
-				 ntop->getPrefs()->get_pid_path(), rc);
+
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "Deleted PID %s: [rc: %d][%s]",
+				 ntop->getPrefs()->get_pid_path(),
+				 rc, strerror(errno));
   }
 #endif
 
   delete ntop;
+
+#ifdef linux
+  switch(afterShutdownAction) {
+    case after_shutdown_nop: break;
+    case after_shutdown_reboot: system("/sbin/reboot &"); break;
+    case after_shutdown_poweroff: system("/sbin/shutdown -h now &"); break;
+    default: break;
+  }
+#endif
+  
   _exit(0);
 }
 
@@ -73,6 +101,8 @@ void initWinsock32() {
   }
 }
 
+/* ******************************** */
+
 extern "C" {
 int ntop_main(int argc, char *argv[])
 #else
@@ -88,7 +118,8 @@ int main(int argc, char *argv[])
   int core_id;
   char path[MAX_PATH];
   FILE *fd;
-
+  ThreadedActivity *boot_activity;
+  
 #ifdef WIN32
   initWinsock32();
 #endif
@@ -97,9 +128,11 @@ int main(int argc, char *argv[])
   if((prefs = new(std::nothrow) Prefs(ntop)) == NULL)   _exit(0);
 
 #ifndef WIN32
-  if((argc == 2) && (argv[1][0] != '-'))
+  if((argc >= 2) && (argv[1][0] != '-')) {
     rc = prefs->loadFromFile(argv[1]);
-  else
+    if(argc > 2)
+      rc = prefs->loadFromCLI(argc, argv);
+  } else
 #endif
     rc = prefs->loadFromCLI(argc, argv);
 
@@ -107,9 +140,12 @@ int main(int argc, char *argv[])
 
   ntop->registerPrefs(prefs, false);
 
-  if(ntop->getRedis() == NULL)
-    return(-1);
-
+  if((boot_activity = new ThreadedActivity(BOOT_SCRIPT_PATH))) {
+    /* Don't call run() as by the time the script will be run the delete below will free the memory */
+    boot_activity->runScript();
+    delete boot_activity;
+  }
+  
   prefs->registerNetworkInterfaces();
 
   if(prefs->get_num_user_specified_interfaces() == 0) {
@@ -121,6 +157,9 @@ int main(int argc, char *argv[])
   ntop->registerNagios();
 #endif
 
+  prefs->reloadPrefsFromRedis();
+  prefs->validate();
+  
   if(prefs->daemonize_ntopng())
     ntop->daemonize();
 
@@ -135,11 +174,13 @@ int main(int argc, char *argv[])
   for(int i=0; i<MAX_NUM_INTERFACES; i++) {
     NetworkInterface *iface = NULL;
 
-    if((ifName = ntop->get_if_name(i)) == NULL)
+    if((ifName = ntop->get_if_name(i)) == NULL
+       || !strncmp(ifName, "view:", 5) /* Defer view interfaces init */)
       continue;
 
     try {
       /* [ zmq-collector.lua@tcp://127.0.0.1:5556 ] */
+#ifndef HAVE_NEDGE
       if(!strcmp(ifName, "dummy")) {
 	iface = new DummyInterface();
       } else if((strstr(ifName, "tcp://") || strstr(ifName, "ipc://"))) {
@@ -152,14 +193,16 @@ int main(int argc, char *argv[])
 	  endpoint = ifName;
 
 	iface = new CollectorInterface(endpoint);
-#if defined(HAVE_PF_RING) && (!defined(__mips)) && (!defined(__arm__))
+#if defined(HAVE_PF_RING) && (!defined(__mips)) && (!defined(__arm__)) && (!defined(__i686__)) && (!defined(__ARM_ARCH))
       } else if(strstr(ifName, "zcflow:")) {
 	iface = new ZCCollectorInterface(ifName);
 #endif
-      } else {
+      } else
+#endif
+	{
 	iface = NULL;
 
-#ifdef NTOPNG_PRO
+#if defined(NTOPNG_PRO) && !defined(WIN32)
 	if(strncmp(ifName, "bridge:", 7) == 0)
 	  iface = new PacketBridge(ifName);
 #endif
@@ -175,29 +218,36 @@ int main(int argc, char *argv[])
 #endif
 	
 #ifdef HAVE_PF_RING
-	if(iface == NULL)
+	if((iface == NULL) && (!strstr(ifName, ".pcap")))
 	  iface = new PF_RINGInterface(ifName);
 #endif
       }
     } catch(...) {
+      ntop->getTrace()->traceEvent(TRACE_INFO, "An exception occurred during interface creation: %s. Falling back to pcap", ifName);
+      if(iface) delete iface;
       iface = NULL;
     }
 
+#ifndef HAVE_NEDGE
     if(iface == NULL) {
       try {
 	iface = new PcapInterface(ifName);
       } catch(...) {
-	ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to create interface %s", ifName);
+	ntop->getTrace()->traceEvent(TRACE_ERROR, "An exception occurred during interface creation: %s", ifName);
+	if(iface) delete iface;
 	iface = NULL;
       }
     }
+#endif
 
-    if(iface) {
+    if(iface) {      
       if(affinity != NULL) {
+	char *tmp;
+	
 	if(indexAffinity == 0)
-	  core_id_s = strtok(affinity, ",");
+	  core_id_s = strtok_r(affinity, ",", &tmp);
 	else 
-	  core_id_s = strtok(NULL, ",");
+	  core_id_s = strtok_r(NULL, ",", &tmp);
             
 	if(core_id_s != NULL)
 	  core_id = atoi(core_id_s);
@@ -208,39 +258,35 @@ int main(int argc, char *argv[])
 	iface->setCPUAffinity(core_id);
       }
 
-      if(prefs->get_packet_filter() != NULL)
+      if(prefs->get_packet_filter())
 	iface->set_packet_filter(prefs->get_packet_filter());
 
       ntop->registerInterface(iface);
     }
   } /* for */
 
+  /* Instantiated deferred view interfaces */
+  for(int i = 0; i < MAX_NUM_INTERFACES; i++) {
+    NetworkInterface *iface = NULL;
+
+    if((ifName = ntop->get_if_name(i)) == NULL || strncmp(ifName, "view:", 5))
+      continue;
+
+    if((iface = new ViewInterface(ifName)))
+      ntop->registerInterface(iface);
+  }
+
+#ifndef HAVE_NEDGE
   ntop->createExportInterface();
-
   ntop->getElasticSearch()->startFlowDump();
-
+  ntop->getLogstash()->startFlowDump();
+#endif
+  
   if(ntop->getInterfaceAtId(0) == NULL) {
     ntop->getTrace()->traceEvent(TRACE_ERROR, "Startup error: missing super-user privileges ?");
     exit(0);
   }
 
-  for(int i = 0 ; i < MAX_NUM_INTERFACES ; i++) {
-    char *ifName;
-    NetworkInterfaceView *iv;
-
-    if((ifName = prefs->getInterfaceViewAt(i)) != NULL) {
-      iv = new NetworkInterfaceView(ifName);
-
-      if(iv->get_numInterfaces() > 0) ntop->registerInterfaceView(iv); else delete iv;
-    }
-
-    if((ifName = prefs->getInterfaceAt(i)) != NULL) {
-      iv = new NetworkInterfaceView(ifName);
-      
-      if(iv->get_numInterfaces() > 0) ntop->registerInterfaceView(iv); else delete iv;
-    }
-  }
-
 #ifndef WIN32
   if(prefs->get_pid_path() != NULL) {
     FILE *fd;
@@ -258,18 +304,46 @@ int main(int argc, char *argv[])
 	ntop->getTrace()->traceEvent(TRACE_ERROR, "The PID file %s is empty: is your disk full perhaps ?",
 				     prefs->get_pid_path());
     } else
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to store PID in file %s",
-				   prefs->get_pid_path());
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to store PID in file %s: %s",
+				   prefs->get_pid_path(), strerror(errno));
   }
 #endif
 
-  if(prefs->do_change_user())
+  /*
+    It's safe to drop privileges now if http and https ports
+    are non-privileged. Otherwise it is necessary to delay
+    the privilege drop after the web server bind()
+   */
+  if(prefs->do_change_user()
+     && (prefs->get_http_port()  >= 1024)
+     && (prefs->get_https_port() >= 1024))
     Utils::dropPrivileges();
 
   ntop->loadGeolocation(prefs->get_docs_dir());
-  ntop->registerHTTPserver(new HTTPserver(prefs->get_http_port(),
-					  prefs->get_docs_dir(),
-					  prefs->get_scripts_dir()));
+  ntop->loadMacManufacturers(prefs->get_docs_dir());
+  ntop->loadTrackers();
+  ntop->registerHTTPserver(new HTTPserver(prefs->get_docs_dir(), prefs->get_scripts_dir()));
+
+  /*
+    If mysql flows dump is enabled, then it is necessary to create
+    and update the database schema
+   */
+  if(prefs->do_dump_flows_on_mysql()) {
+    /* create the schema only one time, no need to call it for every interface */
+    if(!ntop->getInterfaceAtId(0)->createDBSchema()){
+      ntop->getTrace()->traceEvent(TRACE_ERROR,
+				   "Unable to create database schema, quitting.");
+      exit(EXIT_FAILURE);
+    }
+  } else if(prefs->do_read_flows_from_nprobe_mysql()) {
+    /* Create a view only one time for the first interface */
+    if(!ntop->getInterfaceAtId(0)->createNprobeDBView()){
+      ntop->getTrace()->traceEvent(TRACE_ERROR,
+				   "Unable to create a view on the nProbe database.");
+      exit(EXIT_FAILURE);
+    }
+  }
+
   /*
     We have created the network interface and thus changed user. Let's not check
     if we can write on the data directory
@@ -283,12 +357,26 @@ int main(int argc, char *argv[])
     ntop->getTrace()->traceEvent(TRACE_ERROR,
 				 "Unable to write on %s [%s]: please specify a different directory (-d)",
 				 ntop->get_working_dir(), path);
-    exit(0);
+    exit(EXIT_FAILURE);
   } else {
     fclose(fd); /* All right */
     unlink(path);
   }
   
+#ifndef WIN32
+  if(prefs->daemonize_ntopng())
+#endif
+    {
+      char path[MAX_PATH];
+
+      Utils::mkdir_tree(ntop->get_data_dir());
+      Utils::mkdir_tree(ntop->get_working_dir());
+      snprintf(path, sizeof(path), "%s/ntopng.log", ntop->get_working_dir() /* "C:\\Windows\\Temp" */);
+      ntop->fixPath(path);
+      ntop->registerLogFile(path);
+      ntop->rotateLogs(true /* Force rotation to start clean */);
+    }
+
   if(prefs->get_httpbl_key() != NULL)
     ntop->setHTTPBL(new HTTPBL(prefs->get_httpbl_key()));
 
@@ -297,9 +385,12 @@ int main(int argc, char *argv[])
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "Scripts/HTML pages directory: %s",
 			       ntop->get_install_dir());
 
-  signal(SIGINT, sigproc);
+#ifndef WIN32
+  signal(SIGHUP,  sighup);
+  signal(SIGINT,  sigproc);
   signal(SIGTERM, sigproc);
-  signal(SIGINT, sigproc);
+  signal(SIGINT,  sigproc);
+#endif
 
 #if defined(WIN32) && defined(DEMO_WIN32)
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "-----------------------------------------------------------");
@@ -310,9 +401,15 @@ int main(int argc, char *argv[])
 
   ntop->start();
 
+  if(ntop->getGlobals()->isShutdownRequested()) {
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "Shutdown requested: hold on");
+    sleep(3); /* Wait until all open activities have been completed */
+    ntop->shutdown();
+  }
+  
   sigproc(0);
   delete ntop;
-
+ 
   return(0);
 }
 
