@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2013 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2017 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -225,7 +225,7 @@ extern "C" {
   //
   //  PARAMETERS:
   //    target - pointer to the string to be allocated and created
-  //    start  - zero-based offest into the list to the first arg value used to
+  //    start  - zero-based offset into the list to the first arg value used to
   //             build the list.
   //    argc - length of the argument list
   //    argv - array of strings, the argument list.
@@ -909,7 +909,7 @@ void runService(int argc, char ** argv)
   lpszArgv = argv;
 #endif
 
-  AddToMessageLog("Starting ntopg");
+  AddToMessageLog("Starting ntopng");
 
   _tprintf(TEXT("Running %s.\n"), TEXT(SZSERVICEDISPLAYNAME));
 
@@ -1096,7 +1096,7 @@ void WINAPI serviceMain(DWORD dwArgc, LPTSTR *lpszArgv)
 
   // Do it! In ServiceStart, we'll send additional status reports to the
   // service control manager, especially the SERVICE_RUNNING report once
-  // our JVM is initiallized and ready to be invoked.
+  // our JVM is initialized and ready to be invoked.
   ServiceStart(dwNewArgc, lpszNewArgv);
 
   // Release the allocated storage used by our arg list. Java programmers
