@@ -341,7 +341,7 @@ static int dbvm_get_value(lua_State *L) {
 
 static int dbvm_get_name(lua_State *L) {
   sdb_vm *svm = lsqlite_checkvm(L, 1);
-  int index = luaL_checknumber(L, 2);
+  int index = (int)luaL_checknumber(L, 2);
   dbvm_check_index(L, svm, index);
   lua_pushstring(L, sqlite3_column_name(svm->vm, index));
   return 1;
@@ -349,7 +349,7 @@ static int dbvm_get_name(lua_State *L) {
 
 static int dbvm_get_type(lua_State *L) {
   sdb_vm *svm = lsqlite_checkvm(L, 1);
-  int index = luaL_checknumber(L, 2);
+  int index = (int)luaL_checknumber(L, 2);
   dbvm_check_index(L, svm, index);
   lua_pushstring(L, sqlite3_column_decltype(svm->vm, index));
   return 1;
@@ -498,7 +498,7 @@ static int dbvm_bind_parameter_count(lua_State *L) {
 
 static int dbvm_bind_parameter_name(lua_State *L) {
   sdb_vm *svm = lsqlite_checkvm(L, 1);
-  int index = luaL_checknumber(L, 2);
+  int index = (int)luaL_checknumber(L, 2);
   dbvm_check_bind_index(L, svm, index);
   lua_pushstring(L, sqlite3_bind_parameter_name(svm->vm, index));
   return 1;
@@ -715,6 +715,7 @@ typedef struct {
   int ud;
 } lcontext;
 
+#if 0
 static lcontext *lsqlite_make_context(lua_State *L) {
   lcontext *ctx = (lcontext*)lua_newuserdata(L, sizeof(lcontext));
   lua_rawgeti(L, LUA_REGISTRYINDEX, sqlite_ctx_meta_ref);
@@ -723,6 +724,7 @@ static lcontext *lsqlite_make_context(lua_State *L) {
   ctx->ud = LUA_NOREF;
   return ctx;
 }
+#endif
 
 static lcontext *lsqlite_getcontext(lua_State *L, int index) {
   lcontext *ctx = (lcontext*)luaL_checkudata(L, index, sqlite_ctx_meta);
@@ -911,7 +913,7 @@ static int db_interrupt(lua_State *L) {
 /*
 ** Registering SQL functions:
 */
-
+#if 0
 static void db_push_value(lua_State *L, sqlite3_value *value) {
   switch (sqlite3_value_type(value)) {
   case SQLITE_TEXT:
@@ -1058,6 +1060,7 @@ static void db_sql_finalize_function(sqlite3_context *context) {
   lua_settop(L, top);
 }
 
+
 /*
 ** Register a normal function
 ** Params: db, function name, number arguments, [ callback | step, finalize], user data
@@ -1191,6 +1194,7 @@ static int db_create_collation(lua_State *L) {
 			      (void(*)(void*))collfree);
   return 0;
 }
+#endif
 
 /*
 ** trace callback:
@@ -1614,7 +1618,7 @@ static int db_exec_callback(void* user, int columns, char **data, char **names)
   /* call lua function */
   if(!lua_pcall(L, 4, 1, 0)) {
     if(lua_isnumber(L, -1))
-      result = lua_tonumber(L, -1);
+      result = (int)lua_tonumber(L, -1);
   }
 
   lua_settop(L, top);
@@ -1999,9 +2003,11 @@ static const luaL_Reg dblib[] = {
   {"error_message",       db_errmsg               },
   {"interrupt",           db_interrupt            },
 
+#if 0
   {"create_function",     db_create_function      },
   {"create_aggregate",    db_create_aggregate     },
   {"create_collation",    db_create_collation     },
+#endif
 
   {"trace",               db_trace                },
   {"progress_handler",    db_progress_handler     },
