@@ -68,12 +68,14 @@ void DeleteEntry(void *a) {
  * convert prefix information to bytes
  */
 u_char *
-prefix_tochar (prefix_t * prefix)
+prefix_tochar (prefix_t * prefix, unsigned short family)
 {
   if(prefix == NULL)
     return (NULL);
 
-  return ((u_char *) & prefix->add.sin);
+  if(family == AF_INET) return ((u_char *) & prefix->add.sin);
+  else if(family == AF_INET6) return ((u_char *) & prefix->add.sin6);
+  else /* if(family == AF_MAC) */ return ((u_char *) & prefix->add.mac);
 }
 
 int
@@ -501,6 +503,47 @@ patricia_process (patricia_tree_t *patricia, void_fn2_t func)
   } PATRICIA_WALK_END;
 }
 
+static size_t
+patricia_clone_walk(patricia_node_t *node, patricia_tree_t *dst)
+{
+  patricia_node_t *cloned_node;
+  size_t n = 0;
+
+  if(node->l) {
+    n += patricia_clone_walk(node->l, dst);
+  }
+
+  if(node->prefix) {
+    cloned_node = patricia_lookup(dst, node->prefix);
+
+    if(cloned_node)
+      cloned_node->user_data = node->user_data; /* TODO: clone void * data */
+
+    n++;
+  }
+
+  if(node->r) {
+    n += patricia_clone_walk(node->r, dst);
+  }
+
+  return n;
+}
+
+patricia_tree_t *
+patricia_clone (const patricia_tree_t * const from)
+{
+  if(!from) return (NULL);
+
+  patricia_tree_t *patricia = New_Patricia(from->maxbits);
+
+  if(!patricia) return (NULL);
+
+  if(from->head)
+    patricia_clone_walk(from->head, patricia);
+
+  return (patricia);
+}
+
 size_t
 patricia_walk_inorder(patricia_node_t *node, void_fn3_t func, void *data)
 {
@@ -582,13 +625,15 @@ patricia_search_exact (patricia_tree_t *patricia, prefix_t *prefix)
     return (NULL);
   assert (node->bit == bitlen);
   assert (node->bit == node->prefix->bitlen);
-  if(comp_with_mask (prefix_tochar (node->prefix), prefix_tochar (prefix),
+  if(comp_with_mask (prefix_tochar (node->prefix, node->prefix->family), prefix_tochar (prefix, prefix->family),
 		     bitlen)) {
 #ifdef PATRICIA_DEBUG
     fprintf (stderr, "patricia_search_exact: found %s/%d\n",
 	     prefix_toa (node->prefix), node->prefix->bitlen);
 #endif /* PATRICIA_DEBUG */
-    return (node);
+
+    if(node->prefix->family == prefix->family)
+      return (node);
   }
   return (NULL);
 }
@@ -617,7 +662,7 @@ patricia_search_best2 (patricia_tree_t *patricia, prefix_t *prefix, int inclusiv
 
   while (node->bit < bitlen) {
 
-    if(node->prefix) {
+    if((node->prefix) && (node->prefix->family == prefix->family)) {
 #ifdef PATRICIA_DEBUG
       fprintf (stderr, "patricia_search_best: push %s/%d\n",
 	       prefix_toa (node->prefix), node->prefix->bitlen);
@@ -652,7 +697,7 @@ patricia_search_best2 (patricia_tree_t *patricia, prefix_t *prefix, int inclusiv
       break;
   }
 
-  if(inclusive && node && node->prefix)
+  if(inclusive && node && node->prefix && (node->prefix->family == prefix->family))
     stack[cnt++] = node;
 
 #ifdef PATRICIA_DEBUG
@@ -674,8 +719,8 @@ patricia_search_best2 (patricia_tree_t *patricia, prefix_t *prefix, int inclusiv
     fprintf (stderr, "patricia_search_best: pop %s/%d\n",
 	     prefix_toa (node->prefix), node->prefix->bitlen);
 #endif /* PATRICIA_DEBUG */
-    if(comp_with_mask (prefix_tochar (node->prefix),
-		       prefix_tochar (prefix),
+    if(comp_with_mask (prefix_tochar (node->prefix, node->prefix->family),
+		       prefix_tochar (prefix, prefix->family),
 		       node->prefix->bitlen) && node->prefix->bitlen <= bitlen) {
 #ifdef PATRICIA_DEBUG
       fprintf (stderr, "patricia_search_best: found %s/%d\n",
